/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.Repository;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * A very simple Dialog with a message, a yes and no button
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class GeneralDialog extends JDialog {

    private JLabel           label;
    private JButton          buttonYes;
    private JButton          buttonNo;
    private JPanel           panel      = new JPanel(false);

    /** stores the result of the dialog (Boolean) */
    public static Object     result     = null;

    /**
     * Initializes and builds the dialog, then shows it. Modal is used.
     *
     * @param message   display String
     */
    public GeneralDialog(String message) {
        init();
        label.setText(message);

        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

        this.pack();
        this.setVisible(true);
        this.setModal(true);
        this.show();
    }

    /**
     * Builds the dialog
     */
    private void init() {
        this.setResizable(false);
        this.getContentPane().setLayout(new GridLayout());

        // Input panel
        GridBagLayout gridbInput        = new GridBagLayout();
        GridBagConstraints  cInput      = new GridBagConstraints();
        panel.setLayout(gridbInput);
        cInput.fill = GridBagConstraints.HORIZONTAL;

        label       = ComponentFactory.getLabel("");
        buttonYes   = ComponentFactory.getButton(Resources.getText(DCLabels.lblYes), 100);
        buttonNo    = ComponentFactory.getButton(Resources.getText(DCLabels.lblNo),  100);
        buttonYes.addActionListener(new YesButtonAction());
        buttonNo.addActionListener(new NoButtonAction());

        cInput.insets = new Insets(5,10,0,10);
        cInput.gridy = 0; cInput.gridx = 0;
        cInput.gridwidth = 8;
        gridbInput.setConstraints(label, cInput);
        cInput.gridwidth = 1;
        cInput.insets = new Insets(10,10,5,10);
        cInput.gridy = 1; cInput.gridx = 0;
        gridbInput.setConstraints(buttonYes, cInput);
        cInput.gridy = 1; cInput.gridx = 1;
        gridbInput.setConstraints(buttonNo, cInput);

        panel.add(label);
        panel.add(buttonNo);
        panel.add(buttonYes);
        
        panel.setBackground(Settings.getValueAsColor(
                            Repository.settings.stColor));

        this.setBounds(100,100,100,100);
        this.getContentPane().add(panel);
        this.setTitle(Resources.getText(DCMessages.msgQuestion));
        this.setBackground(Settings.getValueAsColor(
                           Repository.settings.stColor));
    }

    /**
     * Closes and disposes this dialog
     */
    public void close() {
        this.setVisible(false);
        this.setModal(false);
        this.dispose();
    }

    private class YesButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = new Boolean(true);
            close();
        }
    }

    private class NoButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = new Boolean(false);
            close();
        }
    }
}