/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.Repository;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * Confirmation dialog for importing a file. The IIF data is shown as well
 * as the location of the import file
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ImportListDialog extends JDialog {

    private JPanel              panelInput;
    private JPanel              panelActions;

    private JTextField          textSource;
    private JLabel              labelSource;
    private JLabel              labelIIF;
    private JEditorPane         textIIF;
    private JButton             buttonYes;
    private JButton             buttonNo;

    /** Stores the result of this dialog (Boolean) */
    public static Object        result          = null;

    /**
     * Builds and shows this dialog
     *
     * @param sourceFile    location of the sourcefile
     * @param IIFData       seperator data
     */
    public ImportListDialog(String sourceFile, String[][] IIFData) {
        buildPanel();
        textSource.setText(sourceFile);
        String  sIIF = "";

        if (IIFData != null) {
            for (int i = 0; i < IIFData.length; i++) {
                for (int j = 0; j < IIFData[i].length; j++) {
                    if (j == 0) {
                        sIIF    = sIIF + IIFData[i][j].trim() + "  " +
                                  Resources.getText(DCLabels.lblEndsWith) + "  ";
                    } else {
                        sIIF    = sIIF + IIFData[i][j].trim() + '\n';
                    }
                }
            }
        }
        textIIF.setText(sIIF);
        this.pack();
        this.setModal(true);
        this.setVisible(true);
    }

    /**
     * Creates this dialog
     */
    private void buildPanel() {
        panelInput = new JPanel(false);
        panelInput.setBackground(Settings.getValueAsColor(
                                 Repository.settings.stColor));
        panelActions = new JPanel(false);
        panelActions.setBackground(Settings.getValueAsColor(
                                   Repository.settings.stColor));
        this.setBackground(Settings.getValueAsColor(
                           Repository.settings.stColor));
                                   

        // ************************************************************************
        // INPUT PANEL
        // Set layout style
        GridBagLayout gridbInput        = new GridBagLayout();
        panelInput.setLayout(gridbInput);

        // Initialize components
        labelSource     = ComponentFactory.getLabel(
                          Resources.getText(DCLabels.lblSourceFile), 250);
        textSource      = ComponentFactory.getTextField(250);
        labelIIF        = ComponentFactory.getLabel(Resources.getText(DCLabels.lblIIFData), 250);
        textIIF         = new JEditorPane();

        textSource.setEditable(false);
        textIIF.setEditable(false);
        textSource.setBackground(ComponentFactory.getDisabledColor());
        textIIF.setBackground(ComponentFactory.getDisabledColor());

        textIIF.setPreferredSize(new Dimension(250,70));
        JScrollPane scrollIn = new JScrollPane(textIIF);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        scrollIn.setPreferredSize(new Dimension(250,70));

        // Build panel
        panelInput.add( labelSource,    new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets(0, 10, 0, 10), 0, 0));
        panelInput.add( textSource,     new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets(0, 10, 0, 10), 0, 0));
        panelInput.add( labelIIF,       new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets(0, 10, 0, 10), 0, 0));
        panelInput.add( scrollIn,       new GridBagConstraints( 0, 3, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets(0, 10, 0, 10), 0, 0));

        // ************************************************************************
        // ACTIONS PANEL
        // Set layout style
        GridBagLayout gridbActions        = new GridBagLayout();
        panelActions.setLayout(gridbActions);

        // Initialize components
        buttonYes       = ComponentFactory.getButton(
                          Resources.getText(DCLabels.lblYes), 100);
        buttonNo        = ComponentFactory.getButton(
                          Resources.getText(DCLabels.lblNo),  100);
        buttonNo.addActionListener(new NoButtonAction());
        buttonYes.addActionListener(new YesButtonAction());

        // Build panel
        panelActions.add( buttonYes,      new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets(5,10, 0, 5), 0, 0));
        panelActions.add( buttonNo,       new GridBagConstraints( 1, 0, 1, 1, 0.0, 0.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets(5, 0, 0, 10), 0, 0));

        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        this.getContentPane().setLayout(new GridBagLayout());

        // Initialize components
        this.setResizable(false);
        this.setBounds(100,100,100,100);
        this.setTitle(Resources.getText(DCLabels.lblVerification));

        // Build panel
        this.getContentPane().add( panelInput,    new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                  new Insets(5, 0, 5, 0), 0, 0));
        this.getContentPane().add( panelActions,  new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
                                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                  new Insets(10, 0,10, 0), 0, 0));
    }

    /**
     * Closes and disposes this dialog
     */
    public void close() {
        this.setVisible(false);
        this.setModal(false);
        this.dispose();
    }

    private class YesButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = new Boolean(true);
            close();
        }
    }

    private class NoButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = new Boolean(false);
            close();
        }
    }

}


