/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.ShortTextField;
import rjw.dc.core.Repository;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * Login shows a small login window. The user needs to fill the database name and
 * select the driver used to communicate with this database (ms, sun or other).
 * An extra driver can be defined in the ini file
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class LoginDialog extends JDialog {

    public static boolean done = false;

    private JLabel labelDriver;
    private JTextField textDriver;
    private JLabel labelDatabaseName;
    private ShortTextField textDatabaseName;

    private JButton buttonOK;
    private JButton buttonCancel;

    private JPanel panelInput = new JPanel(false);
    private JPanel panelActions = new JPanel(false);

    /**
     * Initializes and creates this dialog, then it is shown
     */
    public LoginDialog() {
        super(MessagesWriter.logForm);
        done = false;
        buildDialog();
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        this.setTitle(Resources.getText(DCLabels.lblConnection));
    
    	addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                	System.exit(0);
                }
    	});

        this.setSize(new Dimension(350,150));
        this.setModal(true);
    }

    /**
     * Closes and disposes this dialog
     */
    public void close() {
        this.setVisible(false);
        this.setModal(false);
        this.dispose();
    }

    /**
     * Verifies the selected database String and driver and saves them to the
     * Settings class
     */
    protected void verifyConnection(ActionEvent e) {
        String s = textDatabaseName.getText().trim();
        if (!s.equals("")) {
            Settings.setValue(Repository.settings.stConnectionString,
                              textDatabaseName.getText().trim());
        }

        done = true;
        close();
    }

    /**
     * Created this dialog
     */
    private void buildDialog() {
        Color background = ComponentFactory.getDisabledColor();
        this.getContentPane().setBackground(background);
        this.setBackground(background);
        
        // ************************************************************************
        // INPUT PANEL
        // Set layout style
        GridBagLayout gridbInput = new GridBagLayout();
        panelInput.setLayout(gridbInput);
        
        // Initialize Components
        labelDatabaseName = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDatabaseName));
        textDatabaseName = ComponentFactory.getShortTextField(20);
        labelDriver = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDatabaseDriver));
        textDriver = ComponentFactory.getTextField();
        
        textDatabaseName.setText(Settings.getValueAsString(Repository.settings.stConnectionString));
        textDriver.setText(Settings.getValueAsString(Repository.settings.stDatabaseDriver));

        // Build panel
        panelInput.add(labelDriver,         new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                            GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL, 
                                            new Insets(5, 5, 5, 5), 0, 0));
        panelInput.add(textDriver,          new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0,
                                            GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 5, 5), 0, 0));
        panelInput.add(labelDatabaseName,   new GridBagConstraints(0, 1, 2, 1, 1.0, 1.0,
                                            GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 5, 5), 0, 0));
        panelInput.add(textDatabaseName,    new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0,
                                            GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 5, 5), 0, 0));
        panelInput.setBackground(background);


        // ************************************************************************
        // ACTION PANEL
        // Set layout style
        panelActions.setLayout(new GridBagLayout());

        buttonOK = ComponentFactory.getButton(Resources.getText(DCLabels.lblOK));
        buttonCancel = ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));
        
        buttonOK.addActionListener(new YesButtonAction());
        buttonCancel.addActionListener(new CancelButtonAction());

        panelActions.add(buttonOK,      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST, GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel,  new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST, GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));
        panelActions.setBackground(background);

        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        this.setResizable(true);
        this.getContentPane().setLayout(new GridBagLayout());
        
        // Build panel
        this.getContentPane().add(panelInput,   new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                                GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                new Insets(5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelActions, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                                                GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0, 0));

        this.setTitle(Resources.getText(DCMessages.msgQuestion));
        
        this.setBounds(200, 200, 200, 200);
        this.pack();
        this.setSize(new Dimension(200,100));
    }


    private class YesButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            verifyConnection(e);
        }
    }

    private class CancelButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }
}
