/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JTabbedPane;

import rjw.apps.organizemp3.OrganizeMp3UI;
import rjw.dc.console.components.CatagoryForm;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.da.DataExchangePanel;
import rjw.dc.console.movie.InsertMoviePanel;
import rjw.dc.console.movie.SearchMoviePanel;
import rjw.dc.console.mp3.InsertMP3Panel;
import rjw.dc.console.mp3.SearchMP3Panel;
import rjw.dc.console.note.NotePanel;
import rjw.dc.console.software.InsertSoftwarePanel;
import rjw.dc.console.software.SearchSoftwarePanel;
import rjw.dc.console.sql.SQLPanel;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.software.CatagoryCollection;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;
import rjw.flex.settings.SettingsFile;
import rjw.flex.settings.ui.SettingsView;

/**
 * Holder of all the UI panels. The panels are shown as tabs.
 * By using this frame background colors can be set and requests can be past
 * to the panels by using the <url>RequestorCollection</url> class.
 * At the top of the frame a menubar is shown. From this menu the module can
 * be selected, the log form and record form can be opened and the MP3
 * organizer can be selected.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class MainFrame extends JFrame {
    private static JTabbedPane tabbedPane;
    
    private static JButton buttonCatagory;
    private static JButton buttonMP3Organizer;

    private static SearchSoftwarePanel      searchPanel;
    private static SQLPanel                 sqlPanel;
    private static NotePanel                notePanel;
    private static SearchMP3Panel           searchMP3Panel;
    private static InsertMP3Panel           insertMP3Panel;
    private static InsertMoviePanel         insertAviPanel;
    private static SearchMoviePanel         searchAviPanel;
    private static DataExchangePanel        daPanel;
    private static SettingsView             settingsView;  
    
    private static JComboBox                comboModule;
    private static JMenuBar                 menuBar;

    private static InsertSoftwarePanel      insertPanel;
    private static boolean                  firstTime       = true;

    /**
     * Initializes all the panels and puts them in a tabbed pane. The menubar
     * is build and its actions specified. The background color for all UI
     * components is set
     */
    public MainFrame() {
        if (firstTime == true) {
            GridBagConstraints c = new GridBagConstraints();
            GridBagLayout gridbag = new GridBagLayout();
            this.getContentPane().setLayout(gridbag);
            
            c.fill = GridBagConstraints.VERTICAL;
            tabbedPane = new JTabbedPane();

            settingsView = new SettingsView("Data Crow Settings Panel");  
            settingsView.setDisclaimer(
                "              __                       " + '\n' +
                "       <-----/@@\\----->               " + '\n' +
                "      <-< <  \\\\//  > >->             " + '\n' +
                "        <-<-\\ __ /->->                " + '\n' +
                "        Data /  \\ Crow                " + '\n' +
                "            ^    ^                     " + '\n' +
                "          0.25 Alpha " + '\n' +
                "Created By Robert-Jan van der Waals" + '\n' +
                "Original product: CD Library 3.03" + '\n' +
                "" + '\n' +
                "This product includes Hypersonic SQL."  + '\n' +
                "Originally developed by Thomas Mueller and the Hypersonic SQL Group"  + '\n' +
                ""  + '\n' +
                "This product used JFlexSett 0.1 alpha" + '\n' +
                "Created By Robert-Jan van der Waals" + '\n' +
                "" + '\n' +
                "Find me at http://sourceforge.net/projects/datacrow");
                
            settingsView.setSize(Settings.getValueAsDimension(Repository.settings.stSettingsViewSize));
            ImageIcon image = new ImageIcon("./Images/Settings.gif");
            settingsView.setIconImage(image.getImage());

            searchPanel     = new SearchSoftwarePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitSearch), Messages._STANDARD);
            insertPanel     = new InsertSoftwarePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitInsert), Messages._STANDARD);
            daPanel         = new DataExchangePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitDataExchange), Messages._STANDARD);
            sqlPanel        = new SQLPanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitSQL), Messages._STANDARD);
            searchMP3Panel  = new SearchMP3Panel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitMP3Search), Messages._STANDARD);
            insertMP3Panel  = new InsertMP3Panel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitMP3Insert), Messages._STANDARD);
            notePanel       = new NotePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitNote), Messages._STANDARD);
            insertAviPanel  = new InsertMoviePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitMovieInsert), Messages._STANDARD);
            searchAviPanel  = new SearchMoviePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitMovieSearch), Messages._STANDARD);

            try {
                setTitle(Resources.getText(DCLabels.lblMainTitle));
                setResizable(false);
                addWindowListener(new WindowAdapter() {
                    public void windowClosing(WindowEvent e) {
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgApplicationStops), 
                        Messages._STANDARD);

                    Settings.setValue(Repository.settings.stMainViewSize, getSize());
                    SettingsFile.save();
                    
                    System.exit(0);
                    }
                });

                image = new ImageIcon("./Images/mainIcon.gif");
                this.setIconImage(image.getImage());

                menuBar = createMenuBar();

                int iDefModule = Settings.getValueAsInt(Repository.settings.stModule);
                if (iDefModule > -1) {
                    comboModule.setSelectedIndex(iDefModule);
                }
                setModule(false);

                searchPanel.setBorder(BorderFactory.createRaisedBevelBorder());
                insertPanel.setBorder(BorderFactory.createRaisedBevelBorder());
                sqlPanel.setBorder(BorderFactory.createRaisedBevelBorder());
                searchMP3Panel.setBorder(BorderFactory.createRaisedBevelBorder());
                insertMP3Panel.setBorder(BorderFactory.createRaisedBevelBorder());
                notePanel.setBorder(BorderFactory.createRaisedBevelBorder());
                insertAviPanel.setBorder(BorderFactory.createRaisedBevelBorder());
                searchAviPanel.setBorder(BorderFactory.createRaisedBevelBorder());
                daPanel.setBorder(BorderFactory.createRaisedBevelBorder());

                getContentPane().add(menuBar, new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                     new Insets(0, 0, 0, 0), 0, 0));
                getContentPane().add(tabbedPane, new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                     new Insets(0, 0, 0, 0), 0, 0));

                this.setBounds(50,50,50,50);

                pack();
            } catch (Error err) {
                Messages.setMessage(err.getMessage(), Messages._STANDARD);
            }
            final Color bgColor = Settings.getValueAsColor(Repository.settings.stColor);
            this.applyBackgroundColor(bgColor);
            this.setResizable(true);
            this.setSize(Settings.getValueAsDimension(Repository.settings.stMainViewSize));
            firstTime = false;
        }
    }

    /**
     * Creates the menu
     */
    private JMenuBar createMenuBar() {
        JButton buttonLogForm = ComponentFactory.getButton(
                                Resources.getText(DCLabels.lblOpenLog));
        JButton buttonRecord =  ComponentFactory.getButton(
                                Resources.getText(DCLabels.lblOpenRecord));
        buttonMP3Organizer =ComponentFactory.getButton(
                                    Resources.getText(DCLabels.lblOpenMP3Organizer));
        buttonCatagory = ComponentFactory.getButton(
                                 Resources.getText(DCLabels.lblCatagoryForm));
        JButton buttonSettings = ComponentFactory.getButton(
                                 Resources.getText(DCLabels.lblSettings));
                                
        
        comboModule = ComponentFactory.getComboBox();
        comboModule.setForeground(new Color(0, 0, 0));

        comboModule.addItem(Resources.getText(DCLabels.lblModuleSoftware));
        comboModule.addItem(Resources.getText(DCLabels.lblModuleMP3));
        comboModule.addItem(Resources.getText(DCLabels.lblModuleMovie));
        comboModule.addActionListener(new ModuleAction());

        buttonLogForm.addActionListener(new LogFormMenuAction());
        buttonRecord.addActionListener(new RecordFormMenuAction());
        buttonMP3Organizer.addActionListener(new MP3OrganizerFormMenuAction());
        buttonSettings.addActionListener(new SettingsFormMenuAction());
        buttonCatagory.addActionListener(new CatagoryFormMenuAction());

        JMenuBar menuBar = ComponentFactory.getMenuBar();

        menuBar.add(buttonRecord);
        menuBar.add(buttonLogForm);                                  
        menuBar.add(buttonMP3Organizer);                                  
        menuBar.add(buttonCatagory);                                  
        menuBar.add(buttonSettings);         
        menuBar.add(comboModule);         

        return menuBar;
    }

    /**
     * Applies the background color to all panels and their components
     *
     * @color   background color
     */
    public void applyBackgroundColor(Color color) {
        try {
            searchPanel.setBackground(color);
            insertPanel.setBackground(color);
            sqlPanel.setBackground(color);
            searchMP3Panel.setBackground(color);
            insertMP3Panel.setBackground(color);
            notePanel.setBackground(color);
            insertAviPanel.setBackground(color);
            searchAviPanel.setBackground(color);
            daPanel.setBackgrounds(color);
            settingsView.setBackgroundColor(color);

            setCompBackColors(searchPanel.getComponents(), color);
            setCompBackColors(insertPanel.getComponents(), color);
            setCompBackColors(sqlPanel.getComponents(), color);
            setCompBackColors(insertMP3Panel.getComponents(), color);
            setCompBackColors(searchMP3Panel.getComponents(), color);
            setCompBackColors(this.getComponents(), color);
            setCompBackColors(notePanel.getComponents(), color);
            setCompBackColors(searchAviPanel.getComponents(), color);
            setCompBackColors(insertAviPanel.getComponents(), color);
            setCompBackColors(daPanel.getComponents(), color);
            setCompBackColors(settingsView.getComponents(), color);
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Applied the background color to all components
     *
     * @param comps     the UI components
     * @param color     background color
     */
    private void setCompBackColors(Component[] comps, Color color) {
        try {
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
            }
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Sets the module. Only the tabs for this module are shown. The module
     * is selected from the module combobox from the menubar
     */
    private void setModule(boolean checkSame) {
        int iModule = Settings.getValueAsInt(Repository.settings.stModule);

        if (checkSame && iModule == comboModule.getSelectedIndex()) {
            return;
        }

        if (comboModule.getSelectedItem().toString().trim()
            .equals(Resources.getText(DCLabels.lblModuleSoftware))) {

            tabbedPane.removeAll();
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSearch), searchPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabInsert), insertPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabDataExchange), daPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSQL), sqlPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabNote), notePanel);
            buttonCatagory.setVisible(true);
            buttonCatagory.setText(Resources.getText(DCLabels.lblType));
            buttonMP3Organizer.setVisible(false);
        } else if (comboModule.getSelectedItem().toString().trim()
                    .equals(Resources.getText(DCLabels.lblModuleMP3))) {

            tabbedPane.removeAll();
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSearch), searchMP3Panel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabInsert), insertMP3Panel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabDataExchange), daPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSQL), sqlPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabNote), notePanel);
            buttonCatagory.setText(Resources.getText(DCLabels.lblMP3Genres));
            buttonCatagory.setVisible(true);
            buttonMP3Organizer.setVisible(true);
        } else if (comboModule.getSelectedItem().toString().trim()
                    .equals(Resources.getText(DCLabels.lblModuleMovie))) {
            tabbedPane.removeAll();
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSearch), searchAviPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabInsert), insertAviPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSQL), sqlPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabNote), notePanel);
            buttonCatagory.setVisible(false);
            buttonMP3Organizer.setVisible(false);            
        }

        Settings.setValue(Repository.settings.stModule, new Integer(comboModule.getSelectedIndex()));
        daPanel.switchModule();
    }

    /**
     * Display requests and actions are passed to the different panels
     *
     * @param requestors    a collection of requests for specified panels
     * @param vData         data to be displayed (mainly used for tables)
     */
    public void displayResults(RequestorCollection requestors, Vector vData) {
        Requestor[] reqObjects = requestors.toRequestorArray();
        Requestor requestor = null;

        for (int i = 0; i < reqObjects.length; i++) {
            requestor = reqObjects[i];

            Class clazz = requestor.getClazz();
            String sMethod = requestor.getMethod();
            Object[] parameters = requestor.getParameters();
            
            if (clazz.getName().equals("rjw.dc.console.software.SearchSoftwarePanel")) {
                if (sMethod.equals("fillTable")) {
                    searchPanel.fillTable(vData, false);
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI), 
                        Messages._STANDARD);
                }

                if (sMethod.equals("searchValues")) {
                    searchPanel.searchValues();
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI), 
                        Messages._STANDARD);
                }
                
                if (sMethod.equals("updateRow")) {
                    searchPanel.updateRow(parameters[0].toString(),
                                          (Object[]) parameters[1]);
                }
            }

            if (clazz.getName().equals("rjw.dc.console.RecordForm")) {
                int iModule = Settings.getValueAsInt(Repository.settings.stModule);

                if (sMethod.equals("close")) {
                    if (iModule == 0) {
                        if (parameters != null && parameters[0] != null) {
                            if (parameters[0].equals("SearchPanel")) {
                                searchPanel.closeRecordForm(parameters[1].toString());
                            } else if (parameters[0].equals("InsertPanel")) {
                                insertPanel.closeRecordForm(parameters[1].toString());
                            }
                        }
                    } else if (iModule == 1) {
                        if (parameters != null && parameters[0] != null) {
                            if (parameters[0].equals("SearchMP3Panel")) {
                                searchMP3Panel.closeRecordForm(parameters[1].toString());
                            } else if (parameters[0].equals("InsertMP3Panel")) {
                                insertMP3Panel.closeRecordForm(parameters[1].toString());
                            }
                        }
                    } else if (iModule == 2) {
                        if (parameters != null && parameters[0] != null) {
                            if (parameters[0].equals("SearchMoviePanel")) {
                                searchAviPanel.closeRecordForm(parameters[1].toString());
                            } else if (parameters[0].equals("InsertMoviePanel")) {
                                insertAviPanel.closeRecordForm(parameters[1].toString());
                            }
                        }
                    }
                }
            }

            if (clazz.getName().equals("rjw.dc.core.objects.software.Catagory")) {
                if (sMethod.equals("setCatagories")) {
                    CatagoryCollection.setCatagories(vData);
                }
            }

            if (clazz.getName().equals("rjw.dc.console.mp3.InsertMP3Panel")) {
                if (sMethod.equals("setStatus")) {
                    insertMP3Panel.setStatus(Resources.getText(DCMessages.msgDataSaved));
                }

                if (sMethod.equals("addRow")) {
                    insertMP3Panel.addRow(vData);
                }
            }

            if (clazz.getName().equals("rjw.dc.console.mp3.SearchMP3Panel")) {
                if (sMethod.equals("fillTable")) {
                    searchMP3Panel.fillTable(vData, false);
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI), 
                        Messages._STANDARD);
                }

                if (sMethod.equals("searchValues")) {
                    searchMP3Panel.searchValues();
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI), 
                        Messages._STANDARD);
                }
                
                if (sMethod.equals("updateRow")) {
                    searchMP3Panel.updateRow(parameters[0].toString(),
                                          (Object[]) parameters[1]);
                }                
            }

            if (clazz.getName().equals("rjw.dc.console.software.InsertSoftwarePanel")) {
                if (sMethod.equals("setStatus")) {
                    insertPanel.setStatus(Resources.getText(DCMessages.msgDataSaved));
                }

                if (sMethod.equals("addRow")) {
                    insertPanel.addRow(vData);
                }
            }

            if (clazz.getName().equals("rjw.dc.console.movie.InsertMoviePanel")) {
                if (sMethod.equals("setStatus")) {
                    insertAviPanel.setStatus(Resources.getText(DCMessages.msgDataSaved));
                }
                
                if (sMethod.equals("updateRowAt")) {
                    insertAviPanel.updateRowAt(((Integer)  parameters[0]).intValue(),
                                               (Object[]) parameters[1],
                                               ((Boolean)  parameters[2]).booleanValue());
                }
            }

            if (clazz.getName().equals("rjw.dc.console.movie.SearchMoviePanel")) {
                if (sMethod.equals("searchValues")) {
                    searchAviPanel.searchValues();
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI), 
                        Messages._STANDARD);
                }

                if (sMethod.equals("fillTable")) {
                    searchAviPanel.fillTable(vData, false);
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI), 
                        Messages._STANDARD);
                }
                
                if (sMethod.equals("updateRow")) {
                    searchAviPanel.updateRow(parameters[0].toString(),
                                          (Object[]) parameters[1]);
                }   
                if (sMethod.equals("updateRowAt")) {
                    searchAviPanel.updateRowAt(((Integer)  parameters[0]).intValue(),
                                               (Object[]) parameters[1],
                                               ((Boolean)  parameters[2]).booleanValue());
                }
            }

            if (clazz.getName().equals("rjw.dc.console.da.DataExchangeSourceEditor")) {
                if (sMethod.equals("setSourceFile")) {
                    daPanel.daSourceEditor.setSourceFile();
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI), 
                        Messages._STANDARD);
                }

            }
        }
    }

    private class RecordFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int selectIndex = tabbedPane.getSelectedIndex();
            int iModule = Settings.getValueAsInt(Repository.settings.stModule);

            switch (selectIndex) {
                case 0:
                    if (iModule == 0) {
                        searchPanel.createRecordForm();
                    } else if (iModule == 1) {
                        searchMP3Panel.createRecordForm();
                    } else {
                        searchAviPanel.createRecordForm();
                    }
                    break;
                case 1:
                    if (iModule == 0) {
                        insertPanel.createRecordForm();
                    } else if (iModule == 1) {
                        insertMP3Panel.createRecordForm();
                    } else {
                        insertAviPanel.createRecordForm();
                    }
                    break;
            }
        }
    }

    private class MP3OrganizerFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            OrganizeMp3UI organizeMP3UI = new OrganizeMp3UI();
            organizeMP3UI.setVisible(true);
        }
    }

    private class SettingsFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            settingsView.setVisible(true);
            
        }
    }

    private class CatagoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            CatagoryForm catagoryForm = new CatagoryForm();
            catagoryForm.setModule(comboModule.getSelectedIndex());
            catagoryForm.setVisible(true);
            catagoryForm.setTitle(buttonCatagory.getText());
        }
    }

    private class LogFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            MessagesWriter._windowShow = true;
            MessagesWriter._windowToFront = true;
        }
    }

   private class ModuleAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            setModule(true);
        }
    }
}