/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.Repository;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * A normal MessageBox
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class MessageBox extends JDialog {

    private static boolean isVisible = true;
    private JTextArea textMessage;
    private JButton buttonClose;
    private JPanel panel = new JPanel(false);

    /** Error message */
    public  static int _ERROR          = 1;
    /** Notification message */
    public  static int _NOTIFICATION   = 2;
    /** Informative message */
    public  static int _INFORMATION    = 3;

    /**
     * Initializes the messagebox and shows it
     *
     * @param message   display String
     * @param button    indicates if an "OK" should be shown
     * @param type      type of the message (see static variables)
     * @param modal     indicates if the messagebox should be shown modal
     */
    public MessageBox(String message, boolean button, int type, boolean modal) {
        init(button);
        setModal(modal);
        display(message, type);
    }

    /**
     * Initializes the messagebox and shows it
     *
     * @param frame     super frame
     * @param message   display String
     * @param button    indicates if an "OK" should be shown
     * @param type      type of the message (see static variables)
     */
    public MessageBox(JFrame frame, String message, boolean button, int type) {
        super(frame);
        init(button);
        setModal(true);
        display(message, type);
    }
    
    public MessageBox(String message, boolean button, int type) {
        init(button);
        setModal(true);
        display(message, type);
    }    

    /**
     * Shows the messagebox
     *
     * @param message   display String
     * @param type      type of the message (see static variables)
     */
    private void display(String message, int type) {
        isVisible = true;
        textMessage.setText(message);
        this.setTitle(Resources.getText(DCMessages.msgSysInfo));

        if (type == _ERROR) {
            this.setTitle(Resources.getText(DCMessages.msgSysError));
        } else if (type == _NOTIFICATION) {
            this.setTitle(Resources.getText(DCMessages.msgSysNotif));
        } else if (type == _INFORMATION) {
            this.setTitle(Resources.getText(DCMessages.msgSysInfo));
        }

        this.setBounds(100,100,100,100);
        this.pack();
        this.toFront();
        this.setVisible(true);
    }

    /**
     * Creates the messagebox
     *
     * @param button    indicates if an "OK" button should be created
     */
    private void init(boolean button) {
        this.setResizable(false);
        this.getContentPane().setLayout(new GridBagLayout());

        this.setBounds(100,100,100,100);
        textMessage = ComponentFactory.getTextArea();
        textMessage.setBackground(ComponentFactory.getDisabledColor());
        textMessage.setEditable(false);

        textMessage.setPreferredSize(new Dimension(350,70));
        textMessage.setBackground(ComponentFactory.getDisabledColor());
        JScrollPane scrollIn = new JScrollPane(textMessage);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        scrollIn.setPreferredSize(new Dimension(350,70));

        buttonClose = ComponentFactory.getButton(
                      Resources.getText(DCLabels.lblOK), 100);
        buttonClose.addActionListener(new MessageButtonAction());

        panel.add(scrollIn);
        panel.setBackground(Settings.getValueAsColor(
                            Repository.settings.stColor));
                            
        if (button == true) {
            panel.add(buttonClose);
        }

        this.getContentPane().add(panel);
        this.getContentPane().setBackground(Settings.getValueAsColor(
                                            Repository.settings.stColor));
    }

    /**
     * Closes and disposes this messagebox
     */
    public void close() {
        isVisible = false;
        this.setVisible(false);
        this.setModal(false);
        this.dispose();
    }

    /**
     * Returns if the messagebox is visible
     *
     * @return (boolean)    visible indicator
     */
    public static boolean getVisible() {
            return isVisible;
    }

    private class MessageButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
}
