/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.LongTextField;
import rjw.dc.console.components.NumberField;
import rjw.dc.console.components.ShortTextField;
import rjw.dc.console.movie.SearchMoviePanel;
import rjw.dc.console.mp3.SearchMP3Panel;
import rjw.dc.console.software.SearchSoftwarePanel;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.core.objects.mp3.Mp3;
import rjw.dc.core.objects.software.Software;
import rjw.dc.messages.Messages;
import rjw.flex.settings.Settings;

/**
 * The most flexible form available in Data Crow, everything is configured from
 * the parameters passed to the constructor.
 * Data shown in this dialog can be saved or updated.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class RecordForm extends JFrame {

    public static final int _MP3 = 0;
    public static final int _SOFTWARE = 1;
    public static final int _MOVIE = 2;
    
    public  Object recordID = null;

    /** Created components. All Components are passed into a Vector */
    private Vector vectComps  = new Vector();
    private Object[][] currentFields;
    private JLabel[] labels;
    private JButton buttonClose;
    private JButton buttonSave;
    private JComponent[] inputFields;

    private JPanel panelInput = new JPanel(false);
    private JPanel panelAction = new JPanel(false);

    /** The ID of the record if it concerns an update */
    private String sID = "";
    private String sFormID = "";
    private static int iFormID = 0;
    
    private final Color color = Settings.getValueAsColor(
                                         Repository.settings.stColor);

    /**
     * Initializes and creates thid form
     *
     * @param fields    the field relative to the database;
     *                  fields[][0] field name (name for the label)
     *                  fields[][1] field type
     *                  fields[][2] field value
     *                  fields[][3] maximum text length
     * @param layout    the layout parameter (next to each other or one above the other
     * @param save      indicates whether to add a savebutton
     * @param title     title of the form
     * @param ID        ID of the current row if this is an update. If it is an insert form,
     *                  pass null!
     */
    public RecordForm(Object[][] fields, boolean save, String title, String ID) {
        this.setIconImage(new ImageIcon("./images/file.gif").getImage());
        
        this.setTitle(title);
        sID             = ID;
        int iFields     = fields.length;
        labels          = new JLabel[iFields];
        inputFields     = new JComponent[iFields];
        currentFields   = fields;

        this.getContentPane().setBackground(color);
        this.setBackground(color);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stRecordFormViewSize, 
                                  getSize());
            }
        });

        // Main panel
        GridBagConstraints  c       = new GridBagConstraints();
        GridBagLayout       gridbag = new GridBagLayout();
        this.getContentPane().setLayout(gridbag);
        c.fill = GridBagConstraints.VERTICAL;

        // Input panel
        GridBagLayout gridbInput        = new GridBagLayout();
        GridBagConstraints  cInput      = new GridBagConstraints();
        panelInput.setLayout(gridbInput);
        cInput.fill = GridBagConstraints.HORIZONTAL;

        // Action Panel
        GridBagLayout gridbAction       = new GridBagLayout();
        GridBagConstraints  cAction     = new GridBagConstraints();
        panelAction.setLayout(gridbAction);
        cAction.fill = GridBagConstraints.HORIZONTAL;

        for (int i = 0; i < iFields; i ++) {
            int maxTextLength = ((Integer) fields[i][3]).intValue();
            labels[i]           = ComponentFactory.getLabel(currentFields[i][0].toString());
            Integer fieldType   = (Integer) currentFields[i][1];
            
            switch (fieldType.intValue()) {
                case (ComponentFactory._NUMBERFIELD) :
                    NumberField numberField = ComponentFactory.getNumberField();
                    inputFields[i] = numberField;
                    numberField.setText(currentFields[i][2].toString());
                    numberField.setBackground(ComponentFactory.getTableColor());
                    vectComps.addElement(numberField);     
                    break;               
                case (ComponentFactory._LONGTEXTFIELD):
                    LongTextField longText = ComponentFactory.getLongTextField(maxTextLength);
                    longText.setMargin(new Insets(0, 0, 0, 0));
                    longText.setText(currentFields[i][2].toString());
                    JScrollPane pane = new JScrollPane(longText);
                    pane.setHorizontalScrollBarPolicy(
                               ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
                    pane.setVerticalScrollBarPolicy(
                               ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
                    inputFields[i] = pane;
                    vectComps.addElement(longText);
                    break;
                case (ComponentFactory._SOFTWARETYPECOMBO):
                    JComboBox comboType= ComponentFactory.getSoftwareCatagoriesComboBox();
                    comboType.setSelectedItem(currentFields[i][2].toString());
                    inputFields[i] = comboType;
                    vectComps.addElement(comboType);
                    break;
                case (ComponentFactory._YESNOCOMBO):
                    JComboBox comboYes= ComponentFactory.getYesNoCombo();
                    comboYes.setSelectedItem(currentFields[i][2].toString());
                    inputFields[i] = comboYes;
                    vectComps.addElement(comboYes);
                    break;
                case (ComponentFactory._MP3GENRECOMBO):
                    JComboBox comboGenre= ComponentFactory.getMP3GenreComboBox();
                    comboGenre.setSelectedItem(currentFields[i][2].toString());
                    inputFields[i] = comboGenre;
                    vectComps.addElement(comboGenre);
                    break;
                case (ComponentFactory._YEARCOMBO):
                    JComboBox comboYear= ComponentFactory.getYearComboBox();
                    comboYear.setSelectedItem(currentFields[i][2].toString());
                    inputFields[i]= comboYear;
                    vectComps.addElement(comboYear);
                    break;
                default:
                    ShortTextField textField = ComponentFactory.getShortTextField(maxTextLength);
                    if (i == 0) {
                        textField.setBackground(ComponentFactory.getDisabledColor());
                        textField.setEditable(false);
                    } else {
                        textField.setBackground(ComponentFactory.getTableColor());
                    }
                    inputFields[i]  = textField;
                    textField.setText(currentFields[i][2].toString());
                    vectComps.addElement(textField);
            }
        }

        // fill the input panel with the generated fields
        int y = 0;
        for (int j = 0 ; j < iFields; j++) {
            panelInput.setBackground(color);
            labels[j].setBackground(color);

            panelInput.add(labels[j], new GridBagConstraints(0, y, 1, 1, 1.0, 1.0
                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                           new Insets(5, 5, 5, 5), 0, 0));

            int stretch = GridBagConstraints.HORIZONTAL;
            int factor = 1;
            if (inputFields[j] instanceof JScrollPane) {
                stretch = GridBagConstraints.BOTH; 
                factor  = 20;
            }
            
            panelInput.add(inputFields[j], new GridBagConstraints(1, y, 1, 1, factor, factor
                          ,GridBagConstraints.NORTHWEST, stretch,
                           new Insets(5, 5, 5, 5), 0, 0));
            y++;
        }

        // Fill the actions panel
        buttonClose     = ComponentFactory.getButton("Close");
        buttonSave      = ComponentFactory.getButton("Save");


        panelAction.setBackground(color);
        panelAction.add(buttonClose, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                        new Insets(5, 5, 5, 5), 0, 0));


        buttonClose.addActionListener(new ExitAction());
        buttonSave.addActionListener(new SaveAction());

        if (save == true) {
            panelAction.add(buttonSave, new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                            new Insets(5, 5, 5, 5), 0, 0));
        }

        this.getContentPane().add(panelInput, new GridBagConstraints(0, 0, 1, 1, 40.0, 40.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                       new Insets(5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelAction, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                      ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                       new Insets(5, 5, 5, 5), 0, 0));

        this.pack();
        this.setResizable(true);
        this.setSize(Settings.getValueAsDimension(Repository.settings.stRecordFormViewSize));
        this.setVisible(true);
    }

    /**
     * Returns the data from screen
     *
     * @return (String[])   the data as array
     */
    private Object[] getData() {
        Object[] values = new Object[vectComps.size()];
        for (int i = 0; i < vectComps.size(); i++) {
            Integer fieldType   = (Integer) currentFields[i][1];
            switch (fieldType.intValue()) {
                case (ComponentFactory._LONGTEXTFIELD):
                    LongTextField compje0 = (LongTextField) vectComps.elementAt(i);
                    values[i] = getValue(compje0.getText());
                    vectComps.setElementAt(compje0, i);
                    break;
                case (ComponentFactory._SOFTWARETYPECOMBO):
                case (ComponentFactory._YESNOCOMBO):
                case (ComponentFactory._MP3GENRECOMBO):
                case (ComponentFactory._MOVIECODECCOMBO):
                case (ComponentFactory._YEARCOMBO):
                    JComboBox   compje3 = (JComboBox) vectComps.elementAt(i);
                    values[i]   = getValue(compje3.getSelectedItem().toString());
                    vectComps.setElementAt(compje3, i);
                    break;
                case (ComponentFactory._NUMBERFIELD):
                    NumberField compje4 = (NumberField) vectComps.elementAt(i);
                    Integer integer = (Integer) compje4.getValue();
                    values[i] = integer;
                    break;
                default:
                    ShortTextField compje5 = (ShortTextField) vectComps.elementAt(i);
                    values[i]   = getValue(compje5.getText());
                    vectComps.setElementAt(compje5, i);
                    break;
            }
        }

        if (values[0].equals("Null")) {
            values[0] = "";
        }

        return values;
    }

    /**
     * Transforms the value to match the database needs
     *
     * @param sValue    value as String
     */
    private String getValue(String value) {
        String sValue = value;
        if (sValue == null) {
            return "Null";
        } else if (sValue.trim().equals("")) {
            return "Null";
        } else {
            return sValue.trim();
        }
    }

    /**
     * Sets the form ID. The form ID equals the record ID
     *
     * @param sFormID   the record ID, used as the form ID
     */
    public void setFormID(String sFormID) {
        this.sFormID = sFormID;
    }

    /**
     * Returns the form ID
     *
     * return (String)  the form ID, which equals the record ID
     */
    public String getFormID() {
        return sFormID;
    }

    /**
     * Saves the values to the database. Handles both MP3 and Software
     * objects
     */
    protected void saveValues(ActionEvent e) {
        int iModule = Settings.getValueAsInt(Repository.settings.stModule);
        if (iModule == 0) {
            this.setFormID(String.valueOf(iFormID++));
            Software software = new Software();
            software.setValues(getData());

            if (sID == null)  {
                String[] param = {"InsertPanel", this.getFormID()};
                Requestor requestorClose = new Requestor(RecordForm.class, "close", param);
                RequestorCollection requestors = new RequestorCollection();
                requestors.addRequestor(requestorClose);

                software.saveNew(requestors);
            } else {
                String[] param = {"SearchPanel", this.getFormID()};
                Requestor requestorClose = new Requestor(RecordForm.class, "close", param);
                Requestor requestor = new Requestor(SearchSoftwarePanel.class, 
                                                    "updateRow", 
                                                    new Object[] {sID,
                                                                  getData()});
                RequestorCollection requestors = new RequestorCollection();
                requestors.addRequestor(requestor);
                requestors.addRequestor(requestorClose);

                software.saveUpdate(requestors);
            }
        } else if (iModule == 1) {
            this.setFormID(String.valueOf(iFormID++));
            Mp3 mp3 = new Mp3();
            mp3.setValues(getData());

            if (sID == null)  {
                String[] param = {"InsertMP3Panel", this.getFormID()};
                Requestor requestorClose = new Requestor(RecordForm.class, "close", param);
                RequestorCollection requestors = new RequestorCollection();
                requestors.addRequestor(requestorClose);

                mp3.saveNew(requestors);
            } else {
                String[] param = {"SearchMP3Panel", this.getFormID()};
                Requestor requestorClose = new Requestor(RecordForm.class, "close", param);
                Requestor requestor = new Requestor(SearchMP3Panel.class, 
                                                    "updateRow", 
                                                    new Object[] {sID,
                                                                  getData()});
                RequestorCollection requestors = new RequestorCollection();
                requestors.addRequestor(requestor);
                requestors.addRequestor(requestorClose);

                mp3.saveUpdate(requestors);
            }
        } else if (iModule == 2) {
            this.setFormID(String.valueOf(iFormID++));
            Movie movie = new Movie();
            movie.setValues(getData());

            if (sID == null)  {
                String[] param = {"InsertMoviePanel", this.getFormID()};
                Requestor requestorClose = new Requestor(RecordForm.class, "close", param);
                RequestorCollection requestors = new RequestorCollection();
                requestors.addRequestor(requestorClose);

                movie.saveNew(requestors);
            } else {
                String[] param = {"SearchMoviePanel", this.getFormID()};
                Requestor requestorClose = new Requestor(RecordForm.class, "close", param);
                Requestor requestor = new Requestor(SearchMoviePanel.class, 
                                                    "updateRow", 
                                                    new Object[] {sID,
                                                                  getData()});
                RequestorCollection requestors = new RequestorCollection();
                requestors.addRequestor(requestor);
                requestors.addRequestor(requestorClose);

                movie.saveUpdate(requestors);
            }
        }
    }

    /**
     * Closes this form by setting the visibility to false
     */
    public void close() {
        Settings.setValue(Repository.settings.stRecordFormViewSize, 
                          getSize());        
        setVisible(false);
    }

    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveValues(e);
        }
    }

    private class ExitAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }

    /**
     * Applies the background color
     *
     * @param color background color
     */
    public void setBackgroundColor(Color color) {
        this.setBackground(color);

        try {
            setCompBackColors(this.getComponents(), color);
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Applies the background color to all the components
     *
     * @param comps     the components on which the color is applied
     * @param color     background color
     */
    private void setCompBackColors(Component[] comps, Color color) {
        try {
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
            }
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }
}
