package rjw.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.colorchooser.AbstractColorChooserPanel;
import javax.swing.colorchooser.ColorSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import rjw.dc.core.DataCrow;
import rjw.flex.settings.ui.ISettingsComponent;

/**
 * Simple HSB color field derived from the JColorChooser field.
 * Applies the color to the application everytime a new one has 
 * been selected 
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class BackgroundColorField extends JComponent implements ISettingsComponent {
    
    private ColorSelectionModel colorModel;
    private Color color;

    public BackgroundColorField() {
        buildComponent();
    }
    
    public Object getValue() {
        return color;   
    }

    public void setValue(Object o) {
        color = (Color) o;
        colorModel.setSelectedColor(color);
    }
    
    public void buildComponent() {
        this.setLayout(new GridBagLayout());
        
        // get the components from the JColorChooser
        AbstractColorChooserPanel panelHSB = (AbstractColorChooserPanel) new JColorChooser().getChooserPanels()[1];
        JPanel panel = (JPanel) panelHSB.getComponent(0);
        colorModel = panelHSB.getColorSelectionModel();

        Component[] comps2 = panel.getComponents();
        
        // add them to this components
        int x = 0;
        for (int j = 0; j < comps2.length; j++) {
            add(comps2[j], new GridBagConstraints(x++, 0, 1, 1, 1.0, 1.0,
                                                  GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                  new Insets(0,0,0,0), 0, 0));
            this.add(comps2[j]);
        }
        
        color = colorModel.getSelectedColor();
        colorModel.addChangeListener(new StateChanged());
    }
    
    private class StateChanged implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            Color temp =colorModel.getSelectedColor();

            if (!color.equals(temp)) {
                color = temp;
                DataCrow.mainFrame.applyBackgroundColor(color);
            }
        }
    }
}
