package rjw.dc.console.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.software.Catagory;
import rjw.dc.core.objects.software.CatagoryCollection;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * Form which shows the catagories of Mp3's, Movies and Software
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class CatagoryForm extends JFrame {

    CatagoryPanel catPanel;
    
    public CatagoryForm() {
        this.setIconImage(new ImageIcon("./images/mainIcon.gif").getImage());
        this.setTitle(Resources.getText(DCLabels.lblCatagories));
        this.getContentPane().setBackground(Settings.getValueAsColor(
                                            Repository.settings.stColor));
        catPanel = new CatagoryPanel();
        getContentPane().setLayout(new GridBagLayout());

        getContentPane().add(catPanel,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        setResizable(true);
        pack();
    }
    
    /**
     * Changes the form to reflect the selected module
     * 
     * @param module the currently active module
     */
    public void setModule(int module) {
        catPanel.setModule(module);
    }

    private class CatagoryPanel extends TablePanel {

        JPanel panelCatagory = new JPanel(false);
        JPanel panelTabAct = new JPanel(false);
        JPanel panelMP3Genres = new JPanel(false);
        JTextArea textMP3;

        public CatagoryPanel() {
            buildPanel();   
        }

        /**
         * Saves the changed Software categories to the database
         */
        protected void saveCategories() {
            Object[] rowData = new Object[table.getColumnCount()];
            boolean bNew = false;
            
            for (Enumeration enum = vChangedRowNumbers.elements(); enum.hasMoreElements(); ) {
                int row = ((Integer) enum.nextElement()).intValue();

                Object id = table.getValueAt(row, 0);
                if (id == null || id.toString().equals("")) {
                    bNew = true;   
                }
                rowData[0] = id;
                
                for (int j = 1; j < table.getColumnCount(); j++) {
                    Object oTest = table.getValueAt(row, j);
                    if (!oTest.toString().equals("")) {
                        rowData[j] = oTest.toString();
                    } else {
                        rowData[j] = "Null";
                    }
                }
                
                Catagory catagory = new Catagory();
                catagory.setValues(rowData);
    
                if (bNew) {            
                    catagory.saveNew(new RequestorCollection());
                } else {
                    catagory.saveUpdate(new RequestorCollection());
                }
            }
            CatagoryCollection.refresh();
            vChangedIDs.removeAllElements();
        }

        /**
         * Builds the panel
         */
        private void buildPanel() {
            Color color = Settings.getValueAsColor(Repository.settings.stColor);
            this.setBackground(color);
            
            panelCatagory.setBackground(color);
            panelTabAct.setBackground(color);
            panelMP3Genres.setBackground(color);
            
            // ************************************************************************
            // CATEGORY PANEL
            // Set layout style
            GridBagLayout gridbCategory = new GridBagLayout();
            panelCatagory.setLayout(gridbCategory);
    
            // Initialize components
            tableModel.setColumnCount(3);
            table = ComponentFactory.getSimpleTable(tableModel, true);
            tableModel.addTableModelListener(new TableValueChangedAction());
            table.setPreferredScrollableViewportSize(new Dimension(390, 285));
    
            DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
    
            TableColumn columnNew   = table.getColumnModel().getColumn(0);
            columnNew.setHeaderValue(Resources.getText(DCLabels.lblColumnCatID));
    
            JTextField text = ComponentFactory.getTextField();
            text.setEditable(false);
            columnNew.setCellEditor(new DefaultCellEditor(text));
            renderer = new DefaultTableCellRenderer();
            renderer.setBackground(ComponentFactory.getDisabledColor());
            columnNew.setCellRenderer(renderer);
            columnNew = table.getColumnModel().getColumn(1);
            columnNew.setHeaderValue(Resources.getText(DCLabels.lblColumnCatName));
            columnNew = table.getColumnModel().getColumn(2);
            columnNew.setHeaderValue(Resources.getText(DCLabels.lblColumnDispose));
            JComboBox comboBox = ComponentFactory.getYesNoCombo();
            columnNew.setCellEditor(new DefaultCellEditor(comboBox));
            renderer  = new DefaultTableCellRenderer();
            columnNew.setCellRenderer(renderer);
    
            JScrollPane scrollResults = new JScrollPane(table);
            scrollResults.setVerticalScrollBarPolicy(scrollResults.VERTICAL_SCROLLBAR_ALWAYS);        
    
            // Build the panel
            panelCatagory.add(scrollResults,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                              new Insets(5, 5, 5, 5), 0, 0));
            
            // ************************************************************************
            // TABLE ACTION PANEL
            // Set layout style
            GridBagLayout gridbTabAct       = new GridBagLayout();
            panelTabAct.setLayout(gridbTabAct);
    
            // Initialize components
            JButton buttonSave = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave));
            JButton buttonAddRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblAddRow));
    
            buttonSave.addActionListener(new SaveCatAction());
            buttonAddRow.addActionListener(new AddRowButtonAction());
    
            // Build the panel
            panelTabAct.add(buttonSave,     new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                            new Insets( 5, 5, 5, 5), 0, 0));
            panelTabAct.add(buttonAddRow,   new GridBagConstraints( 1, 0, 1, 1, 0.0, 0.0
                                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                            new Insets( 5, 5, 5, 5), 0, 0));

    
            // ************************************************************************
            // MP3 GENRE PANEL
            // Set layout style
            GridBagLayout       gridbMP3Genres   = new GridBagLayout();
            panelMP3Genres.setLayout(gridbMP3Genres);
    
            // Initialize components
            textMP3 = ComponentFactory.getTextArea();
            textMP3.setEditable(false);
            
            JScrollPane scrollMP3Genres = new JScrollPane(textMP3);
            scrollMP3Genres.setPreferredSize(new Dimension(405, 302));
            scrollMP3Genres.setHorizontalScrollBarPolicy(scrollMP3Genres.HORIZONTAL_SCROLLBAR_NEVER);
            scrollMP3Genres.setVerticalScrollBarPolicy(scrollMP3Genres.VERTICAL_SCROLLBAR_ALWAYS);        
    
            // Build the panel
            panelMP3Genres.add(scrollMP3Genres, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                              , GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                new Insets( 5, 5, 5, 5), 0, 0));
    
    
            // ************************************************************************
            // MAIN PANEL
            // Set layout style
            GridBagLayout       gridbag         = new GridBagLayout();
            setLayout(gridbag);
    
            // Build the panel
            add(panelCatagory,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                new Insets(5, 5, 5, 5), 0, 0));
            add(panelMP3Genres, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                new Insets(5, 5, 5, 5), 0, 0));
            add(panelTabAct,    new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                new Insets(5, 5, 5, 5), 0, 0));
        }

        /**
         * Retrieves the catagories from the CatagoryCollection class
         * 
         * @see CatagoryCollection
         */
        protected void searchCatagories() {
            fillTable(CatagoryCollection.getCatagories(), true);
            vChangedIDs.removeAllElements();
        }        
        
        
        /**
         * Fills the table with the MP3 genres. The MP3 genres are retrieved
         * from a static String Array
         */
        private void fillMP3GenreList() {
            String[] genres  = Repository.collections.colMP3Genres;
            String   sGenres = "";
           
            for (int i = 0; i < genres.length; i++) {
                if (i != genres.length -1) {
                    sGenres = sGenres + genres[i] + '\n';
                } else {
                    sGenres = sGenres + genres[i];
                }
           }
           textMP3.setText(sGenres);
        }
    
        /**
         * Changes the form to reflect the selected module
         * 
         * @param module the currently active module
         */
        public void setModule(int module) {
            switch (module) {
                case 0:
                    panelCatagory.setVisible(true);
                    panelTabAct.setVisible(true);
                    panelMP3Genres.setVisible(false);
                    searchCatagories();
                    break;
                case 1:
                    panelCatagory.setVisible(false);
                    panelTabAct.setVisible(false);
                    panelMP3Genres.setVisible(true);
                    fillMP3GenreList();
                    break;
            }  
        }

        /**
         * Adds an empty row to the bottom of the tabel
         */
        public class AddRowButtonAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                tableModel.addRow(new Catagory().getValues());
            }
        }

        class SaveCatAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                saveCategories();
            }
        }
    }
}

