 /******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.core.objects.mp3.Mp3;
import rjw.dc.core.objects.software.Catagory;
import rjw.dc.core.objects.software.CatagoryCollection;
import rjw.dc.core.objects.software.Software;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;

import com.l2fprod.gui.plaf.skin.CompoundSkin;
import com.l2fprod.gui.plaf.skin.Skin;
import com.l2fprod.gui.plaf.skin.SkinLookAndFeel;

/**
 * This class is the Component holder for the UI. The look and feel, as well
 * as all the other preference are set here
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ComponentFactory {

    public static final int _SOFTWARETYPECOMBO = 0;
    public static final int _YESNOCOMBO = 1;
    public static final int _MP3GENRECOMBO = 2;
    public static final int _LONGTEXTFIELD = 3;
    public static final int _MOVIECODECCOMBO = 4;
    public static final int _YEARCOMBO = 5;
    public static final int _CHECKBOX = 6;
    public static final int _NUMBERFIELD = 7;
    public static final int _SHORTTEXTFIELD = 8;    

    /**
     * Initializes this class and sets the look and feel for all UI components.
     * The look and feel is only set the first time
     */
    static {
        try {
            Skin skin = new CompoundSkin(SkinLookAndFeel.loadThemePack(".\\skin\\DataCrowSkin.zip"),
                                         SkinLookAndFeel.loadThemePack(".\\skin\\DataCrowSkin.zip"));
            SkinLookAndFeel.setSkin(skin);
            UIManager.setLookAndFeel("com.l2fprod.gui.plaf.skin.SkinLookAndFeel");
        }
        catch (Exception e) {
            Messages.setMessage(e.fillInStackTrace().toString(), Messages._ERROR);
        }
    }

    public static Component getComponent(   int componentIndex, 
                                            String sLabel, 
                                            int maxTextLength) {
        switch (componentIndex) {
            case _SOFTWARETYPECOMBO:
                return getSoftwareCatagoriesComboBox();
            case _YESNOCOMBO:
                return getYesNoCombo();
            case _MP3GENRECOMBO:
                return getMP3GenreComboBox();
            case _LONGTEXTFIELD:
                return getLongTextField(maxTextLength);
            case _MOVIECODECCOMBO:
                return getMovieCodecComboBox();
            case _YEARCOMBO:
                return getYearComboBox();
            case _CHECKBOX:
                return getCheckBox(sLabel);
            case _NUMBERFIELD:
                return getNumberField();
        }
        return getShortTextField(maxTextLength);
    }

    public static JColorChooser getColorChooser() {
        JColorChooser colorChooser = new JColorChooser();
        return colorChooser;
    }

    public static JTree getTree(DefaultMutableTreeNode model) {
        JTree tree = new JTree(model);
        return tree;
    }

    public static GridBagLayout getLayoutManager() {
        GridBagLayout gridb = new GridBagLayout();
        return gridb;
    }

    public static NumberField getNumberField() {
        NumberField numberField = new NumberField();
        return numberField;   
    }

    /**
     * Returns a combobox with the Software database field names
     *
     * @return (JComboBox)  combobox filled with the database field names.
     *                      The first item in the list is an empty String
     */
    public static JComboBox getFieldsComboBox() {
        JComboBox fieldsComboBox = new JComboBox();
        fieldsComboBox.addItem(" ");
        for (int i = 0; i < Repository.dataBase.dbSoftColArray.length; i++) {
            fieldsComboBox.addItem(Repository.dataBase.dbSoftColArray[i].trim());
        }
        fieldsComboBox.setFont(getStandardFont());

        return fieldsComboBox;
    }

    public static JComboBox getYearComboBox() {
        JComboBox yearComboBox = new JComboBox();
        yearComboBox.addItem("");
        Date date = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int year = cal.get(cal.YEAR);

        for (int i = year; i > 1939; i--) {
            yearComboBox.addItem(String.valueOf(i));
        }

        return yearComboBox;
    }

    /**
     * Returns a combobox with Movie codecs
     *
     * @param nameOfField  name of the combobox
     * @return (JComboBox) combobox filled with Movie codecs. The first
     *                      item is an empty String
     */
    public static JComboBox getMovieCodecComboBox() {
        JComboBox codecComboBox = new JComboBox();
        codecComboBox.addItem("");
        codecComboBox.setSelectedIndex(0);
        codecComboBox.setFont(getStandardFont());

        for (int i = 0; i < Repository.collections.colMovieCodecs.length; i++) {
            codecComboBox.addItem(Repository.collections.colMovieCodecs[i]);
        }

        return codecComboBox;
    }

    /**
     * Returns a combobox with MP3 genres
     *
     * @return (JComboBox)  combobox filled with MP3 genres. The first item is
     *                      an empty String
     */
    public static JComboBox getMP3GenreComboBox() {
        JComboBox genreComboBox = new JComboBox();
        genreComboBox.addItem("");
        genreComboBox.setSelectedIndex(0);
        genreComboBox.setFont(getStandardFont());

        for (int i = 0; i < Repository.collections.colMP3Genres.length; i++) {
            genreComboBox.addItem(Repository.collections.colMP3Genres[i]);
        }

        return genreComboBox;
    }

    /**
     * Returns a combobox with Sofware types (Catagories
     *
     * @return (JComboBox)  combobox filled with Software types. The first item
     *                      is an empty String.
     */
    public static JComboBox getSoftwareCatagoriesComboBox() {
        JComboBox typeComboBox = new JComboBox();
        typeComboBox.addItem("");
        typeComboBox = fillSoftwareCatagories(typeComboBox);
        typeComboBox.setSelectedIndex(0);
        typeComboBox.setFont(getStandardFont());

        typeComboBox.addActionListener(new RefreshSoftwareCategories());

        return typeComboBox;
    }

    /**
     * Adds the software catagories to the Software type combobox. The Software
     * types are loaded from the CatagoryCollection class.
     */
    public static JComboBox fillSoftwareCatagories(JComboBox typeComboBox) {
        // initialize the catagory (first time init will init the Collection)
        Object oSelected = typeComboBox.getSelectedItem();
        Catagory cat = new Catagory();
        Vector vData = CatagoryCollection.getCatagories();
        typeComboBox.removeAllItems();
        typeComboBox.addItem("");
        typeComboBox.setFont(getStandardFont());

        for (Enumeration enum = vData.elements(); enum.hasMoreElements(); ) {
            cat = (Catagory) enum.nextElement();
            typeComboBox.addItem(cat.getValue(Catagory._A_NAME));
        }

        try {
            typeComboBox.setSelectedItem(oSelected);
        } catch (Exception exp) {}

        return typeComboBox;
    }

    /**
     * Returns an empty combobox
     *
     * @param length        length of this field
     * @return (JComboBox)  an empty combobox
     */
    public static JComboBox getComboBox(int length) {
        JComboBox comboBox = getComboBox();
        comboBox.setPreferredSize(new Dimension(length, 23));
        return comboBox;
    }

    /**
     * Returns an empty combobox
     *
     * @param nameOfField   name of this field
     * @return (JComboBox)  an empty combobox
     */
    public static JComboBox getComboBox() {
        JComboBox comboBox = new JComboBox();
        comboBox.setFont(getStandardFont());
        return comboBox;
    }

    /**
     * Returns a combobox with the items "yes" and "no", not starting with
     * and empty String
     *
     * @return (JComboBox)  combobox with the items "yes" and "no"
     */
    public static JComboBox getRealYesNoComboBox() {
        JComboBox comboBox = new JComboBox();
        comboBox.addItem(Resources.getText(DCLabels.lblNo));
        comboBox.addItem(Resources.getText(DCLabels.lblYes));
        comboBox.setFont(getStandardFont());
        return comboBox;
    }

    /**
     * Returns a standard checkbox
     *
     * @param nameOfField   name of this field
     * @return (JCheckBox)  simple checbox, no prefenrences are set
     */
    public static JCheckBox getCheckBox(String labelText) {
        JCheckBox checkBox = new JCheckBox(labelText);
        checkBox.setFont(getStandardFont());

        return checkBox;
    }

    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @param length        length of this button
     * @return (JButton)    button, height 23
     */
    public static JButton getButton(String buttonText, int length) {
        JButton button = getButton(buttonText);
        button.setPreferredSize(new Dimension(length, 23));
        return button;
    }
    
    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @param length        length of this button
     * @return (JButton)    button, height 23
     */
    public static JButton getButton(ImageIcon icon) {
        JButton button = new JButton(icon);
        return button;
    }    

    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @return (JButton)    button, height 23
     */
    public static JButton getButton(String buttonText) {
        JButton button = new JButton();
        button.setName("bt" + buttonText);
        button.setText(buttonText);
        button.setFont(getStandardFont());
        return button;
    }

    /**
     * Returns a long text field with all the defaults set
     */
    public static JTextArea getTextArea() {
        JTextArea textArea = new JTextArea();
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setEditable(true);
        textArea.setDoubleBuffered(true);
        textArea.setBackground(getTableColor());
        textArea.setMargin(new Insets(5,5,5,5));
        textArea.setFont(getStandardFont());

        return textArea;
    }

    /**
     * Returns a text area with all the defaults set
     */
    public static LongTextField getLongTextField(int maxTextLength) {
        LongTextField longText = new LongTextField(maxTextLength);
        longText.setWrapStyleWord(true);
        longText.setLineWrap(true);
        longText.setEditable(true);
        longText.setDoubleBuffered(true);
        longText.setBackground(getTableColor());
        longText.setMargin(new Insets(5,5,5,5));
        longText.setFont(getStandardFont());

        return longText;
    }

    /**
     * Returns a standard label with a text
     *
     * @param length        length of this field
     * @param labeltext     display String
     * @return (JLabel)     label, no preferemces are set
     */
    public static  JLabel getLabel(String labelText, int length) {
        JLabel label = getLabel(labelText);
        label.setPreferredSize(new Dimension(length, 23));
        return label;
    }

    /**
     * Returns a standard label with a text
     *
     * @param labeltext     display String
     * @return (JLabel)     label, no preferemces are set
     */
    public static JLabel getLabel(String labelText) {
        JLabel label = new JLabel();
        label.setText(labelText);
        label.setRequestFocusEnabled(false);
        label.setFont(getStandardFont());

        return label;
    }

    /**
     * Returns a standard editor pane
     *
     * @return (JEditorPane)    editor pane, no preferences are set
     */
    public static JEditorPane getEditorPane() {
        JEditorPane editorPane = new JEditorPane();
        editorPane.setFont(getStandardFont());
        editorPane.setBackground(getTableColor());        

        return editorPane;
    }

    public static ShortTextField getShortTextField(int maxTextLength) {
        ShortTextField textField = new ShortTextField(maxTextLength);
        textField.setFont(getStandardFont());        
        return textField;
    }

    /**
     * Returns a textfield
     *
     * @param length  length of this field
     */
    public static JTextField getTextField(int length) {
        JTextField textField = getTextField();
        return textField;
    } 

    /**
     * Returns a textfield
     *
     * @return JTextField   textfield, height 23
     */
    public static JTextField getTextField() {
        JTextField textField = new JTextField();
        textField.setFont(getStandardFont());
        return textField;
    }

    /**
     * Returns a standard tabbed pane
     *
     * @return (JTabbedPane)    tabbed pane, no prefences are set
     */
    public static JTabbedPane getTabbedPane() {
        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.setFont(getStandardFont());

        return tabbedPane;
    }

    /**
     * Returns a standard icon
     *
     * @param iconLocation  location of the icon image
     * @return (Icon)       normal icon
     */
    public static Icon getIcon(String iconLocation) {
        Icon icon = new ImageIcon(iconLocation);
        return icon;
    }

    /**
     * Returns a standard menubar
     *
     * @param nameOfField   name of this field
     * @return (JMenuBar)   standard menubar, no prefences are set
     */
    public static JMenuBar getMenuBar() {
        JMenuBar menuBar = new JMenuBar();
        menuBar.setFont(getStandardFont());
        return menuBar;
    }

    /**
     * Returns a combobox with the items " ", "yes" and "no"
     *
     * @param nameOfField   name of this field
     * @return (JComboBox)  a combox with the items "yes" and "no", the first
     *                      item is an empty String
     */
    public static JComboBox getYesNoCombo() {
        JComboBox comboBox = new JComboBox();
        comboBox.addItem("");
        comboBox.addItem(Resources.getText(DCLabels.lblYes));
        comboBox.addItem(Resources.getText(DCLabels.lblNo));
        comboBox.setFont(getStandardFont());
        return comboBox;
    }
    
    private static JTable getDefaultTable(DefaultTableModel tableModel,
                                          boolean autoScrolls) {
        JTable table = new JTable(tableModel);

        table.setBackground(getTableColor());
        table.setAutoscrolls(autoScrolls);

        table.setDefaultRenderer(Object.class, new MyTableCellRenderer());
        table.getTableHeader().setForeground(getTableHeaderColor());
        table.getTableHeader().setFont(getTableHeaderFont());
        table.setFont(getStandardFont());

        table.setRowSelectionAllowed(true);
        table.setColumnSelectionAllowed(true);
        table.setCellSelectionEnabled(true);
        table.setRequestFocusEnabled(true);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        table.setAlignmentY(JTable.TOP_ALIGNMENT);
        table.getTableHeader().setReorderingAllowed(true);
        
        return table;
    }

    /**
     * Returns a table for presentation of Software objects
     *
     * @param tableModel    model to be used upon this tabel
     * @param autoScrolls   specifies if the autoscrolls should be used
     * @return (JTable)     table with table header names and comboboxes for
     *                       Software types and booleans
     */
    public static JTable getSoftwareTable(
                         DefaultTableModel tableModel,
                         boolean autoScrolls) {

        JTable table = getDefaultTable(tableModel, autoScrolls);
        table = convertToSoftwareTable(table);
        return table;
    }

    public static JTable convertToSoftwareTable(JTable table) {
        ((DefaultTableModel) table.getModel()).setColumnCount(
                             Repository.dataBase.dbSoftColArrayPK.length);
                             
        String[] headers = new Software().getLabels();
        int[] maxLength = new Software().getMaxFieldLengths();
        for (int i = 0; i < Repository.dataBase.dbSoftColArrayPK.length; i++) {
            TableColumn columnNew = table.getColumnModel().getColumn(i);

            if (i == 0) {
                columnNew.setHeaderValue("#");                
            } else {
                columnNew.setHeaderValue(headers[i]);    
            }

            if (i == 0) {
                JTextField text = new JTextField();
                text.setEditable(false);
                columnNew.setCellEditor(new DefaultCellEditor(text));
                columnNew.setMaxWidth(75);
                DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
                renderer.setBackground(getDisabledColor());
                columnNew.setCellRenderer(renderer);
            } else if (i == 5 || i == 7) {
                JComboBox comboBox = getYesNoCombo();
                columnNew.setCellEditor(new DefaultCellEditor(comboBox));
                columnNew.setMaxWidth(75);
            } else if (i == 4) {
                JComboBox comboBox = getSoftwareCatagoriesComboBox();
                comboBox.addActionListener(new RefreshSoftwareCategories());
                columnNew.setPreferredWidth(50);
                columnNew.setCellEditor(new DefaultCellEditor(comboBox));
            } else {
                ShortTextField text = new ShortTextField(maxLength[i]);
                columnNew.setCellEditor(new DefaultCellEditor(text));
                columnNew.setPreferredWidth(50);
            }
            columnNew.setHeaderRenderer(getTableHeaderRenderer());
        }

        return table;
    }

    public static DefaultTableCellRenderer getTableHeaderRenderer() {
        DefaultTableCellRenderer hdrRenderer = 
                        new DefaultTableCellRenderer();
        hdrRenderer.setBackground(new Color(0,51,51));
        hdrRenderer.setForeground(new Color(255,255,255));
        hdrRenderer.setBorder(new EtchedBorder());
        return hdrRenderer;
    }

    /**
     * Returns a table for presentation of MP3 objects
     *
     * @param tableModel    model to be used upon this tabel
     * @param autoScrolls   specifies if the autoscrolls should be used
     * @return (JTable)     table with table header names and comboboxes for
     *                      MP3 genres and booleans
     */
     public static JTable getMP3Table(
                          DefaultTableModel tableModel,
                          boolean autoScrolls) {

        JTable table = getDefaultTable(tableModel, autoScrolls);
        table = convertToMP3Table(table);
        return table;
    }

    public static JTable convertToMP3Table(JTable table) {
        ((DefaultTableModel) table.getModel()).setColumnCount(
                             Repository.dataBase.dbMP3ColArrayPK.length);
        String[] headers = new Mp3().getLabels();
        int[] maxLengths = new Mp3().getMaxFieldLengths();
        for (int i = 0; i < Repository.dataBase.dbMP3ColArrayPK.length; i++) {
            TableColumn columnNew = table.getColumnModel().getColumn(i);

            if (i == 0) {
                columnNew.setHeaderValue("#");                
            } else {
                columnNew.setHeaderValue(headers[i]);    
            }

            if (i == 0) {
                JTextField text = new JTextField();
                text.setEditable(false);
                columnNew.setCellEditor(new DefaultCellEditor(text));
                DefaultTableCellRenderer renderer =
                    new DefaultTableCellRenderer();
                renderer = new DefaultTableCellRenderer();
                renderer.setBackground(getDisabledColor());
                columnNew.setCellRenderer(renderer);
                columnNew.setMaxWidth(75);
            } else if (i == 5) {
                JComboBox comboBox = getYearComboBox();
                columnNew.setPreferredWidth(25);
                columnNew.setCellEditor(new DefaultCellEditor(comboBox));
            } else if (i == 10) {
                JComboBox comboBox = getYesNoCombo();
                columnNew.setPreferredWidth(25);
                columnNew.setCellEditor(new DefaultCellEditor(comboBox));
                columnNew.setMaxWidth(75);
            } else if (i == 7 || i == 8) {
                NumberField numberField = getNumberField();
                columnNew.setCellEditor(new DefaultCellEditor(numberField));
            } else if (i == 6) {
                JComboBox comboBox = getMP3GenreComboBox();
                columnNew.setPreferredWidth(50);
                columnNew.setCellEditor(new DefaultCellEditor(comboBox));
            } else {
                ShortTextField textField = getShortTextField(maxLengths[i]);
                columnNew.setCellEditor(new DefaultCellEditor(textField));
            }
            columnNew.setHeaderRenderer(getTableHeaderRenderer());            
        }
        table.getTableHeader().setReorderingAllowed(true);
        return table;
    }


    /**
     * Returns a table for presentation of Movie objects
     *
     * @param nameOfField   name of this field
     * @param tableModel    model to be used upon this tabel
     * @param autoScrolls   specifies if the autoscrolls should be used
     * @return (JTable)     table with table header names and comboboxes for
     *                       MP3 genres and booleans
     */
     public static JTable getMovieTable(
                          DefaultTableModel tableModel,
                          boolean autoScrolls) {

        JTable table = getDefaultTable(tableModel, autoScrolls);
        table = convertToMovieTable(table);
        return table;
    }

    public static JTable convertToMovieTable(JTable table) {
        ((DefaultTableModel) table.getModel()).setColumnCount(
                             Repository.dataBase.dbMovieColArrayPK.length);
        String[] headers = new Movie().getLabels();
        int[] maxLengths = new Movie().getMaxFieldLengths();
        
        for (int i = 0; i < Repository.dataBase.dbMovieColArrayPK.length; i++) {
            TableColumn columnNew = table.getColumnModel().getColumn(i);

            if (i == 0) {
                columnNew.setHeaderValue("#");                
            } else {
                columnNew.setHeaderValue(headers[i]);    
            }

            if (i == 0) {
                JTextField text = new JTextField();
                text.setEditable(false);
                columnNew.setCellEditor(new DefaultCellEditor(text));
                DefaultTableCellRenderer renderer =
                    new DefaultTableCellRenderer();
                renderer = new DefaultTableCellRenderer();
                renderer.setBackground(getDisabledColor());
                columnNew.setMaxWidth(75);
                columnNew.setCellRenderer(renderer);
            } else if (i == 8) {
                JComboBox comboBox = getYearComboBox();
                columnNew.setPreferredWidth(25);
                columnNew.setCellEditor(new DefaultCellEditor(comboBox));
            } else if (i == 9) {
                NumberField numberField = getNumberField();
                columnNew.setCellEditor(new DefaultCellEditor(numberField));
            } else if (i == 12) {
                JComboBox comboBox = getYesNoCombo();
                columnNew.setPreferredWidth(25);
                columnNew.setCellEditor(new DefaultCellEditor(comboBox));
                columnNew.setMaxWidth(75);
            } else {
                ShortTextField textField = getShortTextField(maxLengths[i]);
                columnNew.setCellEditor(new DefaultCellEditor(textField));
            }
            columnNew.setHeaderRenderer(getTableHeaderRenderer());
        }
        table.getTableHeader().setReorderingAllowed(true);
        return table;
    }


    /**
     * Returns a standard table
     *
     * @param tableModel    model to be used upon this tabel
     * @param autoScrolls   specifies if the autoscrolls should be used
     * @return (JTable)     table without headers and components
     */
    public static JTable getSimpleTable(
                         TableModel tableModel,
                         boolean autoScrolls) {

        JTable table = new JTable(tableModel);
        table.setBackground(getTableColor());
        table.setAutoscrolls(autoScrolls);
        table.setColumnSelectionAllowed(true);
        table.setCellSelectionEnabled(true);
        table.setRequestFocusEnabled(true);
        table.setAlignmentY(JTable.TOP_ALIGNMENT);
        table.getTableHeader().setForeground(getTableHeaderColor());
        table.getTableHeader().setFont(getTableHeaderFont());
        table.setDefaultRenderer(Object.class, new MyTableCellRenderer());
        table.setFont(getStandardFont());
        table.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);

        return table;
    }


    /**
     * Table cell renderer for displaying tooltips of the content of the cells
     */
    private static class MyTableCellRenderer
        extends javax.swing.table.DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(
                JTable table, Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {

            Component c = super.getTableCellRendererComponent(
                                                                table,
                                                                value,
                                                                isSelected,
                                                                hasFocus,
                                                                row,
                                                                column);
            if (value instanceof Object) {
                ((JComponent) c).setToolTipText(value.toString());
            }
            return c;
        }
    }

    /**
     * Returns a standard border with a title
     *
     * @param title             display String
     * @return (TitledBorder)   border with text
     */
    public static TitledBorder getBorder(String title) {
        TitledBorder border = new TitledBorder(new javax.swing.border.EtchedBorder(), title);
        border.setTitleFont(new Font("Comic Sans MS", Font.BOLD, 11));
        border.setTitleColor(new Color(0,0,0));
        return border;
    }



    /**
     * Returns the default font for the menubar
     *
     * @return (Font)   Arial Narrow, normal, pixelheight 9
     */
    public static Font getDefaultFont() {
     //   return new Font("Arial Narrow", Font.NORMAL, 9);
     return new Font("Comic Sans MS", Font.BOLD, 9);
    }

    /**
     * Returns the default font for the menubar
     *
     * @return (Font)   Comic Sans MS font, bold, pixelheight 9
     */
    public static Font getTableHeaderFont() {
        return new Font("Comic Sans MS", Font.BOLD, 9);
    }

    /**
     * Returns the default font for the menubar
     *
     * @return (Font)   Comic Sans MS font, bold, pixelheight 9
     */
    public static Font getMenuBarFont() {
        return new Font("Comic Sans MS", Font.BOLD, 9);
    }

    /**
     * Returns the default font for comboboxes in the menubar
     *
     * @return (Font)   Comic Sans MS font, bold, pixelheight 11
     */
    public static Font getMenuBarFontCombo() {
        return new Font("Comic Sans MS", Font.BOLD, 11);
    }

    /**
     * Returns the default font
     *
     * @return (Font)   Verdana font, pixelheight 11
     */
    public static Font getStandardFont() {
        return new Font("Arial", Font.PLAIN, 12);
    }

    /**
     * Returns the color for displaying disabled fields
     *
     * @return (Color)  light grey color, red 204, green 204, blue 204
     */
    public static Color getDisabledColor() {
        return new Color(204,204,204);
    }

    /**
     * Returns the color for the table header
     *
     * @return (Color)  light grey color, red 204, green 204, blue 204
     */
    public static Color getTableHeaderColor() {
        return new Color(204,204,204);
    }

    /**
     * Returns the color for the background of tables
     *
     * @return (Color)  light yellow, red 246, green 246, blue 230
     */
    public static Color getTableColor() {
        //return new Color(246, 246, 230);
        return new Color(255, 255, 255);
    }

    /**
     * Forces a refresh of the content of the Software categories combobox
     */
    public static class RefreshSoftwareCategories implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JComboBox comboBox  = (JComboBox) e.getSource();
            comboBox = fillSoftwareCatagories(comboBox);
        }
    }
    
    private static class TableInputVerifier extends InputVerifier {
        public boolean verify(JComponent component) {
            return true;
        }
    }
}
