package rjw.dc.console.components;

import java.awt.Toolkit;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import rjw.flex.settings.ui.ISettingsComponent;

/**
 * Simple field, derived from JTextField, which does not allow non
 * numeric characters to be inserted
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class NumberField extends JTextField implements ISettingsComponent {

    protected class WholeNumberDocument extends PlainDocument {

        protected WholeNumberDocument() {
        }

        public void insertString(int i, String s, AttributeSet attributeset)
            throws BadLocationException {

            char ac[] = s.toCharArray();
            char ac1[] = new char[ac.length];
            int j = 0;
            for(int k = 0; k < ac1.length; k++)
                if(Character.isDigit(ac[k]))
                    ac1[j++] = ac[k];
                else
                    toolkit.beep();

            super.insertString(i, new String(ac1, 0, j), attributeset);
        }
    }

    private Toolkit toolkit;
    private NumberFormat integerFormatter;

    public NumberField() {
        super();
        toolkit = Toolkit.getDefaultToolkit();
        integerFormatter = NumberFormat.getNumberInstance(Locale.US);
        integerFormatter.setParseIntegerOnly(true);
    }

    protected Document createDefaultModel() {
        return new WholeNumberDocument();
    }

    public Object getValue() {
        int i = -1;
        try {
            i = integerFormatter.parse(getText()).intValue();
        } catch(ParseException _ex) {
            toolkit.beep();
        }
        return new Integer(i);
    }

    public void setValue(int i) {
        setText(integerFormatter.format(i));
    }

    public void setValue(Object o) {
        setText(integerFormatter.format(((Integer) o).intValue()));
    }
}
