package rjw.dc.console.components;

import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import rjw.dc.util.BrowserLauncher;
import rjw.flex.settings.ui.ISettingsComponent;


/**
 * Simple field, derived from JTextField, which does not allow non
 * strings longer then the maximum database field length to be 
 * inserted
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ShortTextField extends JTextField implements ISettingsComponent {

    int MAX_TEXT_LENGTH = 0;

    protected class ShortStringDocument extends PlainDocument {

        protected ShortStringDocument() {
        }

        public void insertString(int i, String s, AttributeSet attributeset)
            throws BadLocationException {

            if (i + 1 > MAX_TEXT_LENGTH) {
                toolkit.beep();
            } else {
                super.insertString(i, new String(s), attributeset);
            }
        }
    }

    private Toolkit toolkit;

    public ShortTextField(int maxTextLength) {
        super();
        this.addMouseListener(new OpenUrlAction());
        MAX_TEXT_LENGTH = maxTextLength;
        toolkit = Toolkit.getDefaultToolkit();
    }

    protected Document createDefaultModel() {
        return new ShortStringDocument();
    }

    public Object getValue() {
        return getText();
    }

    public void setValue(String s) {
        setText(s);
    }

    public void setValue(Object o) {
        setText(o.toString());
    }
    
    private class OpenUrlAction implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                try {
                    URL url = new URL(getText());
                    BrowserLauncher.openURL(url.toString());
                } catch (Exception ignore) {}
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {
        }        
        
    }
}
