/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;

import rjw.dc.console.MessageBox;
import rjw.dc.console.RecordForm;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.messages.Messages;
import rjw.dc.processes.ImdbUpdate;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;

/**
 * Base panel for all panels which contain a JTable with all the
 * fields of the database.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class TablePanel extends DataCrowPanel {

    protected RecordForm recordForm;
    protected static Vector vRecordForms = new Vector();

    public DefaultTableModel tableModel = new DefaultTableModel();
    public JTable table;
    protected Vector vChangedIDs = new Vector();
    protected Vector vChangedRowNumbers = new Vector();

    protected int succSaved = 0;
    protected int failedSaved = 0;

    /**
     * Fills the table with the specified values
     *
     * @param vTable    vector containing the rows of data to be displayed
     */
    public void fillTable(Vector vTable, boolean displayID) {
        clearTable();
        try {
            DataCrowObject o = null;

            for (Enumeration enum = vTable.elements(); enum.hasMoreElements(); ) {
                o = (DataCrowObject) enum.nextElement();
                tableModel.addRow(o.getValues());
            }
            String sRecords = new Integer(vTable.size()).toString();
            setStatus(Resources.getText(DCMessages.msgSearchResult, sRecords));
        } catch (java.lang.Exception exp) {
            Messages.setMessage(
                exp.toString(), Messages._ERROR);
        }
    }
    
    public int getRowNumberWithID(String ID) {
        for (int i = 0; i < tableModel.getRowCount(); i++) {
            String sID = table.getValueAt(i, 0).toString();
            if (sID.equals(ID)) {
                return i;
            }
        }
        return -1;
    }
    
    public void updateRow(String ID, Object[] values) {
        int row = getRowNumberWithID(ID);
        
        if (row != -1) {
            for (int j = 0; j < values.length; j++) {
                table.setValueAt(values[j], row, j);
            }
        }
    }

    public void addRow(Vector vDataCrowObjects) {
        for (Enumeration enum = vDataCrowObjects.elements(); enum.hasMoreElements(); ) {
            DataCrowObject lcObj = (DataCrowObject) enum.nextElement();
            Object[] row = lcObj.getValues();
            row[0] = String.valueOf(tableModel.getRowCount() + 1);
            tableModel.addRow(row);
        }
    }

    /**
     * Clear the table of all values
     */
    public void clearTable() {
        tableModel.setRowCount(0);
        setStatus(Resources.getText(DCMessages.msgTableCleared));
    }

    /**
     * Adds an empty row to the bottom of the tabel
     */
    public class AddRowButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Vector v = new Vector();
            v.add(String.valueOf(tableModel.getRowCount() + 1));
            tableModel.addRow(v);
        }
    }

    /**
     * Removes the selected rows
     */
    public class RemoveRowButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (tableModel.getRowCount() < 1 ) {
                return;
            }
            int[] rows = table.getSelectedRows();
            if (rows.length == 0) {
                tableModel.removeRow(tableModel.getRowCount() -1);

            } else {
                for (int i = rows.length; i > 0; i--) {
                    tableModel.removeRow(rows[i-1]);
                }
            }
        }
    }

    public Vector getNewValues(String[] databaseColumns) {
        Vector vRows = new Vector(0);
        
        for (int i = 0; i < table.getRowCount(); i++) {
            Object[] row = new String[databaseColumns.length];
            for (int j = 0; j < row.length; j++) {
                if (j == 0) {
                    row[j] = "";
                } else {
                    Object oTest = table.getValueAt(i,j);
                    if (oTest != null && !oTest.toString().trim().equals("")) {
                        row[j] = oTest.toString();
                    } else {
                        row[j] = "Null";
                    }
                }
            }
            vRows.addElement(row);
        }
        return vRows;
    }

    public Vector getChangedValues(String[] databaseColumns) {
        Vector vRows = new Vector(0);
        int row = 0;
        for (Enumeration enum = vChangedIDs.elements(); enum.hasMoreElements(); ) {
            String[] rowData = new String[databaseColumns.length];
            row = getRowNumberWithID(enum.nextElement().toString());
            if (row != -1) {
                for (int j = 0; j < databaseColumns.length; j++) {
                    Object oTest = table.getValueAt(row, j);
                    if (oTest != null) {
                        rowData[j] = oTest.toString();
                        if (rowData[j].trim().equals("")) {
                            rowData[j] = Repository.dataBase.dBEmptyValue;
                        }
                    } else {
                        rowData[j] = Repository.dataBase.dBEmptyValue;
                    }
                }
            }
            vRows.addElement(rowData);
        }
        return vRows;
    }

    /**
     * Updates a row at the given rownumber. Skips the ID number
     * and thus starts with the update in column 1 instead of 0
     */
    public void updateRowAt(int rowNumber, 
                            Object[] rowData, 
                            boolean overWrite) {
        for (int i = 0; i < rowData.length; i++) {
            Object oNew = rowData[i];
            Object oExist = table.getValueAt(rowNumber, i + 1);
            
            if (!overWrite && (oExist != null && 
                               !oExist.toString().trim().equals(""))) {
                oNew = oExist;   
            }
            
            if (oNew == null || oNew.toString().trim().equals("")) {
                oNew = oExist;   
            }
            
            table.setValueAt(oNew, rowNumber, i + 1);
        }
    }
    
    public void startIMDBUpdate(boolean selectedRowsOnly, boolean update) {
        Vector vNames = new Vector(0);
        Vector vIDs = new Vector(0);
        
        if (selectedRowsOnly) {
            int[] rows = table.getSelectedRows();
            
            if (rows == null || rows.length < 1) {
                new MessageBox(DataCrow.mainFrame,
                               Resources.getText(DCMessages.msgSelectRow),
                               true, MessageBox._INFORMATION);
                return;
            }
            
            for (int i = 0; i < rows.length; i++) {
                Object oName = table.getValueAt(rows[i], 1);
                if (oName != null) {
                    vNames.add(oName);
                    vIDs.add(new Integer(rows[i]));
                }
            }
        } else {
            for (int i = 0; i < tableModel.getRowCount(); i++) {
                Object oName = table.getValueAt(i, 1);
                if (oName != null) {
                    vNames.add(oName);
                    vIDs.add(new Integer(i));              
                }
            }
            if (vNames.size() < 1) {
                new MessageBox(DataCrow.mainFrame,
                               Resources.getText(DCMessages.msgNoRowsFound),
                               true, MessageBox._INFORMATION);
                return;   
            }
        }
                
        Object[][] o = new Object[vNames.size()][2];
        for (int j = 0; j < o.length; j++) {
            o[j][0] = vIDs.elementAt(j);
            o[j][1] = vNames.elementAt(j);            
        } 
        
        Messages.setMessage(Resources.getText(DCMessages.msgIMDBUpdateStarts), 
                            Messages._STANDARD);
        Thread threadIMDB = new Thread(new ImdbUpdate(o, update));
        threadIMDB.setDaemon(true);
        threadIMDB.setPriority(1);
        threadIMDB.start();
    }    

    /**
     * Closes the opened record form. All opened record forms are cached. By using
     * the form ID, the appropriate record form is retrieved and closed
     *
     * @param sFormID   the ID of the form to be closed. The ID is the same as the
     *                  record ID in the database
     */
    public void closeRecordForm(String sFormID) {
        for (Enumeration enum = vRecordForms.elements(); enum.hasMoreElements(); ) {
            recordForm  = (RecordForm) enum.nextElement();

            if (recordForm != null && recordForm.getFormID().equals(sFormID)) {
                recordForm.dispose();
                vRecordForms.remove(recordForm);
            }
        }
    }

    /**
     * Clears the table
     */
    public class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clearTable();
        }
    }

    /**
     * Handles changes made to the table (on screen)
     */
    public class TableValueChangedAction implements TableModelListener {
        public void tableChanged(TableModelEvent e) {
            if (e.getType() == e.INSERT || e.getType() == e.UPDATE) {
                if (table.getSelectedRow() != -1) {
                    Integer integer = new Integer(table.getSelectedRow());
                    if (!vChangedRowNumbers.contains(integer)) {
                        vChangedRowNumbers.addElement(integer);
                    }
                    
                    String sID = table.getValueAt(table.getSelectedRow(), 0).toString();
                    if (!vChangedIDs.contains(sID)) {
                        vChangedIDs.addElement(sID);
                    }
                }
            }
        }
    }
}
