/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.da;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JPanel;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.DataCrowPanel;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;

/**
 * The Data Exchange panel, consisting out of DataExchangeProcessPanel
 * and DataExchangeSourceEditor
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DataExchangePanel extends DataCrowPanel {

    public  DataExchangeProcessPanel daProcessPanel;
    public  DataExchangeSourceEditor daSourceEditor;

    private JPanel panelStatus;
    private JLabel labelStatus;

    /**
     * Initializes and creates the panel
     */
    public DataExchangePanel() {
        buildPanel();
    }

    /**
     * On changing the module the general settings panel and the import
     * panel have to adjust their presentation and behaviour
     *
     * @param iModule
     */
    public void switchModule() {
        daProcessPanel.switchModule();
        daSourceEditor.switchModule();
    }

    /**
     * Creates the panel
     */
    private void buildPanel() {
        panelStatus = new JPanel(false);


        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

        // Initialize components
        labelStatus = ComponentFactory.getLabel(
                      Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(labelStatus, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0,  0, 0, 0), 0, 0));

        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbExportCD   = new GridBagLayout();
        setLayout(gridbExportCD);

        // Initialize components
        daProcessPanel = new DataExchangeProcessPanel();
        daSourceEditor = new DataExchangeSourceEditor();

        panelStatus.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblStatus)));

        // Build the panel
        add( daProcessPanel, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets( 0, 0, 0, 0), 0, 0));
        add( daSourceEditor, new GridBagConstraints( 0, 1, 1, 1, 40.0, 40.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets( 0, 0, 0, 0), 0, 0));
        add( panelStatus,    new GridBagConstraints( 0, 2, 2, 1, 1.0, 1.0
                            ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                             new Insets( 5, 5, 5, 5), 0, 0));
    }

    /**
     * Applies the background color to the sub panels
     *
     * @param color     background color
     */
    public void setBackgrounds(Color color) {
        this.setBackground(color);
        daProcessPanel.setBackground(color);
        daSourceEditor.setBackground(color);

        try {
            setCompBackColors(daProcessPanel.getComponents(), color);
            setCompBackColors(daSourceEditor.getComponents(), color);
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Applies the background color to all the components
     *
     * @param comps     the components on which the color is applied
     * @param color     background color
     */
    private void setCompBackColors(Component[] comps, Color color) {
        try {
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
            }
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }
}
