/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.da;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.RandomAccessFile;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import rjw.dc.console.BrowserDialog;
import rjw.dc.console.ExportCDDialog;
import rjw.dc.console.ImportListDialog;
import rjw.dc.console.MessageBox;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.TablePanel;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.CdRom;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.processes.ExportMP3CD;
import rjw.dc.processes.ExportSoftwareCD;
import rjw.dc.processes.ImporList;
import rjw.dc.processes.ImportInformationFile;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Part of the SettingsPanel. Can starty Export and Import jobs
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DataExchangeProcessPanel extends TablePanel implements TableModelListener {

    private File                iniFile;
    private RandomAccessFile    fileAccess;

    private JPanel             panelExportActions;
    private JPanel             panelImportActions;
    private JPanel             panelExportCD;
    private JPanel             panelImportFile;
    private JPanel             panelImportSettings;
    private JPanel             panelTabAct;

    private JButton            buttonSaveTable;
    private JButton            buttonLoadTable;
    private JButton            buttonAddRow;
    private JButton            buttonRemoveRow;

    private JButton            buttonFileImport;
    private JLabel             labelSource;
    private JTextField         textImportFile;
    private JTextField         textExportFile;
    private JTextField         textCDName;
    private JLabel             labelCDName;
    private JLabel             labelFile;
    private JButton            buttonCDExport;
    private JButton            buttonFilChoose;
    private JLabel             labelLocation;
    private JButton            buttonImpFile;
    private JButton            buttonExpFile;
    private File               selectedDir;
    private JComboBox          comboLoc;
    private JLabel             labelStatus;

    private JCheckBox checkVerify;
    private JCheckBox checkToFile;
    private JCheckBox checkToTable;

    private static final WorkFlow wf = new WorkFlow();


    public DataExchangeProcessPanel() {
        buildPanel();
    }

    protected void importFile() {
        Object      oTrue   = new Boolean(true);

        File test = new File(getImportFile());
        if (!test.exists()) {
            new MessageBox( DataCrow.mainFrame,
                            Resources.getText(DCMessages.msgIIFSelectImpFile), 
                            true, MessageBox._NOTIFICATION);
            return;
        }
        ImportInformationFile   iff       = new ImportInformationFile(getImportFile());
        ImportListDialog        impDialog = new ImportListDialog(getImportFile(), iff.getIIFValues());

        if (impDialog.result != null) {
            if (impDialog.result.equals(oTrue)) {
                Messages.setMessage(Resources.getText(DCMessages.msgImportStarts), Messages._STANDARD);
                MessagesWriter._useLogFile = false;
                Thread threadImportFile = new Thread(new ImporList(getImportFile()), "ImportFile");
                threadImportFile.setDaemon(true);
                threadImportFile.setPriority(1);
                threadImportFile.start();
                MessagesWriter._windowToFront = true;
                MessagesWriter._windowShow    = true;
            }
        }
    }

    protected void startCDExport() {
        boolean bVerify = checkVerify.isSelected();
        boolean bToFile = checkToFile.isSelected();
        boolean bToTable = checkToTable.isSelected();
        final String location = comboLoc.getSelectedItem().toString();
        String cdName = textCDName.getText().trim();

        if (cdName.equals("")) {
            new MessageBox(DataCrow.mainFrame,
                           Resources.getText(DCMessages.msgEnterCDName), 
                           true, MessageBox._INFORMATION);
            return;
        } else if (getExportFile().equals("")) {
            new MessageBox(DataCrow.mainFrame,
                           Resources.getText(DCMessages.msgEnterFilename), 
                           true, MessageBox._INFORMATION);
            return;
        }

        try {
            File testje         = new File(getExportFile());
            RandomAccessFile    randomAccessTest;
            randomAccessTest    = new RandomAccessFile(testje, "rw");
            randomAccessTest.close();
        } catch (Exception ioExp) {
            new MessageBox( DataCrow.mainFrame,
                            Resources.getText(DCMessages.msgInvalidFileName), 
                            true, MessageBox._INFORMATION);
            return;
        }

        Object oTrue   = new Boolean(true);
        ExportCDDialog expDialog = new ExportCDDialog(
                                          location,
                                          new Boolean(bVerify).toString(),
                                          getExportFile(),
                                          cdName,
                                          new Boolean(bToFile).toString(),
                                          new Boolean(bToTable).toString());

        if (expDialog.result != null) {
            if (expDialog.result.equals(oTrue)) {
                int iModule = Settings.getValueAsInt(Repository.settings.stModule);

                Messages.setMessage(Resources.getText(DCMessages.msgExportStarts), 
                                    Messages._STANDARD);
                MessagesWriter._useLogFile    = false;

                Thread  exportCDThread = null;
                if (iModule == 0) {
                    exportCDThread  = new Thread(new ExportSoftwareCD(location, bVerify, getExportFile(), cdName, bToFile, bToTable), "ExportCD-Prio5");
                } else if (iModule == 1) {
                    exportCDThread  = new Thread(new ExportMP3CD(location, bVerify, getExportFile(), cdName, bToFile, bToTable), "ExportCD-Prio5");
                } 
                
                exportCDThread.setPriority(1);
                exportCDThread.setDaemon(true);
                exportCDThread.start();
                MessagesWriter._windowToFront = true;
                MessagesWriter._windowShow    = true;
            }
        }
    }

    public void fillFileComboBox() {
        Object o = comboLoc.getSelectedItem();
        comboLoc.removeAllItems();

        CdRom cdRom = new CdRom();
        String[] driveLetters = cdRom.getDriveLetters();

        for (int i = 0; i < driveLetters.length; i++) {
            if (driveLetters[i] != null) {
                comboLoc.addItem(driveLetters[i]);
            }
        }
        comboLoc.setSelectedItem(o);
    }

    public void switchModule() {
        setValues();    
    }

    private void setValues() {
        String  sourceFile = "";
        String  targetFile = "";

        int iModule = Settings.getValueAsInt(Repository.settings.stModule);
        if (iModule == 0) {
            sourceFile = Settings.getValueAsString(Repository.settings.stSoftImportFile);
            targetFile = Settings.getValueAsString(Repository.settings.stSoftExportFile);
        } else if (iModule == 1) {
            sourceFile = Settings.getValueAsString(Repository.settings.stMP3ImportFile);
            targetFile = Settings.getValueAsString(Repository.settings.stMP3ExportFile);
        } else if (iModule == 2) {
            sourceFile = Settings.getValueAsString(Repository.settings.stMovieImportFile);
            targetFile = Settings.getValueAsString(Repository.settings.stMovieExportFile);
        } 

        try {
            setImportFile(new File(targetFile.trim()));
        } catch (Exception ignore) {}

        try {
            setExportFile(new File(sourceFile.trim()));
        } catch (Exception ignore) {}

        setTableMode(true);
        fillTable();
    }

    protected void fillTable() {
        clearTable();
        String[][]  data    = null;

        if (!getImportFile().equals(new String(""))) {
            ImportInformationFile         iif     = new ImportInformationFile(getImportFile());
            if (iif != null) {
                data = iif.getIIFValues();
            }

            if (data == null) {
                return;
            } else if (data.length <= 0) {
                return;
            }

            for (int i = 0; i < data.length; i++) {
                String[] temp = new String[tableModel.getColumnCount()];
                temp[0] = data[i][0];
                temp[1] = Resources.getText(DCLabels.lblEndsWith);
                temp[2] = data[i][1];

                tableModel.addRow(temp);
            }
        }
    }

    private String getExportFile() {
        if (textExportFile.getText() == null) {
            return "";
        }

        return textExportFile.getText().trim();
    }

    private String getImportFile() {
        try {
            String text = textImportFile.getText().trim();
            File file   = new File(text);
            if (file.exists()) {
                return text;
            } else {
                return "";
            }
        } catch (Exception exp) {}
        return "";
    }

    private class ImportFileButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (ExportSoftwareCD.running == false && ImporList.running == false) {
                importFile();
            } else {
                new MessageBox( DataCrow.mainFrame,
                                Resources.getText(DCMessages.msgProcessRunsAlready), 
                                true, MessageBox._INFORMATION);
            }
        }
    }

    protected void showFileSaveDialog() {
        BrowserDialog textFile = new BrowserDialog(Resources.getText(
                                     DCLabels.lblFileBrowser));
        File test = textFile.showCreateFileDialog(this);

        if (test != null) {
            try {
                setExportFile(test);
                RandomAccessFile random = new RandomAccessFile(test, "rw");
                random.close();
            } catch (Exception exp) {
                new MessageBox( DataCrow.mainFrame,
                                Resources.getText(DCMessages.msgFileCouldNotbeCreated), 
                                true, MessageBox._ERROR);
                Messages.setMessage(
                    Resources.getText(DCMessages.msgFileCouldNotbeCreated), 
                    Messages._ERROR);
                Messages.setMessage(exp.getMessage(), Messages._ERROR);
            }
        }
    }

    private String[][] getAllTableValues() {
        String[][] values   = new String[table.getRowCount()][table.getColumnCount() - 1];
        for (int i = 0; i < table.getRowCount(); i++) {
            for (int j = 0; j < table.getColumnCount(); j++) {
                // Skip the endswith column
                if (j != 1) {
                    Object oValue = table.getValueAt(i, j);
                    if (oValue != null) {
                        String sValue = oValue.toString();
                        if (!sValue.equals(new String(""))) {
                            if (j > 1) {
                                values[i][j-1] = sValue;
                            } else {
                                values[i][j] = sValue;
                            }
                        }
                    }
                }
            }
        }
        return cleanUp(values);
    }

    protected void saveAllTableValues() {
        String      impFile = getImportFile();
        impFile     = impFile.trim();

        if (impFile.equals(new String(""))) {
            new MessageBox( DataCrow.mainFrame, 
                            Resources.getText(DCMessages.msgIIFSelectImpFile), 
                            true, MessageBox._NOTIFICATION);
            return;
        } else {
            File test   = new File(impFile.trim());
            if (!test.exists()) {
                new MessageBox(DataCrow.mainFrame,
                               Resources.getText(DCMessages.msgIIFSelectImpFile), 
                               true, MessageBox._NOTIFICATION);
                return;
            }
        }
        String[][] values = getAllTableValues();

        new ImportInformationFile(values, impFile);

        new MessageBox( DataCrow.mainFrame,
                        Resources.getText(DCMessages.msgDataSaved), 
                        true, MessageBox._INFORMATION);
    }

    protected void showFileOpenDialog() {
        BrowserDialog file = new BrowserDialog(
                                 Resources.getText(DCLabels.lblFileBrowser));
        File test = file.showOpenFileDialog(this, null);
        setImportFile(test);
    }

    private void setExportFile(File exportFile) {
        if (exportFile != null) {
            String sFileName = exportFile.toString();
            textExportFile.setText(sFileName);

            int IModule = Settings.getValueAsInt(Repository.settings.stModule);
            if (IModule == 0) {
                Settings.setValue(Repository.settings.stSoftExportFile, sFileName);
            } else if (IModule == 1) {
                Settings.setValue(Repository.settings.stMP3ExportFile, sFileName);
            } else if (IModule == 2) {
                Settings.setValue(Repository.settings.stMovieExportFile, sFileName);
            }
        }
    }

    private boolean validateImportFile(File importFile) {
        String sModule = "";

        if (importFile == null) {
            return false;
        } else {
            String sFileName = importFile.toString();
            if (sFileName.trim().equals("")) {
                return false;
            }
        }

        int iModule = Settings.getValueAsInt(Repository.settings.stModule);

        try {
            RandomAccessFile access = new RandomAccessFile(importFile, "r");
            sModule = access.readLine().trim();

            if (iModule == 0 &&
                !sModule.equals("->" + Repository.dataBase.dBSoftTableName.trim() + "<-")) {
                new MessageBox(DataCrow.mainFrame,
                               Resources.getText(DCMessages.msgInvalidSoftwareImportFile),
                               true, MessageBox._ERROR);
                return false;
            } else if (iModule == 1 &&
                !sModule.equals("->" + Repository.dataBase.dBMP3TableName.trim() + "<-")) {
                new MessageBox(DataCrow.mainFrame,
                               Resources.getText(DCMessages.msgInvalidMP3ImportFile),
                               true, MessageBox._ERROR);
                return false;
            } 
        } catch (Exception exp) {
            return false;
        }
        return true;
    }

    private void setImportFile(File importFile) {
        int iModule = Settings.getValueAsInt(Repository.settings.stModule);

        if (!validateImportFile(importFile)) {
            textImportFile.setText("");

            if (iModule == 0) {
                Settings.setValue(Repository.settings.stSoftImportFile, "");
            } else if (iModule == 1) {
                Settings.setValue(Repository.settings.stMP3ImportFile, "");
            } 

            clearTable();
            return;
        }

        textImportFile.setText(importFile.toString());

        fillTable();
        setTableMode(true);

        if (iModule == 0) {
            Settings.setValue(Repository.settings.stSoftImportFile,
                              textImportFile.getText());
        } else if (iModule == 1) {
            Settings.setValue(Repository.settings.stMP3ImportFile,
                              textImportFile.getText());
        } 

        Requestor req = new Requestor(DataExchangeSourceEditor.class,
                                      "setSourceFile",
                                      null);
        RequestorCollection reqCol = new RequestorCollection();
        reqCol.addRequestor(req);
        wf.displayResults(new Vector(), reqCol);
    }

    private void setEndsWith() {
        int row                  = table.getSelectedRow();
        boolean columHORIZONTALFilled  = false;
        boolean columnTwoFilled  = false;

        if (row < 2) {
            // No Row selected, do nothing.
            return;
        }

        table.getModel().removeTableModelListener(this);

        Object oValue = table.getValueAt(row, 0);
        if (oValue != null &&
            !oValue.toString().trim().equals(new String(""))) {
            columHORIZONTALFilled = true;
        }

        oValue = table.getValueAt(row, 2);
        if (oValue != null &&
            !oValue.toString().trim().equals(new String(""))) {
            columnTwoFilled = true;
        }

        if (columHORIZONTALFilled && columnTwoFilled) {
           table.setValueAt(Resources.getText(DCLabels.lblEndsWith), row, 1);
        } else {
           table.setValueAt("", row, 1);
        }

        table.getModel().addTableModelListener(this);
    }

    public void tableChanged(TableModelEvent e) {
        setEndsWith();
    }

    private void setTableMode(boolean editable) {
        table.getModel().addTableModelListener(this);
        JTextField text = ComponentFactory.getTextField();
        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
        Color color     = null;
        if (editable == true) {
            color = renderer.getBackground();
        } else {
            color = ComponentFactory.getDisabledColor();
        }

        text.setEditable(editable);
        TableColumn column      = table.getColumnModel().getColumn(0);
        column.setCellEditor(new DefaultCellEditor(ComponentFactory.getFieldsComboBox()));
        renderer.setBackground(color);
        column.setCellRenderer(renderer);

        // Ends With column
        text.setEditable(false);
        renderer                = new DefaultTableCellRenderer();
        column                  = table.getColumnModel().getColumn(1);
        renderer.setBackground(new Color(204,204,204));
        column.setCellEditor(new DefaultCellEditor(text));
        column.setCellRenderer(renderer);

        text.setEditable(editable);
        renderer                = new DefaultTableCellRenderer();
        renderer.setBackground(color);
        column                  = table.getColumnModel().getColumn(2);
        column.setCellEditor(new DefaultCellEditor(text));
        column.setCellRenderer(renderer);
    }

    private String[][] cleanUp(String[][] data) {
        int counter1 = 0;
        int counter2 = 0;
        int counter3 = 0;
        String[][] newData  = null;
        try {
            for (int i = 0; i < data.length; i++) {
                counter2 = 0;
                for (int j = 0; j < data[i].length; j++) {
                    if (data[i][j] != null) {
                        if (counter2 == 0) {
                            counter1++;
                        }
                        counter2++;
                        if (counter2 != 0 && counter2 > counter3) {
                            counter3 = counter2;
                        }
                    }
                }
            }

            newData = new String[counter1][counter3];

            for (int k = 0; k < newData.length; k++) {
                for (int l = 0; l < newData[k].length; l++) {
                    newData[k][l] = data[k][l];
                }
            }
        } catch (Exception exp) {
            return data;
        }
        return newData;
    }

    private void buildPanel() {
        panelImportSettings = new JPanel(false);
        panelImportFile     = new JPanel(false);
        panelExportCD       = new JPanel(false);
        panelTabAct         = new JPanel(false);
        panelExportActions  = new JPanel(false);
        panelImportActions  = new JPanel(false);


        // ************************************************************************
        // EXPORT CD PANEL
        // Set layout style
        GridBagLayout       gridbExportCD   = new GridBagLayout();
        panelExportCD.setLayout(gridbExportCD);

        // Initialize components
        labelLocation           = ComponentFactory.getLabel(Resources.getText(DCLabels.lblCDLocation));
        comboLoc                = ComponentFactory.getComboBox();
        buttonExpFile           = ComponentFactory.getButton("..");
        checkVerify             = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblAskPerDir));
        labelFile               = ComponentFactory.getLabel(Resources.getText(DCLabels.lblToExpFile));
        textExportFile          = ComponentFactory.getTextField();
        buttonFilChoose         = ComponentFactory.getButton("..");
        labelCDName             = ComponentFactory.getLabel(Resources.getText(DCLabels.lblLocationName));
        textCDName              = ComponentFactory.getTextField();
        checkToFile             = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblExportToFile));
        checkToTable            = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblExportToTable));

        buttonExpFile.addActionListener(new SelectALocation());
        textExportFile.setEditable(false);
        textExportFile.setBackground(new Color(204,204,204));
        comboLoc.addActionListener(new RefreshFileCombo());

        fillFileComboBox();

        // Build the panel
        panelExportCD.add(  labelLocation,  new GridBagConstraints( 0, 0, 1, 1, 30.0, 30.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 5, 0, 10), 0, 0));
        panelExportCD.add(  comboLoc,       new GridBagConstraints( 1, 0, 2, 1, 50.0, 50.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 10, 0, 5), 0, 0));
        panelExportCD.add(  labelFile     , new GridBagConstraints( 0, 1, 1, 1, 30.0, 30.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 5, 0, 10), 0, 0));
        panelExportCD.add(  textExportFile, new GridBagConstraints( 1, 1, 1, 1, 50.0, 50.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 10, 0,  0), 0, 0));
        panelExportCD.add(  buttonExpFile,  new GridBagConstraints( 2, 1, 1, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                            new Insets( 0,  0, 0, 5), 0, 0));
        panelExportCD.add(  labelCDName,    new GridBagConstraints( 0, 2, 1, 1, 30.0, 30.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 5, 5, 10), 0, 0));
        panelExportCD.add(  textCDName,     new GridBagConstraints( 1, 2, 2, 1, 50.0, 50.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 10, 5, 5), 0, 0));
        panelExportCD.add(  checkVerify,    new GridBagConstraints( 0, 3, 1, 1, 50.0, 50.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 5, 0, 0), 0, 0));
        panelExportCD.add(  checkToFile,    new GridBagConstraints( 0, 4, 1, 1, 50.0, 50.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 5, 0, 0), 0, 0));
        panelExportCD.add(  checkToTable,   new GridBagConstraints( 0, 5, 1, 1, 50.0, 50.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets( 0, 5, 0, 0), 0, 0));


        // ************************************************************************
        // EXPORT FILE ACTION PANEL
        // Set layout style
        GridBagLayout       gridbExpAct = new GridBagLayout();
        panelExportActions.setLayout(gridbExpAct);

        // Initilialize components
        buttonCDExport      = ComponentFactory.getButton(Resources.getText(DCLabels.lblStart));
        buttonCDExport.addActionListener(new ExportCDButtonAction());

        // Build the panel
        panelExportActions.add(buttonCDExport,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                                new Insets( 0, 10, 5, 10), 0, 0));


        // ************************************************************************
        // IMPORT FILE PANEL
        // Set layout style
        GridBagLayout       gridbImportFile = new GridBagLayout();
        panelImportFile.setLayout(gridbImportFile);

        // Initialize components
        labelSource             = ComponentFactory.getLabel(Resources.getText(DCLabels.lblSourceFile));
        textImportFile          = ComponentFactory.getTextField();
        buttonImpFile           = ComponentFactory.getButton("..");

        textImportFile.setEditable(false);
        textImportFile.setBackground(new Color(204,204,204));

        buttonImpFile.addActionListener(new SelectAFile());
        labelSource.setToolTipText(Resources.getText(DCTooltips.tpSourceFile));

        // Build the panel
        panelImportFile.add(  labelSource,    new GridBagConstraints( 0, 0, 1, 1, 30.0, 30.0
                                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                              new Insets( 0, 5, 5, 10), 0, 0));
        panelImportFile.add(  textImportFile, new GridBagConstraints( 1, 0, 1, 1, 50.0, 50.0
                                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                              new Insets( 0, 37, 5,0), 0, 0));
        panelImportFile.add(  buttonImpFile,  new GridBagConstraints( 2, 0, 1, 1, 1.0, 1.0
                                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                              new Insets( 0,  0, 5, 5), 0, 0));


        // ************************************************************************
        // IMPORT FILE ACTION PANEL
        // Set layout style
        GridBagLayout       gridbImpAct = new GridBagLayout();
        panelImportActions.setLayout(gridbImpAct);

        // Initialize components
        buttonFileImport    = ComponentFactory.getButton(Resources.getText(DCLabels.lblStart));
        buttonFileImport.addActionListener(new ImportFileButtonAction());

        // Build the panel
        panelImportActions.add( buttonFileImport,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                                   new Insets( 0, 10, 5, 10), 0, 0));


        // ************************************************************************
        // IMPORT SETTINGS PANEL
        // Set layout style
        GridBagLayout       gridbImportSettings = new GridBagLayout();
        panelImportSettings.setLayout(gridbImportSettings);

        // Initialize components
        tableModel.setColumnCount(3);
        tableModel.setRowCount(15);
        table   = ComponentFactory.getSimpleTable(tableModel, false);
        JScrollPane scrollSettings = new JScrollPane(table);
        scrollSettings.setVerticalScrollBarPolicy(scrollSettings.VERTICAL_SCROLLBAR_ALWAYS);


        TableColumn column  = table.getColumnModel().getColumn(0);
        column.setHeaderRenderer(ComponentFactory.getTableHeaderRenderer());
        column.setHeaderValue(Resources.getText(DCLabels.lblFields));

        column  = table.getColumnModel().getColumn(1);
        column.setHeaderValue("");
        column.setHeaderRenderer(ComponentFactory.getTableHeaderRenderer());

        column  = table.getColumnModel().getColumn(2);
        column.setHeaderRenderer(ComponentFactory.getTableHeaderRenderer());
        column.setHeaderValue(Resources.getText(DCLabels.lblSeperator));

        table.getModel().addTableModelListener(this);

        // Build the panel
        panelImportSettings.add( scrollSettings,  new GridBagConstraints( 0, 0, 1, 1, 10.0, 10.0
                                                 ,GridBagConstraints.SOUTHWEST, GridBagConstraints.BOTH,
                                                  new Insets( 0, 5, 5, 5), 0, 0));


        // ************************************************************************
        // TABLE ACTION PANEL
        // Set layout style
        GridBagLayout       gridbTabAct = new GridBagLayout();
        panelTabAct.setLayout(gridbTabAct);

        // Initialize Components
        buttonSaveTable = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave), 100);
        buttonLoadTable = ComponentFactory.getButton(Resources.getText(DCLabels.lblLoad), 100);
        buttonRemoveRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblRemoveRow), 100);
        buttonAddRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblAddRow), 100);

        buttonSaveTable.addActionListener(new SaveTableButtonAction());
        buttonLoadTable.addActionListener(new LoadTableButtonAction());
        buttonAddRow.addActionListener(new AddRowAction());
        buttonRemoveRow.addActionListener(new RemoveRowButtonAction());

        // Build the panel
        panelTabAct.add( buttonSaveTable,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                           new Insets( 0, 10, 0, 5), 0, 0));
        panelTabAct.add( buttonLoadTable,  new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                           new Insets( 0, 0, 0, 5), 0, 0));
        panelTabAct.add( buttonRemoveRow,  new GridBagConstraints( 2, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                           new Insets( 0, 0, 0, 5), 0, 0));
        panelTabAct.add( buttonAddRow,     new GridBagConstraints( 3, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                           new Insets( 0, 0, 0, 10), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbag         = new GridBagLayout();
        setLayout(gridbag);

        // Initialize components
        panelImportSettings.setBorder(ComponentFactory.getBorder(Resources.getText(DCLabels.lblIIFData)));
        panelImportFile.setBorder(ComponentFactory.getBorder(Resources.getText(DCLabels.lblImportaFile)));
        panelExportCD.setBorder(ComponentFactory.getBorder(Resources.getText(DCLabels.lblExportaCD)));

        // Build the panel
        add( panelExportCD,         new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelExportActions,    new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelImportFile,       new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelImportActions,    new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelImportSettings,   new GridBagConstraints( 1, 0, 1, 3, 10.0, 10.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelTabAct,           new GridBagConstraints( 1, 3, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));

        setValues();
    }


    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************

    private class SaveTableButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveAllTableValues();
        }
    }

    private class SelectAFile implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showFileOpenDialog();
        }
    }

    private class SelectALocation implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showFileSaveDialog();
        }
    }

    private class LoadTableButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            fillTable();
        }
    }
    
    public class AddRowAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.addRow(new Object[] {null, null, null});
        }
    }    

    public class RefreshFileCombo implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            fillFileComboBox();
        }
    }

    private class ExportCDButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (ExportSoftwareCD.running == false && ImporList.running == false) {
                startCDExport();
            } else {
                new MessageBox(DataCrow.mainFrame,
                               Resources.getText(DCMessages.msgProcessRunsAlready), 
                               true, 
                               MessageBox._INFORMATION);
            }
        }
    }
}
