/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.da;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.RandomAccessFile;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import rjw.dc.console.MessageBox;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.DataCrowPanel;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DataExchangeSourceEditor extends DataCrowPanel {

    private JPanel panelResult = new JPanel(false);
    private JPanel panelTableActions = new JPanel(false);
    
    private String sSourceFile = "";

    private JButton buttonSave;
    private JTextArea textArea;

    public DataExchangeSourceEditor() {
        buildPanel();
    }

    public void setSourceFile() {
        textArea.setText("");
        sSourceFile = "";
        int iModule = Settings.getValueAsInt(Repository.settings.stModule);

        if (iModule == 0) {
            sSourceFile = Settings.getValueAsString(Repository.settings.stSoftImportFile);
        } else if (iModule == 1) {
            sSourceFile = Settings.getValueAsString(Repository.settings.stMP3ImportFile);
        } else if (iModule == 2) {
        }

        RandomAccessFile access = null;
        try {
            access = new RandomAccessFile(sSourceFile, "rw");
            long filePointer = 0;
            long fileLength = access.length();

            StringBuffer sb = new StringBuffer("");
            while (filePointer < fileLength) {
                sb.append(access.readLine().trim() + '\r' + '\n');
                filePointer = access.getFilePointer();
            }

            textArea.setText(sb.toString());

            access.close();   
        } catch (Exception ignore) {
        }
    }

    public void switchModule() {
        setSourceFile();
    }

    public void buildPanel() {

       // ************************************************************************
        // RESULT PANEL
        // Set layout style
        panelResult.setLayout(ComponentFactory.getLayoutManager());

        // Initialize components
        textArea = ComponentFactory.getTextArea();
        textArea.setBackground(ComponentFactory.getTableColor());

        JScrollPane scroll = new JScrollPane(textArea);
        scroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelResult.add(    scroll,  new GridBagConstraints( 0, 0, 1, 1, 10.0, 10.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(0, 5, 5, 5), 0, 0));


       // ************************************************************************
        // TABLE ACTIONS PANEL
        // Set layout style
        GridBagLayout gridbTabAct = new GridBagLayout();
        panelTableActions.setLayout(gridbTabAct);

        buttonSave = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave));
        buttonSave.addActionListener(new SaveSourceFile());

        // Build the panel
        panelTableActions.add(buttonSave, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets(0, 10, 0, 5), 0, 0));

        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        panelResult.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblSourceFileEditor)));

        setLayout(ComponentFactory.getLayoutManager());
        add(panelResult,            new GridBagConstraints( 0, 0, 3, 1, 10.0, 10.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add(panelTableActions,      new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
    }
    
    private class SaveSourceFile implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            try {
                File file = new File(sSourceFile);
                file.delete();
                file.createNewFile();
                RandomAccessFile access = new RandomAccessFile(file, "rw");
                
                access.writeBytes(textArea.getText());
            } catch (Exception exp) {
                new MessageBox(DataCrow.mainFrame,
                               exp.getLocalizedMessage(), 
                               true, MessageBox._ERROR);
            }
        }
        
    }
}
