package rjw.dc.console.movie;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

import rjw.dc.console.MessageBox;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.http.HttpConnection;
import rjw.dc.http.IMDBQuery;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

public class ImdbSearchForm extends JDialog {
    
    private final String sODB       = Settings.getValueAsString(
                                      Repository.settings.stODB);
    private final String sProxy     = Settings.getValueAsString(
                                      Repository.settings.stProxyServerName);
    private final int iProxyPort    = Settings.getValueAsInt(
                                      Repository.settings.stProxyServerPort);
    private final String sUserAgent = Settings.getValueAsString(
                                      Repository.settings.stHTTPUserAgent);
    private boolean overwrite       = Settings.getValueAsBoolean(
                                      Repository.settings.stIMDBOverwriteExistingValues);
    
    private JTextField textSearch;
    private JCheckBox checkOverwrite;
    
    private Thread threadSearch;
    
    private JProgressBar progress;
    private JButton buttonSearch;
    private JButton buttonStop;  
    
    private boolean running = false;
    
    private DefaultTableModel tableModel = new DefaultTableModel();
    private JTable tableResult;
    private boolean update = true;
    private int rowNumber = -1;
    
    private boolean isUpdateRunning = false;
    
    public ImdbSearchForm(int rowNumber, String title, String name, boolean update) {
        super(DataCrow.mainFrame);
        super.setTitle(title);
        
        isUpdateRunning = true;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                running = false;
            }
        });        
        
        buildDialog();
        this.update = update;
        this.rowNumber = rowNumber;
        textSearch.setText(name);
        searchImdb();
        setVisible(true);
        setModal(true);
    }

    public void stop() {
        Settings.setValue(Repository.settings.stIMDBOverwriteExistingValues,
                          new Boolean(checkOverwrite.isSelected()));
        Settings.setValue(Repository.settings.stImdbSearchFormViewSize,
                          getSize());
        isUpdateRunning = false;

        setVisible(false);
        running = false;
        progress.setValue(0);
        dispose();
    }

    public boolean isUpdateRunning() {
        return isUpdateRunning;   
    } 

    public void setData(Object[] data) {
        tableModel.addRow(data);
    }
    
    private void searchImdb() {
        String name = textSearch.getText();
        threadSearch = new Thread(new SearchJob(name), "SearchJob");
        threadSearch.setDaemon(true);
        threadSearch.setPriority(1);
        threadSearch.start();        
    }
    
    protected void transferToInsertTable() {
        try {
            Class clazz = null;
            if (update) {
                clazz = SearchMoviePanel.class;
            } else {
                clazz = InsertMoviePanel.class;   
            }
            Object[] row = getSelectedData();
    
            if (row == null) {
                return;
            }
    
            Object[] params = {new Integer(rowNumber), 
                               row, 
                               new Boolean(checkOverwrite.isSelected())};
                               
            Requestor request = new Requestor(clazz, 
                                              "updateRowAt", 
                                              params);
                                              
            RequestorCollection requestors = new RequestorCollection();
            requestors.addRequestor(request);
            WorkFlow wf = new WorkFlow();
            wf.displayResults(new Vector(), requestors);
        } finally {
            stop();
        }
    }
    
    public Object[] getSelectedData() {
        int row = tableResult.getSelectedRow();
        
        if (row < 0) {
            new MessageBox(DataCrow.mainFrame,
                           Resources.getText(DCMessages.msgSelectRow),
                           true, MessageBox._NOTIFICATION);
            return null;
        }
        
        Object[] o = new Object[tableResult.getColumnCount()];
        for (int i = 0; i < o.length; i++) {
            o[i] = tableResult.getValueAt(row, i);
        }
        return o;
    }
    
    public void buildDialog() {
        Color color = Settings.getValueAsColor(Repository.settings.stColor);
        this.getContentPane().setBackground(color);
        JPanel actionPanel = new JPanel(false);
        actionPanel.setBackground(color);
        JPanel resultPanel = new JPanel(false);
        resultPanel.setBackground(color);        
        JPanel searchPanel = new JPanel(false);
        searchPanel.setBackground(color);
        JPanel progressPanel = new JPanel(false);
        progressPanel.setBackground(color);
        
        // The search part
        textSearch = ComponentFactory.getTextField();
        buttonSearch =  ComponentFactory.getButton(
                        Resources.getText(DCLabels.lblSearch));
        buttonSearch.addActionListener(new SearchAction());
        
        searchPanel.setLayout(new GridBagLayout());                                
        searchPanel.add(textSearch,   new GridBagConstraints( 0, 0, 1, 1, 90.0, 90.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
        searchPanel.add(buttonSearch, new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));

        // Action panel
        JButton buttonTransfer = ComponentFactory.getButton(
                                 Resources.getText(DCLabels.lblTransfer));
        buttonStop = ComponentFactory.getButton(
                     Resources.getText(DCLabels.lblStop));
        JButton buttonClose = ComponentFactory.getButton(
                              Resources.getText(DCLabels.lblClose));
        
        buttonTransfer.addActionListener(new TransferRow());
        buttonClose.addActionListener(new CloseWindow());
        buttonStop.addActionListener(new StopAction());

        actionPanel.add(buttonTransfer);
        actionPanel.add(buttonStop);
        actionPanel.add(buttonClose);

        // Result panel
        // get the default movie table and remove the unneeded rows (keep, location and such)
        tableResult = ComponentFactory.getMovieTable(tableModel, true);
        tableResult.getColumnModel().removeColumn(tableResult.getColumnModel().getColumn(12));
        tableResult.getColumnModel().removeColumn(tableResult.getColumnModel().getColumn(11));
        tableResult.getColumnModel().removeColumn(tableResult.getColumnModel().getColumn(10));
        tableResult.getColumnModel().removeColumn(tableResult.getColumnModel().getColumn(0));

        JScrollPane scrollResults = new JScrollPane(tableResult);
        scrollResults.setVerticalScrollBarPolicy(scrollResults.VERTICAL_SCROLLBAR_ALWAYS);        

        checkOverwrite = ComponentFactory.getCheckBox(
                         Resources.getText(DCLabels.lblOverwriteExistingValues));
        checkOverwrite.setToolTipText(
                       Resources.getText(DCTooltips.tpOverwriteExistingValues));

        resultPanel.setLayout(new GridBagLayout());
        resultPanel.add(scrollResults,    new GridBagConstraints( 0, 0, 1, 1, 90.0, 90.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                          new Insets(5, 5, 5, 5), 0, 0));   
        resultPanel.add(checkOverwrite,   new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                          new Insets(5, 5, 5, 5), 0, 0));   

        // Progress panel
        progress = new JProgressBar();
        progress.setIndeterminate(true);
        progressPanel.setLayout(new GridBagLayout());
        progressPanel.add(progress, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));

        // Main panel                                          
        this.getContentPane().setLayout(new GridBagLayout());
        this.getContentPane().add(searchPanel, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        this.getContentPane().add(resultPanel, new GridBagConstraints( 0, 1, 1, 1, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        this.getContentPane().add(actionPanel, new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        this.getContentPane().add(progressPanel,new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0
                             ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        
        checkOverwrite.setSelected(overwrite);
        this.setResizable(true);
        this.pack(); 
        buttonStop.setEnabled(false);
        this.setSize(Settings.getValueAsDimension(Repository.settings.stImdbSearchFormViewSize));
    }

    private class SearchAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            searchImdb();   
        }
    }
    
    private class TransferRow implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            transferToInsertTable();   
        }
    }

    private class CloseWindow implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            stop();
        }
    }
    
    private class StopAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            running = false;
            buttonSearch.setEnabled(true);
            buttonStop.setEnabled(false);
        }
    }    
    
    private class SearchJob extends Thread {
        
        String name = "";
        
        public SearchJob(String name) {
            this.name = name;
        }
        
        public void run() {
            try {
                running = true;
                buttonSearch.setEnabled(false);
                buttonStop.setEnabled(true);
                searchImdb(name);
            } catch (Exception exp) {
                exp.printStackTrace();
                Messages.setMessage(exp.toString(), Messages._ERROR);
                new MessageBox(DataCrow.mainFrame,
                               Resources.getText(DCMessages.msgProcessCrash), 
                               true, MessageBox._ERROR);
            }
        }      

        private void searchImdb(String name) {
            String sHost = Settings.getValueAsString(Repository.settings.stODB);
            HttpConnection connection = null;
            
            try {
                connection = new HttpConnection(sHost);
            } catch (Exception exp) {
                new MessageBox(DataCrow.mainFrame, exp.getMessage(), true, MessageBox._ERROR);
                Messages.setMessage("<HTTP> " + exp.getMessage(), Messages._ERROR);
                return;
            }
            
            try {
                IMDBQuery imdbQuery = new IMDBQuery(name);
                String[] ids = imdbQuery.getIDs(connection);
                progress.setMaximum(ids.length);
                progress.setValue(0);
                progress.setIndeterminate(false);
                
                for (int i = 0; i < ids.length; i++) {
                    if (!running) {
                        return;   
                    }
                    
                    imdbQuery.fillImdbData(ids[i], connection);
                    Object[] row = new Object[10];
                    
                    int j = 0;
                    row[j++] = new Integer(rowNumber);
                    row[j++] = imdbQuery.getValue(imdbQuery._TITLE);
                    row[j++] = imdbQuery.getValue(imdbQuery._PLOT);
                    row[j++] = imdbQuery.getValue(imdbQuery._URL);
                    row[j++] = imdbQuery.getValue(imdbQuery._GENRE);
                    row[j++] = imdbQuery.getValue(imdbQuery._ACTORS);
                    row[j++] = imdbQuery.getValue(imdbQuery._DIRECTOR);
                    row[j++] = imdbQuery.getValue(imdbQuery._COUNTRY);
                    row[j++] = imdbQuery.getValue(imdbQuery._YEAR);
                    row[j++] = imdbQuery.getValue(imdbQuery._RUNTIME);

                    progress.setValue(i + 1);
                    setData(row);
                }
            } catch (Exception exp) {
                Messages.setMessage("<HTTP> " + exp.getMessage(), Messages._ERROR);
                new MessageBox(DataCrow.mainFrame, exp.getMessage(), true, MessageBox._ERROR);   
            } finally {
                buttonSearch.setEnabled(true);
                buttonStop.setEnabled(false);
                connection.close();     
                finishAll();       
            }
        }
        
        /**
         * Stops the process
         */
        private void finishAll() {
            try {
                Messages.setMessage(
                    Resources.getText(DCMessages.msgIMDBProcessStops), 
                    Messages._STANDARD);
            } catch (Exception ignore) {}
        }
    }
}
