/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.movie;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ToolTipManager;

import rjw.dc.console.MessageBox;
import rjw.dc.console.RecordForm;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.NumberField;
import rjw.dc.console.components.ShortTextField;
import rjw.dc.console.components.TablePanel;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.CdRom;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.db.QueryOptions;
import rjw.dc.processes.ImporList;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;

/**
 * Panel used for searching and updating existing Movie records. Searching
 * is done by using condition (title like <value>) and using options such as
 * all conditions must applsy or the seperate conditions must apply
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SearchMoviePanel extends TablePanel {

    private JLabel                  labelTitle;
    private ShortTextField          textTitle;
    private JLabel                  labelDescription;
    private JTextField              textDescription;
    private JLabel                  labelWebpage;
    private ShortTextField          textWebpage;
    private JLabel                  labelCatagory;
    private ShortTextField          textCatagory;
    private JLabel                  labelActors;
    private ShortTextField          textActors;
    private JLabel                  labelDirector;
    private ShortTextField          textDirector;
    private JLabel                  labelCountry;
    private ShortTextField          textCountry;
    private JLabel                  labelYear;
    private JComboBox               comboYear;
    private JLabel                  labelPlaylength;
    private NumberField             numberPlayLength;
    private JLabel                  labelFileName;
    private ShortTextField          textFileName;
    private JLabel                  labelLocation;
    private ShortTextField          textLocation;
    private JLabel                  labelKeep;
    private JComboBox               comboKeep;

    private JComboBox               comboAndOr;
    private JLabel                  labelAndOr;
    private JComboBox               comboPrecise;
    private JLabel                  labelPrecise;

    private JLabel                  labelOrderBy;
    private JComboBox               comboOrderBy;

    private JButton                 buttonSearch;
    private JButton                 buttonClear;
    private JButton                 buttonSave;
    private JButton                 buttonIMDB;
    private JButton                 buttonIMDBAll;

    private JPanel                  panelStatus;
    private JPanel                  panelInput;
    private JPanel                  panelResult;
    private JPanel                  panelTabAct;
    private JPanel                  panelSearchOpts;
    private JPanel                  panelSearchAct;

    private CdRom                   cdrom = new CdRom();
    private Requestor               requestor;

    /**
     * Creates and shows the panel
     */
    public SearchMoviePanel() {
        buildPanel();
    }

    /**
     * Searches for existing Movie records. The records are searched for by using
     * the conditions as filled in and the options.
     */
    public void searchValues() {
        Movie Avi = new Movie();
        Avi.setValues(getSearchValues());

        requestor = new Requestor(this.getClass(), "fillTable", null);
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestor);

        Avi.load(getQueryOptions(), requestors);
        setStatus(Resources.getText(DCMessages.msgSearchHasBeenExecuted));
    }

    /**
     * Creates an Array of Strings of the search values. The search values
     * are the conditions for a search action
     *
     * @return String[] array of search conditions
     */
    private Object[] getSearchValues() {
        Object[] searchValues = new Object[Repository.dataBase.dbMovieColArray.length];

        int i = 0;
        searchValues[i++] = textTitle.getText().trim();
        searchValues[i++] = textDescription.getText().trim();
        searchValues[i++] = textWebpage.getText().trim();
        searchValues[i++] = textCatagory.getText();
        searchValues[i++] = textActors.getText().trim();
        searchValues[i++] = textDirector.getText().trim();
        searchValues[i++] = textCountry.getText().trim();
        searchValues[i++] = comboYear.getSelectedItem().toString().trim();
        searchValues[i++] = numberPlayLength.getValue();
        searchValues[i++] = textFileName.getText().trim();
        searchValues[i++] = textLocation.getText().trim();
        searchValues[i++] = comboKeep.getSelectedItem().toString().trim();
        return searchValues;
    }

    /**
     * Returns the query options used for searching values
     *
     * @return QueryOptions
     */
    private QueryOptions getQueryOptions() {
        boolean bPreciseSelect = false;
        boolean bComplyToAllCondition = true;
        String  sYes = Resources.getText(DCLabels.lblYes);
        String  sOr = Resources.getText(DCLabels.lblOr);

        if (comboAndOr.getSelectedItem().equals(sOr)) {
            bComplyToAllCondition = false;
        }
        if (comboPrecise.getSelectedItem().equals(sYes)) {
            bPreciseSelect = true;
        }

        String sOrderBy = "";
        if (comboOrderBy.getSelectedItem() != null) {
            sOrderBy    = comboOrderBy.getSelectedItem().toString();
        }

        return new QueryOptions(sOrderBy, bComplyToAllCondition, bPreciseSelect);
    }

    /**
     * Saves all updated values to the database. The changed row numbers are
     * cached for easy retrieval of the changed records
     */
    public void saveUpdatedValues() {
        DataCrowCollection collection = new DataCrowCollection();

        Vector vValues = getChangedValues(Repository.dataBase.dbMovieColArrayPK);
        int i = 0;
        for (Enumeration enum = vValues.elements(); enum.hasMoreElements(); ) {
            int row = getRowNumberWithID((String) vChangedIDs.elementAt(i++));            
            Movie movie = new Movie();
            movie.setValues((String[]) enum.nextElement());
            collection.addObject(movie, new Integer(row));
        }
        collection.saveUpdate(new RequestorCollection());
        vChangedIDs.removeAllElements();              
    }

    /**
     * Creates a record form for the selected record
     */
    public void createRecordForm() {
        if (table.getRowCount() == 0) {
            return;   
        }        
        
        int columnCount = table.getColumnModel().getColumnCount();
        int row = table.getSelectedRow();
        Object[][] data = new Object[columnCount][4];

        Movie movie = new Movie();
        for (int i = 0; i < columnCount; i++) {
            data[i][0] = movie.getLabel(i);
            data[i][3] = new Integer(movie.getMaxFieldLength(i));
            if (i == 2 || i == 4 || i == 5) {
                data[i][1] = new Integer(ComponentFactory._LONGTEXTFIELD);                
            } else if (i ==  8) {
                data[i][1] = new Integer(ComponentFactory._YEARCOMBO);
            } else if (i == 12) {
                data[i][1] = new Integer(ComponentFactory._YESNOCOMBO);
            } else if (i == 9) {
                data[i][1] = new Integer(ComponentFactory._NUMBERFIELD);
            } else {
                data[i][1] = new Integer(ComponentFactory._SHORTTEXTFIELD);
            }

            if (table.getValueAt(row, i) == null) {
                return;
            } else {
                data[i][2] = table.getValueAt(row, i);
            }
        }

        recordForm  = new RecordForm(data,
                                     true,
                                     Resources.getText(DCLabels.lblRecord, table.getValueAt(row,0).toString()),
                                     table.getValueAt(row,0).toString());
        vRecordForms.add(recordForm);
    }

    /**
     * Initializes the panel
     */
    private void buildPanel() {
        // The sub panels
        panelStatus     = new JPanel(false);
        panelInput      = new JPanel(false);
        panelResult     = new JPanel(false);
        panelTabAct     = new JPanel(false);
        panelSearchOpts = new JPanel(false);
        panelSearchAct  = new JPanel(false);

        // ************************************************************************
        // INPUT PANEL
        // Set layout style
        GridBagLayout gridbInput        = new GridBagLayout();
        panelInput.setLayout(gridbInput);

        Movie movie = new Movie();
        // Initialize components
        labelTitle      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieTitle));
        textTitle       = ComponentFactory.getShortTextField(
                            movie.getMaxFieldLength(movie._A_TITLE));
        labelDescription= ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieDescription));
        textDescription = ComponentFactory.getTextField();
        labelWebpage    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblWebPage));
        textWebpage     = ComponentFactory.getShortTextField(movie.getMaxFieldLength(movie._C_WEBPAGE));
        labelCatagory   = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieGenre));
        textCatagory    = ComponentFactory.getShortTextField(movie._D_GENRE);
        labelActors     = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieActors));
        textActors      = ComponentFactory.getShortTextField(
                            movie.getMaxFieldLength(movie._E_ACTORS));
        labelDirector   = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieDirector));
        textDirector    = ComponentFactory.getShortTextField(
                            movie.getMaxFieldLength(movie._F_DIRECTOR));
        labelCountry    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieCountry));
        textCountry     = ComponentFactory.getShortTextField(
                            movie.getMaxFieldLength(movie._G_COUNTRY));
        labelYear       = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieYear));
        comboYear       = ComponentFactory.getYearComboBox();
        labelPlaylength = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMoviePlayLength));
        numberPlayLength  = ComponentFactory.getNumberField();
        labelFileName   = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieFileName));
        textFileName    = ComponentFactory.getShortTextField(
                            movie.getMaxFieldLength(movie._J_FILENAME));
        labelLocation   = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieLocation));
        textLocation    = ComponentFactory.getShortTextField(
                            movie.getMaxFieldLength(movie._K_LOCATION));
        labelKeep       = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovieKeep));
        comboKeep       = ComponentFactory.getYesNoCombo();

        // Build the panel
        panelInput.add( labelTitle,             new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textTitle,              new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelDescription,       new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textDescription,        new GridBagConstraints( 1, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelWebpage,           new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textWebpage,            new GridBagConstraints( 1, 2, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelCatagory,          new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textCatagory,           new GridBagConstraints( 1, 3, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelActors,            new GridBagConstraints( 0, 4, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textActors,             new GridBagConstraints( 1, 4, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelDirector,          new GridBagConstraints( 0, 5, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textDirector,           new GridBagConstraints( 1, 5, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelCountry,           new GridBagConstraints( 0, 6, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textCountry,            new GridBagConstraints( 1, 6, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelYear,              new GridBagConstraints( 0, 7, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( comboYear,              new GridBagConstraints( 1, 7, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelPlaylength,        new GridBagConstraints( 0, 8, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( numberPlayLength,       new GridBagConstraints( 1, 8, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelFileName,          new GridBagConstraints( 0, 9, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textFileName,           new GridBagConstraints( 1, 9, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelLocation,          new GridBagConstraints( 0,10, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textLocation,           new GridBagConstraints( 1,10, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelKeep,              new GridBagConstraints( 0,11, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( comboKeep,              new GridBagConstraints( 1,11, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 5, 5), 0, 0));


        // ************************************************************************
        // SEARCH OPTION PANEL
        // Set layout style
        GridBagLayout gridbSearchOpts   = new GridBagLayout();
        panelSearchOpts.setLayout(gridbSearchOpts);

        // Initialize components
        comboAndOr      = ComponentFactory.getComboBox();
        labelAndOr      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblAndOrSelect));
        comboPrecise    = ComponentFactory.getRealYesNoComboBox();
        labelPrecise    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblPreciseSelect));
        comboOrderBy    = ComponentFactory.getComboBox();
        labelOrderBy    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblOrderBy));

        String[]    colNames    = Repository.dataBase.dbMovieColArray;
        comboOrderBy.addItem("");

        for (int i = 0; i < colNames.length; i++) {
            comboOrderBy.addItem(colNames[i]);
        }

        comboAndOr.addItem(Resources.getText(DCLabels.lblAnd));
        comboAndOr.addItem(Resources.getText(DCLabels.lblOr));

        // Build the panel
        panelSearchOpts.add( labelAndOr,         new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 10), 0, 0));
        panelSearchOpts.add( comboAndOr,         new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 10, 0, 5), 0, 0));
        panelSearchOpts.add( labelPrecise,       new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 10), 0, 0));
        panelSearchOpts.add( comboPrecise,       new GridBagConstraints( 1, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 10, 0, 5), 0, 0));
        panelSearchOpts.add( labelOrderBy,       new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 10), 0, 0));
        panelSearchOpts.add( comboOrderBy,       new GridBagConstraints( 1, 2, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 10, 5, 5), 0, 0));

        // ************************************************************************
        // SEARCH ACTION PANEL
        // Set layout style
        GridBagLayout gridbSearchAct        = new GridBagLayout();
        panelSearchAct.setLayout(gridbSearchAct);

        buttonSearch = ComponentFactory.getButton(Resources.getText(DCLabels.lblSearch));
        buttonSearch.setToolTipText(Resources.getText(DCTooltips.tpSearch));
        buttonSearch.addActionListener(new SearchButtonAction());

        panelSearchAct.add(  buttonSearch,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets( 5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // RESULT PANEL
        // Set layout style
        GridBagLayout gridbResult   = new GridBagLayout();
        panelResult.setLayout(gridbResult);

        // Initialize components
        table = ComponentFactory.getMovieTable(tableModel, true);
        table.addKeyListener(new TableKeyListener());
        tableModel.addTableModelListener(new TableValueChangedAction());
        table.addMouseListener(new DblClickMouseListener());

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(scrollResults.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelResult.add(    scrollResults,  new GridBagConstraints( 0, 0, 1, 1, 80.0, 80.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // TABLE ACTIONS PANEL
        // Set layout style
        GridBagLayout gridbTabAct        = new GridBagLayout();
        panelTabAct.setLayout(gridbTabAct);

        // Create the components
        buttonSave  = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave));
        buttonClear = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));
        buttonIMDB      = ComponentFactory.getButton(Resources.getText(DCLabels.lblIMDBUpdate));
        buttonIMDBAll   = ComponentFactory.getButton(Resources.getText(DCLabels.lblIMDBUpdateAll));


        buttonIMDB.setToolTipText(Resources.getText(DCTooltips.tpIMDBUpdate));
        buttonIMDBAll.setToolTipText(Resources.getText(DCTooltips.tpIMDBUpdateAll));
        buttonSave.setToolTipText(Resources.getText(DCTooltips.tpSave));
        buttonClear.setToolTipText(Resources.getText(DCTooltips.tpClear));

        buttonIMDB.addActionListener(new IMDBUpdate());
        buttonIMDBAll.addActionListener(new IMDBUpdateAll());
        buttonSave.addActionListener(new SaveSearchAction());
        buttonClear.addActionListener(new ClearButtonActionSearch());

        // Build the panel
        panelTabAct.add(    buttonSave,        new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0, 10, 0, 5), 0, 0));
        panelTabAct.add(    buttonClear,       new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0, 0, 0, 0), 0, 0));
        panelTabAct.add(    buttonIMDB,        new GridBagConstraints( 2, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0, 0, 0, 0), 0, 0));
        panelTabAct.add(    buttonIMDBAll,     new GridBagConstraints( 3, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0, 0, 0, 0), 0, 0));


        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(    labelStatus,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 0, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbag     = new GridBagLayout();
        setLayout(gridbag);

        // Initialize components
        panelResult.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblSearchResult)));
        panelStatus.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblStatus)));
        panelInput.setBorder(ComponentFactory.getBorder(
                             Resources.getText(DCLabels.lblSearchCond)));
        panelSearchOpts.setBorder(ComponentFactory.getBorder(
                                  Resources.getText(DCLabels.lblSearchOpts)));

        // Build the panel
        add(    panelInput,       new GridBagConstraints( 0, 0, 1, 2, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelSearchOpts,  new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelSearchAct,   new GridBagConstraints( 1, 1, 2, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelResult,      new GridBagConstraints( 0, 2, 2, 1, 80.0, 80.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelTabAct,      new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelStatus,      new GridBagConstraints( 0, 4, 2, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }

    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************
    private class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());
            searchValues();
            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class ClearButtonActionSearch implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());
            clearTable();
            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class SaveSearchAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());

            if (ImporList.running == false) {
                saveUpdatedValues();
            } else {
                new MessageBox(DataCrow.mainFrame,
                               Resources.getText(DCMessages.msgSavingJobRunning), 
                               true, MessageBox._NOTIFICATION);
            }

            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class TableKeyListener implements KeyListener {
        public void keyPressed(KeyEvent e) {
            if (ImporList.running == false) {
                if (e.getKeyCode() == e.VK_F12) {
                    createRecordForm();
                }
            } else {
                new MessageBox( DataCrow.mainFrame,
                                Resources.getText(DCMessages.msgSavingJobRunning), 
                                true, MessageBox._NOTIFICATION);
            }
        }

        public void keyReleased(KeyEvent e) {}

        public void keyTyped(KeyEvent e) {}

    }

    private class DblClickMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                createRecordForm();
            }        
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }
    
    private class IMDBUpdateAll implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate(false, true);            
        }   
    }
    
    private class IMDBUpdate implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate(true, true);
        }   
    }    
}

