/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.mp3;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ToolTipManager;

import rjw.dc.console.MessageBox;
import rjw.dc.console.RecordForm;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.NumberField;
import rjw.dc.console.components.TablePanel;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.CdRom;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.mp3.Mp3;
import rjw.dc.db.QueryOptions;
import rjw.dc.processes.ImporList;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;

/**
 * Panel used for searching and updating existing MP3 records. Searching
 * is done by using condition (title like <value>) and using options such as
 * all conditions must apply or the seperate conditions must apply
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SearchMP3Panel extends TablePanel {

    private JLabel                 labelFileName;
    private JTextField             textFileName;
    private JLabel                 labelTitle;
    private JTextField             textTitle;
    private JLabel                 labelArtist;
    private JTextField             textArtist;
    private JLabel                 labelAlbum;
    private JTextField             textAlbum;
    private JLabel                 labelYear;
    private JComboBox              comboYear;
    private JLabel                 labelGenre;
    private JComboBox              comboGenre;
    private JLabel                 labelLength;
    private NumberField            numberLength;
    private JLabel                 labelQuality;
    private NumberField            numberQuality;
    private JLabel                 labelLocation;
    private JTextField             textLocation;
    private JLabel                 labelDisp;
    private JComboBox              comboDisp;

    private JComboBox              comboAndOr;
    private JLabel                 labelAndOr;
    private JComboBox              comboPrecise;
    private JLabel                 labelPrecise;

    private JLabel                 labelOrderBy;
    private JComboBox              comboOrderBy;

    private JButton                buttonSearch;
    private JButton                buttonClear;
    private JButton                buttonSave;

    private JPanel                 panelStatus;
    private JPanel                 panelInput;
    private JPanel                 panelResult;
    private JPanel                 panelTabAct;
    private JPanel                 panelSearchOpts;
    private JPanel                 panelSearchAct;

    private CdRom                  cdrom = new CdRom();
    private Requestor              requestor;

    /**
     * Creates and shows the panel
     */
    public SearchMP3Panel() {
        buildPanel();
    }

    /**
     * Searches for existing MP3 records. The records are searched for by using
     * the conditions as filled in and the options.
     */
    public void searchValues() {
        Mp3 mp3 = new Mp3();
        mp3.setValues(getSearchValues());

        requestor = new Requestor(this.getClass(), "fillTable", null);
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestor);

        mp3.load(getQueryOptions(), requestors);
        setStatus(Resources.getText(DCMessages.msgSearchHasBeenExecuted));
    }

    /**
     * Creates an Array of Strings of the search values. The search values
     * are the conditions for a search action
     *
     * @return String[] array of search conditions
     */
    private Object[] getSearchValues() {
        Object[] searchValues = new Object[10];

        searchValues[0] = textFileName.getText().trim();
        searchValues[1] = textTitle.getText().trim();
        searchValues[2] = textArtist.getText().trim();
        searchValues[3] = textAlbum.getText().trim();
        searchValues[4] = comboYear.getSelectedItem().toString().trim();
        searchValues[5] = comboGenre.getSelectedItem().toString().trim();
        searchValues[6] = numberLength.getValue();
        searchValues[7] = numberQuality.getValue();
        searchValues[8] = textLocation.getText().trim();
        searchValues[9] = comboDisp.getSelectedItem().toString().trim();

        return searchValues;
    }

    /**
     * Returns the query options used for searching values
     *
     * @return QueryOptions
     */
    private QueryOptions getQueryOptions() {
        boolean bPreciseSelect = false;
        boolean bComplyToAllCondition = true;
        String  sYes            = Resources.getText(DCLabels.lblYes);
        String  sOr             = Resources.getText(DCLabels.lblOr);

        if (comboAndOr.getSelectedItem().equals(sOr)) {
            bComplyToAllCondition = false;
        }
        if (comboPrecise.getSelectedItem().equals(sYes)) {
            bPreciseSelect = true;
        }

        String sOrderBy = "";
        if (comboOrderBy.getSelectedItem() != null) {
            sOrderBy    = comboOrderBy.getSelectedItem().toString();
        }

        return new QueryOptions(sOrderBy, bComplyToAllCondition, bPreciseSelect);
    }

    /**
     * Saves all updated values to the database. The changed row numbers are
     * cached for easy retrieval of the changed records
     */
    public void saveUpdatedValues() {
        DataCrowCollection collection = new DataCrowCollection();
        Vector vValues = getChangedValues(Repository.dataBase.dbMP3ColArrayPK);
        int i = 0;
        for (Enumeration enum = vValues.elements(); enum.hasMoreElements(); ) {
            int row = getRowNumberWithID((String) vChangedIDs.elementAt(i++));           
            Mp3 mp3 = new Mp3();
            mp3.setValues((String[]) enum.nextElement());
            collection.addObject(mp3, new Integer(row));
        }
        collection.saveUpdate(new RequestorCollection());
        vChangedIDs.removeAllElements();        
    }

    /**
     * Creates a record form for the selected record
     */
    public void createRecordForm() {
        if (table.getRowCount() == 0) {
            return;   
        }
        
        int columnCount = table.getColumnModel().getColumnCount();
        int row = table.getSelectedRow();
        Object[][] data = new Object[columnCount][4];

        Mp3 mp3 = new Mp3();
        for (int i = 0; i < columnCount; i++) {
            data[i][0] = mp3.getLabel(i);
            data[i][3] = new Integer(mp3.getMaxFieldLength(i));
            
            if (i == 1 || i == 2 || i == 3 || i == 4) {
                data[i][1] = new Integer(ComponentFactory._LONGTEXTFIELD);
            } else if (i == 10) {
                data[i][1] = new Integer(ComponentFactory._YESNOCOMBO);
            } else if (i == 5) {
                data[i][1] = new Integer(ComponentFactory._YEARCOMBO);
            } else if (i == 6) {
                data[i][1] = new Integer(ComponentFactory._MP3GENRECOMBO);
            } else if (i == 7 || i == 8) {
                data[i][1] = new Integer(ComponentFactory._NUMBERFIELD);
            } else {
                data[i][1] = new Integer(ComponentFactory._SHORTTEXTFIELD);
            }

            if (table.getValueAt(row, i) == null) {
                return;
            } else {
                data[i][2] = table.getValueAt(row, i);
            }
        }
        recordForm  = new RecordForm(data, 
                                     true,
                                     Resources.getText(DCLabels.lblRecord, table.getValueAt(row,0).toString()),
                                     table.getValueAt(row,0).toString()); 
        vRecordForms.add(recordForm);
    }

    /**
     * Initializes the panel
     */
    private void buildPanel() {
        // The sub panels
        panelStatus     = new JPanel(false);
        panelInput      = new JPanel(false);
        panelResult     = new JPanel(false);
        panelTabAct     = new JPanel(false);
        panelSearchOpts = new JPanel(false);
        panelSearchAct  = new JPanel(false);

        // ************************************************************************
        // INPUT PANEL
        // Set layout style
        GridBagLayout gridbInput        = new GridBagLayout();
        panelInput.setLayout(gridbInput);

        Mp3 mp3 = new Mp3();
        // Initialize components
        labelFileName   = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3FileName));
        textFileName    = ComponentFactory.getShortTextField(
                            mp3.getMaxFieldLength(mp3._A_FILENAME));
        labelTitle      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Title));
        textTitle       = ComponentFactory.getShortTextField(
                            mp3.getMaxFieldLength(mp3._B_TITLE));
        labelArtist     = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Artist));
        textArtist      = ComponentFactory.getShortTextField(
                            mp3.getMaxFieldLength(mp3._C_ARTIST));
        labelAlbum      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Album));
        textAlbum       = ComponentFactory.getShortTextField(
                            mp3.getMaxFieldLength(mp3._D_ALBUM));
        labelYear       = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Year));
        comboYear       = ComponentFactory.getYearComboBox();
        labelGenre      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Genre));
        comboGenre      = ComponentFactory.getMP3GenreComboBox();
        labelLength     = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Length));
        numberLength      = ComponentFactory.getNumberField();
        labelQuality    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Quality));
        numberQuality     = ComponentFactory.getNumberField();
        labelLocation   = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Location));
        textLocation    = ComponentFactory.getShortTextField(
                            mp3.getMaxFieldLength(mp3._I_LOCATION));
        labelDisp       = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMP3Dispose));
        comboDisp       = ComponentFactory.getYesNoCombo();

        // Build the panel
        panelInput.add( labelFileName,   new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textFileName,    new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelTitle,      new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textTitle,       new GridBagConstraints( 1, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelArtist,     new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textArtist,      new GridBagConstraints( 1, 2, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelAlbum,      new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textAlbum,       new GridBagConstraints( 1, 3, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelYear,       new GridBagConstraints( 0, 4, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( comboYear,       new GridBagConstraints( 1, 4, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelGenre,      new GridBagConstraints( 0, 5, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( comboGenre,      new GridBagConstraints( 1, 5, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelLength,     new GridBagConstraints( 0, 6, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( numberLength,      new GridBagConstraints( 1, 6, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelQuality,    new GridBagConstraints( 0, 7, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( numberQuality,     new GridBagConstraints( 1, 7, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelLocation,   new GridBagConstraints( 0, 8, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textLocation,    new GridBagConstraints( 1, 8, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelDisp,       new GridBagConstraints( 0, 9, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( comboDisp,       new GridBagConstraints( 1, 9, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 10, 5, 5), 0, 0));

        // ************************************************************************
        // SEARCH OPTION PANEL
        // Set layout style
        GridBagLayout gridbSearchOpts   = new GridBagLayout();
        panelSearchOpts.setLayout(gridbSearchOpts);

        // Initialize components
        comboAndOr      = ComponentFactory.getComboBox();
        labelAndOr      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblAndOrSelect));
        comboPrecise    = ComponentFactory.getRealYesNoComboBox();
        labelPrecise    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblPreciseSelect));
        comboOrderBy    = ComponentFactory.getComboBox();
        labelOrderBy    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblOrderBy));

        String[]    colNames    = Repository.dataBase.dbMP3ColArray;
        comboOrderBy.addItem("");

        for (int i = 0; i < colNames.length; i++) {
            comboOrderBy.addItem(colNames[i]);
        }

        comboAndOr.addItem(Resources.getText(DCLabels.lblAnd));
        comboAndOr.addItem(Resources.getText(DCLabels.lblOr));

        // Build the panel
        panelSearchOpts.add( labelAndOr,         new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 10), 0, 0));
        panelSearchOpts.add( comboAndOr,         new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 10, 0, 5), 0, 0));
        panelSearchOpts.add( labelPrecise,       new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 10), 0, 0));
        panelSearchOpts.add( comboPrecise,       new GridBagConstraints( 1, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 10, 0, 5), 0, 0));
        panelSearchOpts.add( labelOrderBy,       new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 10), 0, 0));
        panelSearchOpts.add( comboOrderBy,       new GridBagConstraints( 1, 2, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 10, 5, 5), 0, 0));

        // ************************************************************************
        // SEARCH ACTION PANEL
        // Set layout style
        GridBagLayout gridbSearchAct        = new GridBagLayout();
        panelSearchAct.setLayout(gridbSearchAct);


        buttonSearch    = ComponentFactory.getButton(Resources.getText(DCLabels.lblSearch));
        buttonSearch.setToolTipText(Resources.getText(DCTooltips.tpSearch));
        buttonSearch.addActionListener(new SearchButtonAction());

        panelSearchAct.add(  buttonSearch,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets( 5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // RESULT PANEL
        // Set layout style
        GridBagLayout gridbResult   = new GridBagLayout();
        panelResult.setLayout(gridbResult);

        // Initialize components
        table = ComponentFactory.getMP3Table(tableModel, true);
        table.addKeyListener(new TableKeyListener());
        tableModel.addTableModelListener(new TableValueChangedAction());
        table.addMouseListener(new DblClickMouseListener());

        JScrollPane scrollResults = new JScrollPane(table);
		scrollResults.setVerticalScrollBarPolicy(scrollResults.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelResult.add(    scrollResults,  new GridBagConstraints( 0, 0, 1, 1, 20.0, 20.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // TABLE ACTIONS PANEL
        // Set layout style
        GridBagLayout gridbTabAct        = new GridBagLayout();
        panelTabAct.setLayout(gridbTabAct);

        // Create the components
        buttonSave  = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave));
        buttonClear = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));

        buttonSave.setToolTipText(Resources.getText(DCTooltips.tpSave));
        buttonClear.setToolTipText(Resources.getText(DCTooltips.tpClear));

        buttonSave.addActionListener(new SaveSearchAction());
        buttonClear.addActionListener(new ClearButtonActionSearch());


        // Build the panel
        panelTabAct.add(    buttonSave,         new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0, 10, 0, 5), 0, 0));
        panelTabAct.add(    buttonClear,        new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  0, 0, 0), 0, 0));


        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(    labelStatus,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 0, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbag     = new GridBagLayout();
        setLayout(gridbag);

        // Initialize components
        panelResult.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblSearchResult)));
        panelStatus.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblStatus)));
        panelInput.setBorder(ComponentFactory.getBorder(
                             Resources.getText(DCLabels.lblSearchCond)));
        panelSearchOpts.setBorder(ComponentFactory.getBorder(
                                  Resources.getText(DCLabels.lblSearchOpts)));

        // Build the panel
        add(    panelInput,       new GridBagConstraints( 0, 0, 1, 2, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5,  5, 5, 5), 0, 0));
        add(    panelSearchOpts,  new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                new Insets(5,  5, 5, 5), 0, 0));
        add(    panelSearchAct,   new GridBagConstraints( 1, 1, 2, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelResult,      new GridBagConstraints( 0, 2, 2, 1, 60.0, 60.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelTabAct,      new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelStatus,      new GridBagConstraints( 0, 4, 2, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }

    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************
    private class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());
            searchValues();
            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class ClearButtonActionSearch implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());
            clearTable();
            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class SaveSearchAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());

            if (ImporList.running == false) {
                saveUpdatedValues();
            } else {
                new MessageBox(DataCrow.mainFrame,
                    Resources.getText(DCMessages.msgSavingJobRunning), 
                    true, MessageBox._NOTIFICATION);
            }

            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class TableKeyListener implements KeyListener {
        public void keyPressed(KeyEvent e) {
            if (ImporList.running == false) {
                if (e.getKeyCode() == e.VK_F12) {
                    createRecordForm();
                }
            } else {
                new MessageBox(DataCrow.mainFrame,
                    Resources.getText(DCMessages.msgSavingJobRunning), 
                    true, MessageBox._NOTIFICATION);
            }
        }

        public void keyReleased(KeyEvent e) {}

        public void keyTyped(KeyEvent e) {}

    }

    private class DblClickMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                createRecordForm();
            }        
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }
}

