package rjw.dc.console.software;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.TablePanel;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.software.Catagory;
import rjw.dc.core.objects.software.CatagoryCollection;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

public class CatagoryForm extends JFrame {
    
    public CatagoryForm() {
        CatagoryPanel catPanel = new CatagoryPanel();
        getContentPane().setLayout(new GridBagLayout());

        getContentPane().add(catPanel,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        setResizable(true);
        pack();
        setVisible(true);
    }

    private class CatagoryPanel extends TablePanel {
        /**
         * Saves the changed Software categories to the database
         */
        protected void saveCategories() {
            Object[] rowData = new Object[table.getColumnCount()];
            boolean bNew = false;
            
            for (Enumeration enum = vChangedIDs.elements(); enum.hasMoreElements(); ) {
                int row = getRowNumberWithID(enum.nextElement().toString());
                for (int j = 0; j < table.getColumnCount(); j++) {
                    Object oTest = table.getValueAt(row, j);
                    if (oTest != null) {
                        rowData[j] = oTest.toString();
                    } else {
                        bNew = true;    
                        rowData[j] = Repository.dataBase.dBEmptyValue;
                    }
                }
                Catagory catagory = new Catagory();
                catagory.setValues(rowData);
    
                if (bNew) {            
                    catagory.saveNew(new RequestorCollection());
                } else {
                    catagory.saveUpdate(new RequestorCollection());
                }
            }
            CatagoryCollection.refresh();
            vChangedIDs.removeAllElements();
        }
    
        /**
         * Builds the panel
         */
        private void buildPanel() {
            Color color = Settings.getValueAsColor(Repository.settings.stColor);
            this.setBackground(color);
            
            JPanel panelCategory = new JPanel(false);
            panelCategory.setBackground(color);
            JPanel panelTabAct = new JPanel(false);
            panelTabAct.setBackground(color);
            JPanel panelMP3Genres = new JPanel(false);
            panelMP3Genres.setBackground(color);
            
            // ************************************************************************
            // CATEGORY PANEL
            // Set layout style
            GridBagLayout gridbCategory = new GridBagLayout();
            panelCategory.setLayout(gridbCategory);
    
            // Initialize components
            tableModel.setColumnCount(3);
            table = ComponentFactory.getSimpleTable(tableModel, true);
            tableModel.addTableModelListener(new TableValueChangedAction());
            table.setPreferredScrollableViewportSize(new Dimension(390, 285));
    
            DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
    
            TableColumn columnNew   = table.getColumnModel().getColumn(0);
            columnNew.setHeaderValue(Resources.getText(DCLabels.lblColumnCatID));
    
            JTextField text = ComponentFactory.getTextField();
            text.setEditable(false);
            columnNew.setCellEditor(new DefaultCellEditor(text));
            renderer = new DefaultTableCellRenderer();
            renderer.setBackground(ComponentFactory.getDisabledColor());
            columnNew.setCellRenderer(renderer);
            columnNew = table.getColumnModel().getColumn(1);
            columnNew.setHeaderValue(Resources.getText(DCLabels.lblColumnCatName));
            columnNew = table.getColumnModel().getColumn(2);
            columnNew.setHeaderValue(Resources.getText(DCLabels.lblColumnDispose));
            JComboBox comboBox = ComponentFactory.getYesNoCombo();
            columnNew.setCellEditor(new DefaultCellEditor(comboBox));
            renderer  = new DefaultTableCellRenderer();
            columnNew.setCellRenderer(renderer);
    
            JScrollPane scrollResults = new JScrollPane(table);
            scrollResults.setVerticalScrollBarPolicy(scrollResults.VERTICAL_SCROLLBAR_ALWAYS);        
    
            // Build the panel
            panelCategory.add(scrollResults,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                              new Insets(5, 5, 5, 5), 0, 0));
            
            // ************************************************************************
            // TABLE ACTION PANEL
            // Set layout style
            GridBagLayout gridbTabAct       = new GridBagLayout();
            panelTabAct.setLayout(gridbTabAct);
    
            // Initialize components
            JButton buttonSave = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave));
            JButton buttonAddRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblAddRow));
    
            buttonSave.addActionListener(new SaveCatAction());
            buttonAddRow.addActionListener(new AddRowButtonAction());
    
            // Build the panel
            panelTabAct.add(buttonSave,     new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                            new Insets( 5, 5, 5, 5), 0, 0));
            panelTabAct.add(buttonAddRow,   new GridBagConstraints( 1, 0, 1, 1, 0.0, 0.0
                                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                            new Insets( 5, 5, 5, 5), 0, 0));
    
            // ************************************************************************
            // MP3 GENRE PANEL
            // Set layout style
            GridBagLayout       gridbMP3Genres   = new GridBagLayout();
            panelMP3Genres.setLayout(gridbMP3Genres);
    
            // Initialize components
            JTextArea textMP3 = ComponentFactory.getTextArea();
            textMP3.setEditable(false);
            
            JScrollPane scrollMP3Genres = new JScrollPane(textMP3);
            scrollMP3Genres.setPreferredSize(new Dimension(405, 302));
            scrollMP3Genres.setHorizontalScrollBarPolicy(scrollMP3Genres.HORIZONTAL_SCROLLBAR_NEVER);
            scrollMP3Genres.setVerticalScrollBarPolicy(scrollMP3Genres.VERTICAL_SCROLLBAR_ALWAYS);        
    
            // Build the panel
            panelMP3Genres.add(scrollMP3Genres, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                              , GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                new Insets( 5, 5, 5, 5), 0, 0));
    
    
            // ************************************************************************
            // MAIN PANEL
            // Set layout style
            GridBagLayout       gridbag         = new GridBagLayout();
            setLayout(gridbag);
    
            // Build the panel
            add(panelCategory,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                new Insets(5, 5, 5, 5), 0, 0));
            add(panelMP3Genres, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                new Insets(5, 5, 5, 5), 0, 0));
            add(panelTabAct,    new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                new Insets(5, 5, 5, 5), 0, 0));
        }
    
        private class SaveCatAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                saveCategories();
            }
        }
    }
}

