/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.software;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;

import rjw.dc.console.MessageBox;
import rjw.dc.console.RecordForm;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.TablePanel;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.software.Software;
import rjw.dc.processes.ImporList;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;

/**
 * Panel for inserting new Software Objects into the datsabase
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class InsertSoftwarePanel extends TablePanel {
    private JPanel              panelInput;
    private JPanel              panelStatus;
    private JPanel              panelTabAct;

    private JButton             buttonSave;
    private JButton             buttonClear;
    private JButton             buttonAddRow;
    private JButton             buttonRemoveRow;
    private RecordForm          recordForm;

    /**
     * Initializes and creates the panel
     */
    public InsertSoftwarePanel() {
        buildPanel();
    }

    /**
     * Saves the values as filled in the table to the Database
     *
     * @see Software
     */
    public void saveValues() {
        DataCrowCollection collection = new DataCrowCollection();

        Vector vValues = getNewValues(Repository.dataBase.dbSoftColArrayPK);
        int i = 1;
        for (Enumeration enum = vValues.elements(); enum.hasMoreElements(); ) {
            Software software = new Software();
            software.setValues((String[]) enum.nextElement());
            software.setUIRowNumber(i);
            collection.addObject(software, new Integer(i));            
            i++;
        }

        Requestor requestor = new Requestor(this.getClass(), "setStatus", null);
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestor);

        collection.saveNew(requestors);
        setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));
    }

    /**
     * Opens a record form for inserting a single record
     *
     * @see RecordForm
     */
    public void createRecordForm() {
        if (table.getRowCount() == 0) {
            return;   
        }        
        
        int columnCount     = table.getColumnModel().getColumnCount();
        int row             = table.getSelectedRow();
        Object[][] data     = new Object[columnCount][4];

        Software software = new Software();
        for (int i = 0; i < columnCount; i++) {
            data[i][0] = software.getLabel(i);
            data[i][3] = new Integer(software.getMaxFieldLength(i));
            
            if (i == 0 || i == 1 || i == 3 || i == 6) {
                data[i][1] = new Integer(ComponentFactory._SHORTTEXTFIELD);
            } else if (i == 2) {
                data[i][1] = new Integer(ComponentFactory._LONGTEXTFIELD);
            } else if (i == 5 || i == 7) {
                data[i][1] = new Integer(ComponentFactory._YESNOCOMBO);
            } else if (i == 4) {
                data[i][1] = new Integer(ComponentFactory._SOFTWARETYPECOMBO);
            }

            if (table.getValueAt(row, i) == null) {
                data[i][2] = "";
            } else {
                data[i][2] = table.getValueAt(row, i);
            }
        }

        recordForm  =   new RecordForm(data, 
                                       true,
                                       Resources.getText(DCLabels.lblNewRecord), 
                                       null);
                                       
        vRecordForms.add(recordForm); 
    }

    /**
     * Creates the panel
     */
    private void buildPanel() {
        panelInput      = new JPanel(false);
        panelStatus     = new JPanel(false);
        panelTabAct     = new JPanel(false);


        // ************************************************************************
        // INPUT PANEL
        // Set layout style
        GridBagLayout gridbInput = new GridBagLayout();
        panelInput.setLayout(gridbInput);

        // Initialize components
        table   = ComponentFactory.getSoftwareTable(tableModel, true);
        table.addMouseListener(new DblClickMouseListener());
        table.addKeyListener(new TableKeyListener());

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(scrollResults.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelInput.add( scrollResults,      new GridBagConstraints( 0, 0, 7, 1, 20.0, 20.0
                       ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // TABLE ACTION PANEL
        // Set layout style
        GridBagLayout gridbTabAct       = new GridBagLayout();
        panelTabAct.setLayout(gridbTabAct);

        // Initialize components
        buttonSave      = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave));
        buttonClear     = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));
        buttonAddRow    = ComponentFactory.getButton(Resources.getText(DCLabels.lblAddRow));
        buttonRemoveRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblRemoveRow));

         // TODO
        buttonRemoveRow.setToolTipText("");
        buttonSave.setToolTipText(Resources.getText(DCTooltips.tpSave));
        buttonClear.setToolTipText(Resources.getText(DCTooltips.tpClear));
        buttonAddRow.setToolTipText(Resources.getText(DCTooltips.tpAddRow));

        buttonSave.addActionListener(new InsertButtonAction());
        buttonClear.addActionListener(new ClearButtonAction());
        buttonAddRow.addActionListener(new AddRowButtonAction());
        buttonRemoveRow.addActionListener(new RemoveRowButtonAction());

        // Build the panel
        panelTabAct.add( buttonSave,      new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0, 10, 0, 5), 0, 0));
        panelTabAct.add( buttonClear,     new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0, 0, 0, 0), 0, 0));
        panelTabAct.add( buttonRemoveRow,    new GridBagConstraints( 2, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0, 5, 0, 0), 0, 0));
        panelTabAct.add( buttonAddRow,     new GridBagConstraints( 3, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0, 5, 0, 0), 0, 0));


        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(    labelStatus,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                            new Insets(0,  0, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbag     = new GridBagLayout();
        setLayout(gridbag);

        // Initialize components
        panelInput.setBorder(ComponentFactory.getBorder(
                             Resources.getText(DCLabels.lblInsert)));
        panelStatus.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblStatus)));

        // Fill the main panel
        add(panelInput,     new GridBagConstraints( 0, 0, 8, 1, 20.0, 20.0
                           ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                            new Insets( 5, 5, 5, 5), 0, 0));
        add(panelTabAct,    new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets( 5, 5, 5, 5), 0, 0));
        add(panelStatus,    new GridBagConstraints( 0, 2, 8, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }



    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************

    private class InsertButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (ImporList.running == false) {
                saveValues();
            } else {
                new MessageBox(DataCrow.mainFrame,
                    Resources.getText(DCMessages.msgSavingJobRunning), 
                    true, MessageBox._NOTIFICATION);
            }
        }
    }

    private class TableKeyListener implements KeyListener {
        public void keyPressed(KeyEvent e) {
            if (ImporList.running == false) {
                if (e.getKeyCode() == 123) {
                    createRecordForm();
                }
            } else {
                new MessageBox(DataCrow.mainFrame,
                    Resources.getText(DCMessages.msgSavingJobRunning), 
                    true, MessageBox._NOTIFICATION);
            }
        }
        public void keyReleased(KeyEvent e) {}
        public void keyTyped(KeyEvent e) {}

    }

    private class DblClickMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                createRecordForm();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {
        }
    }    
}
