/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.software;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ToolTipManager;

import rjw.dc.console.MessageBox;
import rjw.dc.console.RecordForm;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.TablePanel;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.CdRom;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.software.Catagory;
import rjw.dc.core.objects.software.CatagoryCollection;
import rjw.dc.core.objects.software.Software;
import rjw.dc.db.QueryOptions;
import rjw.dc.processes.ImporList;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;

/**
 * Panel used for searching and updating existing Software objects. Searching
 * is done by using condition (name like <value>) and using options such as
 * all conditions must apply or the seperate conditions must apply
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SearchSoftwarePanel extends TablePanel {

    private JLabel                 labelName;
    private JTextField             textName;
    private JLabel                 labelDesc;
    private JTextField             textDesc;
    private JLabel                 labelFileName;
    private JTextField             textFileName;
    private JLabel                 labelType;
    private JComboBox              comboType;
    private JLabel                 labelLoc;
    private JTextField             textLoc;
    private JLabel                 labelMulti;
    private JComboBox              comboMulti;
    private JLabel                 labelDisp;
    private JComboBox              comboDisp;

    private JComboBox              comboAndOr;
    private JLabel                 labelAndOr;
    private JComboBox              comboPrecise;
    private JLabel                 labelPrecise;

    private JLabel                 labelOrderBy;
    private JComboBox              comboOrderBy;

    private JButton                buttonSearch;
    private JButton                buttonClear;
    private JButton                buttonSave;

    private JPanel                 panelStatus;
    private JPanel                 panelInput;
    private JPanel                 panelResult;
    private JPanel                 panelTabAct;
    private JPanel                 panelSearchOpts;
    private JPanel                 panelSearchAct;

    private CdRom                  cdrom             = new CdRom();
    private Requestor              requestor;

    /**
     * Initializes and creates the panel
     */
    public SearchSoftwarePanel() {
        buildPanel();
    }

    /**
     * Searches for the Software objects by using the conditions and options
     * as specified.
     */
    public void searchValues() {
        Software software = new Software();
        software.setValues(getSearchValues());

        requestor = new Requestor(this.getClass(), "fillTable", null);
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestor);

        software.load(getQueryOptions(), requestors);
        setStatus(Resources.getText(DCMessages.msgSearchHasBeenExecuted));
    }

    /**
     * Returns the condition values
     *
     * @return (String[])   an array of conditions (name like <value>)
     */
    private Object[] getSearchValues() {
        Object[] searchValues = new Object[7];

        int i = 0;
        searchValues[i++] = textName.getText().trim();
        searchValues[i++] = textDesc.getText().trim();
        searchValues[i++] = textFileName.getText().trim();
        searchValues[i++] = comboType.getSelectedItem().toString().trim();
        searchValues[i++] = comboMulti.getSelectedItem().toString().trim();
        searchValues[i++] = textLoc.getText().trim();
        searchValues[i++] = comboDisp.getSelectedItem().toString().trim();

        return searchValues;
    }

    /**
     * Returns the query options used for searching values
     *
     * @return QueryOptions
     */
    private QueryOptions getQueryOptions() {
        boolean bPreciseSelect = false;
        boolean bComplyToAllCondition = true;
        String  sYes            = Resources.getText(DCLabels.lblYes);
        String  sOr             = Resources.getText(DCLabels.lblOr);

        if (comboAndOr.getSelectedItem().equals(sOr)) {
            bComplyToAllCondition = false;
        }
        if (comboPrecise.getSelectedItem().equals(sYes)) {
            bPreciseSelect = true;
        }

        String sOrderBy = "";
        if (comboOrderBy.getSelectedItem() != null) {
            sOrderBy    = comboOrderBy.getSelectedItem().toString();
        }

        return new QueryOptions(sOrderBy, bComplyToAllCondition, bPreciseSelect);
    }

    /**
     * Saves all updated values to the database. The changed row numbers are
     * cached for easy retrieval of the changed records
     */
    public void saveUpdatedValues() {
        DataCrowCollection collection = new DataCrowCollection();

        Vector vValues = getChangedValues(Repository.dataBase.dbSoftColArrayPK);
        int i = 0;
        for (Enumeration enum = vValues.elements(); enum.hasMoreElements(); ) {
            int row = getRowNumberWithID((String) vChangedIDs.elementAt(i++));
            Software software = new Software();
            software.setValues((String[]) enum.nextElement());
            collection.addObject(software, new Integer(row));
        }
        collection.saveUpdate(new RequestorCollection());
        vChangedIDs.removeAllElements();
    }

    /**
     * Creates a record form for the selected record
     */
    public void createRecordForm() {
        if (table.getRowCount() == 0) {
            return;   
        }        
        
        int columnCount     = table.getColumnModel().getColumnCount();
        int row             = table.getSelectedRow();
        Object[][] data     = new Object[columnCount][4];
        
        if (row < 0) {
            return;   
        }

        Software software = new Software();
        for (int i = 0; i < columnCount; i++) {
            data[i][0] = software.getLabel(i);
            data[i][3] = new Integer(software.getMaxFieldLength(i));
            
            if (i == 0 || i == 1 || i == 3 || i == 6) {
                data[i][1] = new Integer(ComponentFactory._SHORTTEXTFIELD);
            } else if (i == 2) {
                data[i][1] = new Integer(ComponentFactory._LONGTEXTFIELD);
            } else if (i == 5 || i == 7) {
                data[i][1] = new Integer(ComponentFactory._YESNOCOMBO);
            } else if (i == 4) {
                data[i][1] = new Integer(ComponentFactory._SOFTWARETYPECOMBO);
            }

            if (table.getValueAt(row, i) == null) {
                return;
            } else {
                data[i][2] = table.getValueAt(row, i);
            }
        }
        recordForm  = new RecordForm(data, 
                                     true, 
                                     Resources.getText(DCLabels.lblRecord, 
                                     table.getValueAt(row,0).toString()),
                                     table.getValueAt(row,0).toString());
        vRecordForms.add(recordForm);
    }

    /**
     * Creates the panel
     */
    private void buildPanel() {
        // The sub panels
        panelStatus     = new JPanel(false);
        panelInput      = new JPanel(false);
        panelResult     = new JPanel(false);
        panelTabAct     = new JPanel(false);
        panelSearchOpts = new JPanel(false);
        panelSearchAct  = new JPanel(false);


        // ************************************************************************
        // INPUT PANEL
        // Set layout style
        GridBagLayout gridbInput        = new GridBagLayout();
        panelInput.setLayout(gridbInput);

        // Initialize components
        Software software = new Software();
        labelName       = ComponentFactory.getLabel(Resources.getText(DCLabels.lblName));
        textName        = ComponentFactory.getShortTextField(software.getMaxFieldLength(
                                                             Software._A_NAME));
        labelDesc       = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDescription));
        textDesc        = ComponentFactory.getTextField();
        labelFileName   = ComponentFactory.getLabel(Resources.getText(DCLabels.lblFileName));
        textFileName    = ComponentFactory.getShortTextField(software.getMaxFieldLength(
                                                             Software._C_FILENAME));
        labelType       = ComponentFactory.getLabel(Resources.getText(DCLabels.lblType));
        comboType       = ComponentFactory.getSoftwareCatagoriesComboBox();
        labelLoc        = ComponentFactory.getLabel(Resources.getText(DCLabels.lblLocation));
        textLoc         = ComponentFactory.getShortTextField(software.getMaxFieldLength(
                                                             Software._F_LOCATION));
        labelMulti      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblMulti));
        comboMulti      = ComponentFactory.getYesNoCombo();
        labelDisp       = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDispose));
        comboDisp       = ComponentFactory.getYesNoCombo();
        
        labelName.setToolTipText(Resources.getText(DCTooltips.tpName));
        labelDesc.setToolTipText(Resources.getText(DCTooltips.tpDescript));
        labelFileName.setToolTipText(Resources.getText(DCTooltips.tpFileName));
        labelType.setToolTipText(Resources.getText(DCTooltips.tpType));
        labelLoc.setToolTipText(Resources.getText(DCTooltips.tpLoc));
        labelMulti.setToolTipText(Resources.getText(DCTooltips.tpMulti));
        labelDisp.setToolTipText(Resources.getText(DCTooltips.tpDispose));

        // Build the panel
        panelInput.add( labelName,      new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textName,       new GridBagConstraints( 1, 0, 1, 1, 2.0, 2.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelDesc,      new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textDesc,       new GridBagConstraints( 1, 1, 1, 1, 2.0, 2.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelFileName,  new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textFileName,   new GridBagConstraints( 1, 2, 1, 1, 2.0, 2.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelType,      new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( comboType,      new GridBagConstraints( 1, 3, 1, 1, 2.0, 2.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelMulti,     new GridBagConstraints( 0, 4, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( comboMulti,     new GridBagConstraints( 1, 4, 1, 1, 2.0, 2.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelLoc,       new GridBagConstraints( 0, 5, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( textLoc,        new GridBagConstraints( 1, 5, 1, 1, 2.0, 2.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 10, 0, 5), 0, 0));
        panelInput.add( labelDisp,      new GridBagConstraints( 0, 6, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 5, 0, 10), 0, 0));
        panelInput.add( comboDisp,      new GridBagConstraints( 1, 6, 1, 1, 2.0, 2.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(0, 10, 5, 5), 0, 0));

        comboType.addActionListener(new RefreshCategoriesAction());


        // ************************************************************************
        // SEARCH OPTION PANEL
        // Set layout style
        GridBagLayout gridbSearchOpts   = new GridBagLayout();
        panelSearchOpts.setLayout(gridbSearchOpts);

        // Initialize components
        comboAndOr      = ComponentFactory.getComboBox();
        labelAndOr      = ComponentFactory.getLabel(Resources.getText(DCLabels.lblAndOrSelect));
        comboPrecise    = ComponentFactory.getRealYesNoComboBox();
        labelPrecise    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblPreciseSelect));
        comboOrderBy    = ComponentFactory.getComboBox();
        labelOrderBy    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblOrderBy));

        String[]    colNames    = Repository.dataBase.dbSoftColArray;
        comboOrderBy.addItem("");

        for (int i = 0; i < colNames.length; i++) {
            comboOrderBy.addItem(colNames[i]);
        }

        comboAndOr.addItem(Resources.getText(DCLabels.lblAnd));
        comboAndOr.addItem(Resources.getText(DCLabels.lblOr));

        // Build the panel
        panelSearchOpts.add( labelAndOr,         new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 5), 0, 0));
        panelSearchOpts.add( comboAndOr,         new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 5), 0, 0));
        panelSearchOpts.add( labelPrecise,       new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 5), 0, 0));
        panelSearchOpts.add( comboPrecise,       new GridBagConstraints( 1, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 5), 0, 0));
        panelSearchOpts.add( labelOrderBy,       new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 5), 0, 0));
        panelSearchOpts.add( comboOrderBy,       new GridBagConstraints( 1, 2, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 5, 0, 5), 0, 0));

        // ************************************************************************
        // SEARCH ACTION PANEL
        // Set layout style
        GridBagLayout gridbSearchAct        = new GridBagLayout();
        panelSearchAct.setLayout(gridbSearchAct);

        buttonSearch = ComponentFactory.getButton(Resources.getText(DCLabels.lblSearch));
        buttonSearch.setToolTipText(Resources.getText(DCTooltips.tpSearch));
        buttonSearch.addActionListener(new SearchButtonAction());

        panelSearchAct.add(  buttonSearch,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets( 5, 5, 5, 5), 0, 0));

        // ************************************************************************
        // RESULT PANEL
        // Set layout style
        GridBagLayout gridbResult   = new GridBagLayout();
        panelResult.setLayout(gridbResult);

        // Initialize components
        table = ComponentFactory.getSoftwareTable(tableModel, true);
        table.addKeyListener(new TableKeyListener());
        tableModel.addTableModelListener(new TableValueChangedAction());
        table.addMouseListener(new DblClickMouseListener());

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(scrollResults.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelResult.add(    scrollResults,  new GridBagConstraints( 0, 0, 1, 1, 20.0, 20.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // TABLE ACTIONS PANEL
        // Set layout style
        GridBagLayout gridbTabAct        = new GridBagLayout();
        panelTabAct.setLayout(gridbTabAct);

        // Create the components
        buttonSave  = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave));
        buttonClear = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));

        buttonSave.setToolTipText(Resources.getText(DCTooltips.tpSave));
        buttonClear.setToolTipText(Resources.getText(DCTooltips.tpClear));

        buttonSave.addActionListener(new SaveSearchAction());
        buttonClear.addActionListener(new ClearButtonActionSearch());


        // Build the panel
        panelTabAct.add(    buttonSave,         new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                            new Insets(0, 10, 0, 5), 0, 0));
        panelTabAct.add(    buttonClear,        new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  0, 0, 0), 0, 0));


        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(    labelStatus, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 0, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbag     = new GridBagLayout();
        setLayout(gridbag);

        // Initialize components
        panelResult.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblSearchResult)));
        panelStatus.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblStatus)));
        panelInput.setBorder(ComponentFactory.getBorder(
                             Resources.getText(DCLabels.lblSearchCond)));
        panelSearchOpts.setBorder(ComponentFactory.getBorder(
                                  Resources.getText(DCLabels.lblSearchOpts)));

        // Build the panel
        add(    panelInput,       new GridBagConstraints( 0, 0, 1, 2, 1.0, 1.0
               ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelSearchOpts,  new GridBagConstraints( 1, 0, 3, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelSearchAct,   new GridBagConstraints( 1, 1, 1, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelResult,      new GridBagConstraints( 0, 2, 4, 1, 80.0, 80.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelTabAct,      new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelStatus,      new GridBagConstraints( 0, 4, 4, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }


    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************
    private class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());
            searchValues();
            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class ClearButtonActionSearch implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());
            clearTable();
            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class RefreshCategoriesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object oSelected = comboType.getSelectedItem();

            Catagory cat = new Catagory();
            Vector vData = CatagoryCollection.getCatagories();
            comboType.removeAllItems();
            comboType.addItem("");

            for (Enumeration enum = vData.elements(); enum.hasMoreElements(); ) {
                cat = (Catagory) enum.nextElement();
                comboType.addItem(cat.getValue(Catagory._A_NAME));
            }

            try {
                comboType.setSelectedItem(oSelected);
            } catch (Exception exp) {}
        }
    }

    private class SaveSearchAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.removeTableModelListener(new TableValueChangedAction());

            if (ImporList.running == false) {
                saveUpdatedValues();
            } else {
                new MessageBox(DataCrow.mainFrame,
                    Resources.getText(DCMessages.msgSavingJobRunning), 
                    true, MessageBox._NOTIFICATION);
            }

            tableModel.addTableModelListener(new TableValueChangedAction());
        }
    }

    private class TableKeyListener implements KeyListener {
        public void keyPressed(KeyEvent e) {
            if (ImporList.running == false) {
                if (e.getKeyCode() == e.VK_F12) {
                    createRecordForm();
                }
            } else {
                new MessageBox(DataCrow.mainFrame,
                    Resources.getText(DCMessages.msgSavingJobRunning), 
                    true, MessageBox._NOTIFICATION);
            }
        }

        public void keyReleased(KeyEvent e) {}

        public void keyTyped(KeyEvent e) {}

    }

    public void refreshCatagories() {
               
    }    

    private class DblClickMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                createRecordForm();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {
        }
    }
}

