/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects;

import java.io.File;
import java.util.Enumeration;
import java.util.Vector;

import rjw.dc.console.GeneralDialog;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Converter;
import rjw.dc.util.Resources;

/**
 * Holder of all the drives on the system. An array of drives,
 * A: to Z: is tested if they exist on the system and passed on
 * another array which holds all available drives.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class CdRom {

    private static final String[] locations =
        {
            "C:\\", "D:\\", "E:\\", "F:\\", "G:\\", "H:\\", "I:\\", "J:\\",
            "K:\\", "L:\\", "M:\\", "N:\\", "O:\\", "P:\\", "Q:\\", "R:\\",
            "S:\\", "T:\\", "U:\\", "V:\\", "W:\\", "X:\\", "Y:\\", "Z:\\" };

    GeneralDialog dialog;
    private Vector vDrives = new Vector();
    private static String[] sDrives = null;

    /**
     * Returns all the active and available drive letters
     *
     * @return (String[])   active and available drive letters
     */
    public String[] getDriveLetters() {
        if (sDrives != null) {
            return sDrives;
        } else {
            sDrives = new String[24];
            for (int i = 0; i < locations.length; i++) {
                File test = new File(locations[i]);

                if (test.exists() && test.isDirectory()) {
                    vDrives.add(locations[i]);
                }
            }
        }

        int j = 0;
        sDrives = new String[vDrives.size()];
        for (Enumeration enum = vDrives.elements();
            enum.hasMoreElements();
            j++) {
            sDrives[j] = enum.nextElement().toString();
        }

        return sDrives;
    }

    /**
     * Retrieves all files and directories from the given locations
     *
     * @param sDrive starting location for reading data
     * @param bConfirmation ask for confirmation for eah directory
     * @param bLeveledSearch read subdirectories
     * @param bReturnDirectories allow directory names in the result
     * @param sExtension the file extension name (filter)
     */
    public Vector getInformation(String sDrive,
                                 boolean bConfirmation,
                                 boolean bLeveledSearch,
                                 boolean bReturnDirectories,
                                 String sExtension) throws Exception {

        Vector result = new Vector();
        File directory = new File(sDrive + "/");
        Object oTrue = new Boolean(true);

        if (!directory.exists()) {
            return null;
        }

        boolean bCriterium = false;
        if (sExtension != null) {
            if (!sExtension.trim().equals(new String(""))) {
                bCriterium = true;
            }
        }

        Vector vUnhandeld = Converter.stringArrayToVector(directory.list());
        File tempFile = null;

        while (vUnhandeld.size() > 0) {
            String sCurrent = sDrive + "/" + vUnhandeld.firstElement().toString();
            tempFile = new File(sCurrent);
            if (tempFile.isDirectory()) {
                if (bConfirmation) {
                    dialog = new GeneralDialog(
                                Resources.getText(DCMessages.msgImport, sCurrent));
                    while (dialog.result == null) {}
                }
                
                if ((bConfirmation && dialog.result.equals(oTrue)) ||
                    !bConfirmation) {
                        
                    dialog.result = null;
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgReadingFrom, sCurrent), 
                        Messages._STANDARD);

                    String[] list = tempFile.list();
                    if (list != null) {
                        for (int j = 0; j < list.length; j++) {
                            String sTempCurrent = sCurrent + "/" + list[j];
                            if (!vUnhandeld.contains(sTempCurrent)) {
                                tempFile = new File(sTempCurrent);
                                if (tempFile.isDirectory()) {
                                    vUnhandeld.addElement(sTempCurrent.substring(sDrive.length() + 1, 
                                                                                 sTempCurrent.length()));
                                } else {
                                    if (bCriterium && criteriumApplied(sTempCurrent, sExtension)) {
                                        result.addElement(sTempCurrent);
                                    } else if (!bCriterium) {
                                        result.addElement(sTempCurrent);
                                    }
                                }
                            }
                        }
                    }
                }  else {
                    dialog.result = null;
                    
                    if (bReturnDirectories) {
                        Messages.setMessage(
                            Resources.getText(DCMessages.msgRegDirName, sCurrent), 
                            Messages._STANDARD);
                        result.addElement(sCurrent);
                    }
                }
            } else {
                if (bCriterium && criteriumApplied(sCurrent, sExtension)) {
                    result.addElement(sCurrent);
                } else if (!bCriterium) {
                    result.addElement(sCurrent);
                }
            }

            vUnhandeld.removeElementAt(0);

            if (!bLeveledSearch) {
                return result;
            }
        }
        
        return result;
    }

    private static boolean criteriumApplied(String sValue, String sSearchString) {
        return sValue.toLowerCase().endsWith(sSearchString.toLowerCase());
    }
}
