package rjw.dc.core.objects;

import java.util.Enumeration;
import java.util.Hashtable;

import rjw.dc.console.MessageBox;
import rjw.dc.core.Repository;
import rjw.dc.db.Query;
import rjw.dc.db.QueryBatch;
import rjw.dc.db.QueryWhat;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.util.Utilities;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

public class DataCrowCollection {
    
    private Hashtable htObjects;
    private static WorkFlow wf;
    
    public DataCrowCollection() {
        htObjects = new Hashtable();
        if (wf == null) {
            wf = new WorkFlow();
        }
    }
    
    public void addObject(DataCrowObject o, Integer rowNumber) {
        if (o.getValue(o._ID) == null || 
           (o.getValue(o._ID).toString().equals("") ||
            o.getValue(o._ID).toString().equalsIgnoreCase("null"))) {

            o.setValue(o._ID, new Utilities().getUniqueID());
        }
        
        htObjects.put(rowNumber, o);
    }    
    
    public void saveUpdate(RequestorCollection requestors) {
        wf.updateCollection(this, requestors);

    }

    public void saveNew(RequestorCollection requestors) {
        wf.insertCollection(this, requestors);
    }
    
    public QueryBatch toQueryBatch(int queryType) {
        QueryBatch batch = new QueryBatch();
        for (Enumeration enum = htObjects.keys(); enum.hasMoreElements(); ) {
            Integer row = (Integer) enum.nextElement();
            DataCrowObject o = (DataCrowObject) htObjects.get(row);
            
            boolean bValid = false;
            if (queryType == Query._INSERT) {
                bValid = o.checkIntegrity(false);
            } else {
                bValid = o.checkIntegrity(true);
            }
            
            if (bValid && uniqueObject(o, row)) {
                QueryWhat what = new QueryWhat(o.getFieldNames(),
                                               o.getTableName());
                Query query = new Query(what, 
                                        o.toQueryCondition(false),
                                        null,
                                        queryType,
                                        new RequestorCollection(),
                                        o.getValue(o._ID).toString(),
                                        row.intValue());
                try {
                    batch.addQuery(query);
                } catch (Exception exp) {
                    // Unwanted query type for queue; ignore ! 
                    // Only used during programming.
                }            
            } else {
                // one of the records in the batch is unvalid
                return null;
            }
        }
        return batch;
    }
    
    
    public boolean uniqueObject(DataCrowObject o, Integer newRow) {
        if (Settings.getValueAsBoolean(Repository.settings.stCheckUniqueness)) {
            int[] requiredFields = o.getRequiredFields();
            Integer oldRow = null;
            
            for (Enumeration enum = htObjects.keys(); enum.hasMoreElements(); ) {
                oldRow = (Integer) enum.nextElement();
                DataCrowObject dcTemp = (DataCrowObject) htObjects.get(oldRow);
                
                if (!dcTemp.equals(o)) {
                    
                    boolean same = true;
                    for (int k = 0; k < requiredFields.length; k++) {
                        if (!dcTemp.getValue(requiredFields[k]).equals(
                            o.getValue(requiredFields[k]))) {
                            same = false;  
                        }
                    }

                    if (same) {
                        new MessageBox(Resources.getText(DCMessages.msgRowIsSameAsRow, 
                                       new String[] {newRow.toString(), oldRow.toString()}), 
                                       true, MessageBox._ERROR); 
                        return false;
                    }
                }
            }
        }          
        return true;
    }    
}
