/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.software;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.db.QueryOptions;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Representation of a Software Catagory.
 * Allows saving, updating and querying of Catagory records
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Catagory extends DataCrowObject {

    /** Catagory name */
    public static final int _A_NAME = 1;
    /** Keep this record Y/N */
    public static final int _B_DISPOSE = 2;

    private static WorkFlow wf;
    
    /**
     * Initializes the Catagory object; sets the required fields, the table name,
     * initializes the value array and the database field names
     */
    public Catagory() {
        if (wf == null) {
            wf = new WorkFlow();
            CatagoryCollection catagories = new CatagoryCollection();
            catagories.load();
        }

        fieldNames = Repository.dataBase.dbCatColArrayPK;
        values = new String[Repository.dataBase.dbCatColArrayPK.length];
        sTableName = Repository.dataBase.dBCatTableName.trim();
        requiredFields = Settings.getValueAsIntegerArray(
                                  Repository.settings.stSoftwareCatRequiredFields);
        maxFieldLength = Repository.dataBase.softwareCatFieldLengths;
        valueTypes = Repository.dataBase.softwareCatValueTypes;
        labels = new String[Repository.dataBase.dbCatColArrayPK.length];
        int i = 0;
        labels[i++] = Resources.getText(DCLabels.lblID);
        labels[i++] = Resources.getText(DCLabels.lblName);
        labels[i++] = Resources.getText(DCLabels.lblDispose); 

        if (values != null) {
            for (int j = 0; j < values.length; j++) {
               setValue(j, new String(""));
            }
        }
    }

    /**
     * Applies values to this Catagory record. The first in the row is the ID
     *
     * @param row   representation of an database row. First in row is the ID
     */
    public void setValues(Object[] row) {
        int i = 0;
        try {
            if (row.length < Repository.dataBase.dbCatColArray.length) {
                throw new Exception(Resources.getText(DCMessages.msgInvalidArray));
            } else if (row.length == Repository.dataBase.dbCatColArrayPK.length) {
                setValue(_ID, row[i++]);
            }
            setValue(_A_NAME, row[i++]);
            setValue(_B_DISPOSE, row[i++]);

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Saves a new record to the database. Values are first validated for validity and
     * integrity
     */
    public void saveNew(RequestorCollection requestors) {
        if (this.checkIntegrity(false)) {
            wf.insertValues(this, requestors);
        }
    }

    /**
     * Updates an existing record. Values are first validated for validity and
     * integrity
     */
    public void saveUpdate(RequestorCollection requestors) {
        if (this.checkIntegrity(true)) {
            wf.updateValues(this, requestors);
        }
    }

    /**
     * Loads the values from the database. UI requests are past to the workflow
     *
     * @param options       search options
     * @param requestors    displaying request for the UI
     */
    public void load(QueryOptions options, RequestorCollection requestors) {
        wf.searchValues(this, options, requestors);
    }
}
