/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.db;

import java.util.Vector;

/**
 * This is the where part of the Query, it contains column names and theit
 * values
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class QueryCondition {

    private Vector vColumns = new Vector();
    private Vector vValues = new Vector();


    /**
     * Creates a new Query conditions. Values must represent the columns!
     * In other words; values[1] is the value for columns[1]
     *
     * @param columns   array of column names
     * @param values    array of values corresponding with the columns
     */
    public QueryCondition(String[] columns, Object[] values, boolean select) {
        if (values != null) {
            for (int i = 0; i < values.length; i++) {
                if (values[i] instanceof String) {
                    String sValue = values[i].toString();
                    if (sValue != null &&
                        !sValue.trim().equals("")) {
    
                        vValues.add("'" + sValue + "'");
                        vColumns.add(columns[i]);
                    }
                } else if (values[i] instanceof Integer) {
                    int integer = ((Integer) values[i]).intValue();
                    if (integer != -1) {
                        vValues.add(String.valueOf(integer));
                        vColumns.add(columns[i]);   
                    } else {
                        if (!select) {
                            vValues.add("0");
                            vColumns.add(columns[i]);
                        }
                    }
                }
            }
        } else if (columns != null) {
            for (int i = 0; i < columns.length; i++) {
                vColumns.add(columns[i].trim());
            }
        }
    }

    /**
     * Returns the columns
     *
     * @return (Vector) the columns
     */
    public Vector getColumns() {
        return vColumns;
    }

    /**
     * Returns the values
     *
     * @return (Vector) the values, corresponding with the columns
     */
    public Vector getValues() {
        return vValues;
    }
}
