package rjw.dc.http;

import http.client.HTTPConnection;
import http.client.HTTPResponse;
import http.client.NVPair;
import java.util.Hashtable;

import rjw.dc.console.MessageBox;
import rjw.dc.core.Repository;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * Creates a connection to a webserver using http
 *
 * @author Robert-Jan van der Waals
 * @since 1.2
 * @version 3.03
 */
public class HttpConnection {

    private String HOST = "";
    private HTTPConnection connection;

    private String HTTP_PROXY = "";
    private HTTPResponse oResponse;
    private NVPair[] oForm;
    private Hashtable oForm_ = new Hashtable();
    
    public HttpConnection(String sHost) throws Exception {
        createConnection(sHost);
    }

    public void close() {
        connection.stop();
    }

    public void createConnection(String sHost) throws Exception {
        String sProxy = Settings.getValueAsString(Repository.settings.stProxyServerName);
        int iProxyPort = Settings.getValueAsInt(Repository.settings.stProxyServerPort);
        String sUserAgent = Settings.getValueAsString(Repository.settings.stHTTPUserAgent);        

        HOST = (sHost.startsWith("http://") ? "" : "http://") + sHost;
        try {
            HTTPConnection.setProxyServer(sProxy, iProxyPort);
            connection = new HTTPConnection(sHost);
            connection.setDefaultHeaders( new NVPair[] {
                                             new NVPair(
                                               "User-Agent",
                                                sUserAgent)});
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }        
    }

    public void addQuery (String sName, String sValue) {
        oForm_.put(sName, sValue);
    }

    public void clearQueryBuffer () {
        oForm_ = new Hashtable();
    }

    public String post(String sFile) throws Exception {
        return new String(get(sFile, true, 0));
    }

    public String get(String sFile) throws Exception {
        return new String(get(sFile, false, 0));
    }

    private byte[] get(String sFile, boolean bPost, int i) throws Exception {
        try {
            if (bPost) {
                oResponse = connection.Post(sFile, oForm);
            } else {
                oResponse = connection.Get(sFile);
            }

            if (oResponse.getStatusCode() >= 300) {
                if (i < 5) {
                    String message = Resources.getText(DCMessages.msgHttpRetry, 
                                     new String[] {HOST + sFile, String.valueOf(i + 1)});
                    new MessageBox(message, true, MessageBox._ERROR); 
                    Messages.setMessage("<HTTP> " + message, Messages._ERROR);

                    try {
                        Thread.sleep(3000);
                    } catch (InterruptedException e2) {}

                    return get(sFile, bPost, ++i);

                } else {
                    String message = Resources.getText(DCMessages.msgCouldNotAccess, HOST + sFile);
                    throw new Exception(message);
                }
            }

            byte[] baRetVal = oResponse.getData();
            return baRetVal;
        } catch (Exception e) {
            new MessageBox(e.getMessage(), true, MessageBox._ERROR);         
            Messages.setMessage("<HTTP> " + e.getMessage(), Messages._ERROR);      

            if (i < 5) {
                new MessageBox(Resources.getText(DCMessages.msgHttpRetry, 
                               new String[] {HOST + sFile, String.valueOf(i + 1)}), 
                               true, MessageBox._ERROR);
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e2) {}

                return get(sFile, bPost, ++i);

            } else {
                throw new Exception(Resources.getText(DCMessages.msgCouldNotAccess, 
                                                      HOST + sFile));                
            }
        }
    }
}
