package rjw.dc.http;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import rjw.dc.core.Repository;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * Access www.imdb.com to gather movie specific information
 * 
 * @author Robert-Jan van der Waals
 * @since 1.2
 * @version 3.03
 */
public class IMDBQuery {

    private Hashtable htData = new Hashtable();

    public static final Object _ID = "Id";
    public static final Object _URL = "Url";
    public static final Object _TITLE = "Title";
    public static final Object _YEAR = "Year";
    public static final Object _DIRECTOR = "Director";
    public static final Object _WRITER = "Writer";
    public static final Object _GENRE = "Genre";
    public static final Object _TAGLINE = "Tagline";
    public static final Object _TAGLINEURL = "TaglineUrl";
    public static final Object _PLOT = "Plot";
    public static final Object _PLOTURL = "PlotUrl";
    public static final Object _RATING = "Rating";
    public static final Object _VOTES = "Votes";
    public static final Object _PLACE = "Place";
    public static final Object _ACTORS = "Actors";
    public static final Object _COUNTRY = "Country";
    public static final Object _RUNTIME = "Runtime";

    private String searchString;
    private String searchUrl;

    /** 
     * initializes the IMDB query
     * 
     * @param searchString the string on which will be searched
     */
    public IMDBQuery(String searchString) throws Exception {
        this.searchString = searchString.replace(' ', '+');
    }

    /**
     * Searches for the maching ID's from the search result(s)
     */
    public String[] getIDs(HttpConnection connection) throws Exception {
        Vector vKeys = new Vector(0);
        
        this.searchUrl = "/Find?" + searchString;
        String sWebPage = connection.get(searchUrl);

        // if there are several possibilities the title of the the imdb web site
        // must be this, so look up the position of this tag
        int Temp = sWebPage.indexOf("<H1><BR>IMDb name and title search</H1>");

        String sTemp = "";
        if (Temp != -1) {
            sTemp = sWebPage.substring(Temp);
            sTemp = sTemp.substring(0, sTemp.indexOf("</TABLE>"));

            while (Temp != -1) {
                Temp = sTemp.indexOf("/Title?");
                if (Temp != -1) {
                    sTemp = sTemp.substring(Temp + 7, sTemp.length());
                    vKeys.add(sTemp.substring(0, 7));
                }
            }
        }
        
        if (vKeys.size() < 1) {
            throw new Exception(Resources.getText(DCMessages.msgNoSearchResults));
        } else {
            String[] keys = new String[vKeys.size()];
            int i = 0;
            for (Enumeration enum = vKeys.elements(); enum.hasMoreElements(); i++) 
            keys[i] = enum.nextElement().toString();
            return keys;
        }
    }

    /**
     * Reads the information from www.imdb.com
     * 
     * @param sID   the movie ID from www.imdb.com
     */
    public void fillImdbData(String sID, HttpConnection connection) throws Exception {
        // set some things/vars we already know
        htData.put(_URL, "http://www.imdb.com/Title?" + sID);
        htData.put(_ID, sID);
        htData.put(_TAGLINEURL, "http://www.imdb.com/Taglines?" + sID);
        htData.put(_PLOTURL, "http://www.imdb.com/Plot?" + sID);

        // temp vars used while gathering the info from imdb
        int Temp, Temp2;
        String sTemp, sTemp2 = null;

        try {
            if (connection != null) {
            // start connection to web server
                String sODB = Settings.getValueAsString(Repository.settings.stODB);
                connection = new HttpConnection(sODB);
            }
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }
        String sWebPage = "";
        try {
            // save the imdb movie webpage in sWebPage 
            sWebPage = connection.get("/Title?" + sID);
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }

        // gather the information from sWebPage
        // first we get the title of the movie and the year from the titel tag
        // from the web page
        sTemp =
            sWebPage.substring(
                sWebPage.indexOf("<title>"),
                sWebPage.indexOf("</title>"));
        htData.put(_TITLE, trim(sTemp.substring(7, sTemp.indexOf("(") - 1).trim()));
        htData.put(_YEAR, sTemp.substring(sTemp.indexOf("(") + 1, 
                                          sTemp.indexOf(")")));

        Temp = sWebPage.indexOf("Directed by");
        if (Temp != -1) {
            sWebPage = sWebPage.substring(Temp);
            Temp = sWebPage.indexOf("Writing credits");
            if (Temp != -1) {
                sTemp2 =
                    sWebPage.substring(
                        Temp,
                        sWebPage.indexOf("<br clear=\"left\">"));
                sTemp = sWebPage.substring(0, Temp - 10);
            } else {
                sTemp =
                    sWebPage.substring(
                        0,
                        sWebPage.indexOf("<br clear=\"left\">"));
            }
            Temp = sTemp.indexOf("(more)");
            if (Temp != -1) {
                sTemp = sTemp.substring(0, Temp - 10);
            }

            Temp = sTemp.indexOf("\">");
            String director = "";
            while (Temp != -1) {
                sTemp = sTemp.substring(Temp + 2);
                Temp = sTemp.indexOf("\">");
                director += sTemp.substring(0, sTemp.indexOf("<")) + ",";
            }
            htData.put(_DIRECTOR, trim(director.substring(0, director.length() - 1)));
        }

        if (sTemp2 == null) {
            Temp = sWebPage.indexOf("Writing credits");
            if (Temp != -1) {
                sWebPage = sWebPage.substring(Temp);
                sTemp2 = sWebPage.substring(0, 
                         sWebPage.indexOf("<br clear=\"left\">"));
            }
        }
 
        if (sTemp2 != null) {
            Temp = sTemp2.indexOf("(more)");
            if (Temp != -1) {
                sTemp2 = sTemp2.substring(0, Temp - 10);
            }
            Temp = sTemp2.indexOf(">WGA<");
            if (Temp != -1) {
                sTemp2 = sTemp2.substring(Temp + 5);
            }

            Temp = sTemp2.indexOf("\">");
            String writer = "";
            while (Temp != -1) {
                sTemp2 = sTemp2.substring(Temp + 2);
                Temp = sTemp2.indexOf("\">");
                writer += sTemp2.substring(0, sTemp2.indexOf("<")) + ",";
            }
            htData.put(_WRITER, trim(writer.substring(0, writer.length() - 1)));
        }

        Temp = sWebPage.indexOf("Genre");
        if (Temp != -1) {
            sTemp = sWebPage.substring(Temp);
            sTemp = sTemp.substring(0, sTemp.indexOf("<br>"));
            Temp = sTemp.indexOf("/\">");
            String genre = "";
            while (Temp != -1) {
                sTemp = sTemp.substring(Temp + 3);
                genre += sTemp.substring(0, sTemp.indexOf("<")) + (",");
                Temp = sTemp.indexOf("/\">");
            }
            htData.put(_GENRE, trim(genre.substring(0, genre.length() - 1)));
        }
        Temp = sWebPage.indexOf("Tagline:");
        if (Temp != -1) {
            sTemp = sWebPage.substring(Temp);
            sTemp = sTemp.substring(13);
            htData.put(_TAGLINE, sTemp.substring(0, sTemp.indexOf("<")));
        } 
        
        Temp = sWebPage.indexOf("Plot Outline:");
        if (Temp == -1)
            Temp = sWebPage.indexOf("Plot Summary:");
        if (Temp != -1) {
            sTemp = sWebPage.substring(Temp);
            int end = sTemp.indexOf("<a") < sTemp.indexOf("<b")
                      ? sTemp.indexOf("<a")
                      : sTemp.indexOf("<b");
            String plot = trim(sTemp.substring(18, end));
            htData.put(_PLOT, plot);
        } 
        
        Temp = sWebPage.indexOf("User Rating:");
        if (Temp != -1) {
            sTemp = sWebPage.substring(Temp);
            sTemp = sTemp.substring(0, sTemp.indexOf("</p>"));
            Temp = sTemp.indexOf("<b>");
            if (Temp != -1) {
                sTemp = sTemp.substring(Temp);
                try {
                    htData.put(_RATING, sTemp.substring(3, 6));
                    htData.put(_VOTES, sTemp.substring(sTemp.indexOf("(") + 1,
                                       sTemp.indexOf(" votes")));
                } catch (Exception e) {}
            }
        }
        
        Temp = sTemp.indexOf("top 250: #");
        if (Temp != -1) {
            sTemp = sTemp.substring(Temp);
            htData.put(_PLACE, sTemp.substring(10, sTemp.indexOf("<")));

        }

        Temp = sWebPage.indexOf("ast overview");
        if (Temp == -1)
            Temp = sWebPage.indexOf("Complete credited cast:");
        if (Temp != -1) {
            sTemp = sWebPage.substring(Temp);
            Temp = sTemp.indexOf("href=\"");
            sTemp = sTemp.substring(Temp, sTemp.indexOf("</table>"));
            Temp = sTemp.indexOf("(more)");
            if (Temp != -1) {
                sTemp = sTemp.substring(0, Temp - 50);
            }

            Temp = sTemp.indexOf("\">");
            String actors = "";
            while (Temp != -1) {
                sTemp = sTemp.substring(Temp + 2);
                actors += sTemp.substring(0, sTemp.indexOf("<")) + (", ");
                Temp = sTemp.indexOf("\"top\">");
                sTemp = sTemp.substring(Temp + 6);
                actors += sTemp.substring(0, sTemp.indexOf("<") - 1) + (", ");
                Temp = sTemp.indexOf("href");
                sTemp = sTemp.substring(Temp + 4);
                Temp = sTemp.indexOf("\">");
            }
            htData.put(_ACTORS, trim(actors.substring(0, actors.length() - 1)));
        }

        Temp = sWebPage.indexOf("Runtime");
        String runtime = "";
        if (Temp != -1) {
            sTemp = sWebPage.substring(Temp + 13);
            if (sTemp.indexOf("<") < sTemp.indexOf(" ")) {
                runtime = sTemp.substring(0, sTemp.indexOf("<"));
            } else {
                Temp2 = sTemp.indexOf("\t");
                if (Temp2 != -1) {
                    if (sTemp.indexOf(" ") < Temp2)
                        runtime = sTemp.substring(0, sTemp.indexOf(" "));
                    else
                        runtime = sTemp.substring(0, sTemp.indexOf("\t"));
                } else {
                     runtime = sTemp.substring(0, sTemp.indexOf(" "));
                }
            }
            try {
                Temp = new Integer(runtime).intValue();
            } catch (NumberFormatException e) {
                runtime = "";
            }
        }
        htData.put(_RUNTIME, runtime);

        Temp = sWebPage.indexOf("Country:</b>");
        if (Temp != -1) {
            sTemp = sWebPage.substring(Temp + 12);
            sTemp = sTemp.substring(sTemp.indexOf(">") + 1);
            htData.put(_COUNTRY, sTemp.substring(0, sTemp.indexOf("<")));
        }
    }

    public String getValue(Object identifier) {
        Object o = htData.get(identifier);
        
        if (o == null) {
            return "";
        } else {
            return o.toString();   
        }
    }

    public String trim(String sRaw) {
        try {
            byte[] byties = sRaw.getBytes("UTF8");
            sRaw = new String(byties);
        } catch (Exception exp) {
        }
        
        String sRetVal = sRaw;
        if ((sRetVal.startsWith("\"")) && (sRetVal.endsWith("\"")))
            sRetVal = sRetVal.substring(1, sRetVal.length() - 1);
            sRetVal = sRetVal.replaceAll("\"", "'");
            sRetVal = sRetVal.replaceAll("&#201;", "");
            sRetVal = sRetVal.replaceAll("&#223;", "");
            sRetVal = sRetVal.replaceAll("&#225;", "");
            sRetVal = sRetVal.replaceAll("&#233;", "");
            sRetVal = sRetVal.replaceAll("&#235;", "");
            sRetVal = sRetVal.replaceAll("&#244;", "");
            sRetVal = sRetVal.replaceAll("&#246;", "");
            sRetVal = sRetVal.replaceAll("&#228;", "");
            sRetVal = sRetVal.replaceAll("&#251;", "");
            sRetVal = sRetVal.replaceAll("&#38;", "");
            sRetVal = sRetVal.replaceAll("&#34;", "");
            sRetVal = sRetVal.replaceAll("\n", "");
            return sRetVal;
    }
}
