package rjw.dc.processes;

import rjw.dc.console.MessageBox;
import rjw.dc.console.movie.ImdbSearchForm;
import rjw.dc.core.DataCrow;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;


/**
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ImdbUpdate extends Thread{

    private Object[][] data;
    private boolean update;

    /** Specifies whether the process is running */
    public static boolean running = false;

    private static final WorkFlow wf = new WorkFlow();

    /**
     * Initializes the process. All preferences are set
     *
     * @param data[][0] the ID of the row
     * @param data[][1] the name of the movie
     * @param update search values for new or existing data ?
     */
    public ImdbUpdate(Object[][] data, boolean update) {
        this.update = update;
        this.data = data;
    }

    /**
     * Starts the process
     */
    public void run() {
        try {
            running = true;
            searchIMDB(data, update);
        } catch (Exception exp) {
            exp.printStackTrace();
            Messages.setMessage(exp.toString(), Messages._ERROR);
            new MessageBox(DataCrow.mainFrame,
                           Resources.getText(DCMessages.msgProcessCrash), 
                           true, MessageBox._ERROR);
            finishAll();
        }
        finishAll();
    }

    /**
     * Opens a result form for every Imdb request so the user can select 
     * the most likely one
     */
    public void searchIMDB(Object[][] data, boolean update) {
        for (int i = 0; i < data.length; i++) {
            int id = ((Integer) data[i][0]).intValue();
            String name = data[i][1].toString();
            String title = Resources.getText(DCLabels.lblSearchingFor, 
                                             new String[] {name, String.valueOf(i+1),
                                                           String.valueOf(data.length)});
            ImdbSearchForm searchForm = new ImdbSearchForm(id, title, name, update);

            while (searchForm.isUpdateRunning()) {
                try {
                    this.sleep(3000);
                } catch (InterruptedException ignore) {}
            }
        }
        finishAll();
    }

    /**
     * Stops the process
     */
    private void finishAll() {
        try {
            running = false;
            Messages.setMessage(
                Resources.getText(DCMessages.msgIMDBProcessStops), 
                Messages._STANDARD);
        } catch (Exception ignore) {}
    }
}
