/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   * 
 *                           sub-project: JFlexSett                           *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.io.File;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JTextField;

import rjw.dc.console.components.BackgroundColorField;
import rjw.dc.console.components.NumberField;
import rjw.dc.console.components.RequiredFieldsTable;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.core.objects.mp3.Mp3;
import rjw.dc.core.objects.software.Catagory;
import rjw.dc.core.objects.software.Software;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;
import rjw.flex.settings.Setting;
import rjw.flex.settings.Settings;
import rjw.flex.settings.SettingsFile;
import rjw.flex.settings.SettingsGroup;

/**
 * Groups are created here, as well as the settings.
 * A similar class like this needs to be implemented in the
 * project which uses jFlexSett.
 * 
 * All settings and their values (and the settings groups) can
 * be found by referencing to the Settings class. 
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SettingInitializator {

    public static final String _General = 
                                Resources.getText(DCLabels.lblGroupGeneral);
    public static final String _RequiredSoftFields = 
                                Resources.getText(DCLabels.lblGroupSoftRequired);
    public static final String _RequiredMp3Fields = 
                                Resources.getText(DCLabels.lblGroupMp3Required);
    public static final String _RequiredMovieFields = 
                                Resources.getText(DCLabels.lblGroupMovieRequired);
    public static final String _BackGroundColor = 
                                Resources.getText(DCLabels.lblColor);
    public static final String _HTTP =
                                Resources.getText(DCLabels.lblHttp);
                                
    
    /**
     * Starts the initialization of the Settings and starts the UI
     */
    public SettingInitializator() {
        createGroups();
        createSettings();

        Settings.setSettingsFile(new File("DataCrow.properties"));
        SettingsFile.load();
    }

    /**
     * Creates the settings group (with their hierarchy)
     */
    public void createGroups() {
        SettingsGroup generalGroup = new SettingsGroup(_General);
        SettingsGroup backGround = new SettingsGroup(_BackGroundColor);
        SettingsGroup http = new SettingsGroup(_HTTP);        
        
        generalGroup.addChild(new SettingsGroup(_RequiredSoftFields));
        generalGroup.addChild(new SettingsGroup(_RequiredMp3Fields));
        generalGroup.addChild(new SettingsGroup(_RequiredMovieFields));
        
        Settings.addGroup(_General, generalGroup);
        Settings.addGroup(_BackGroundColor, backGround);
        Settings.addGroup(_HTTP, http);
    }
    
    /**
     * Creates the settings and adds them to the specified group.
     * If a self-created component needs to be added, make sure this
     * components implements the ISettingsComponent class! 
     * Otherwise the component is not supported.. The getValue and
     * setValue must correspond to the DataType of the setting.
     */
    public void createSettings() {
        Settings.addSetting(_BackGroundColor,                           // Group key
                    new Setting(Repository.datatypes._COLOR,            // Data type
                                Repository.settings.stColor,            // Key
                                new Color(204,204,204),                 // Default value
                                new BackgroundColorField(),                    // UI Component
                                "",                                     // Help text
                                Resources.getText(DCLabels.lblColor),   // Label text
                                false,                                  // show label ?
                                true));                                 // Add to panel ?
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stCheckUniqueness,
                                new Boolean(true),
                                new JCheckBox(Resources.getText(DCLabels.lblUniqueness)),
                                Resources.getText(DCTooltips.tpUniqueness),
                                Resources.getText(DCLabels.lblUniqueness),
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stCheckRequiredFields,
                                new Boolean(true),
                                new JCheckBox(Resources.getText(DCLabels.lblRequired)),
                                Resources.getText(DCTooltips.tpRequiredFields),
                                Resources.getText(DCLabels.lblRequired),
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stConnectionString,
                                "dc",
                                new JTextField(),
                                Resources.getText(DCTooltips.tpConnectionString),
                                Resources.getText(DCLabels.lblConnectionString),
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stDatabaseDriver,
                                "org.hsqldb.jdbcDriver",
                                new JTextField(),
                                Resources.getText(DCTooltips.tpDatabaseDriver),
                                Resources.getText(DCLabels.lblDatabaseDriver),
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stLoadDemoData,
                                new Boolean(false),
                                new JCheckBox(Resources.getText(DCLabels.lblLoadDemoData)),
                                Resources.getText(DCTooltips.tpDemoData),
                                Resources.getText(DCLabels.lblLoadDemoData),
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(  Repository.datatypes._INTEGER,
                                Repository.settings.stMaximumSearchRes,
                                new Integer(10000),
                                new NumberField(),
                                Resources.getText(DCTooltips.tpSearchRes),
                                Resources.getText(DCLabels.lblSearchResults),
                                true,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stMP3ExportFile,
                                "",
                                new JTextField(),
                                "",
                                "",
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stMP3ImportFile,
                                "",
                                new JTextField(),
                                "",
                                "",
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stSoftExportFile,
                                "",
                                new JTextField(),
                                "",
                                "",
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stSoftImportFile,
                                "",
                                new JTextField(),
                                "",
                                "",
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stMovieExportFile,
                                "",
                                new JTextField(),
                                "",
                                "",
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stMovieImportFile,
                                "",
                                new JTextField(),
                                "",
                                "",
                                true,
                                false));                                
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGER,
                                Repository.settings.stModule,
                                new Integer(0),
                                new JTextField(),
                                "",
                                "",
                                true,
                                false));
         Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stODB,
                                "www.imdb.com",
                                new JTextField(),
                                Resources.getText(DCTooltips.tpODB),
                                Resources.getText(DCLabels.lblODB),
                                true,
                                false));
    /*     Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stODBUserName,
                                "",
                                new JTextField(),
                                Resources.getText(DCTooltips.tpODBUserName),
                                Resources.getText(DCLabels.lblODBUserName),
                                true,
                                true));
        Settings.addSetting(_HTTP,
                    new Setting(  Repository.datatypes._STRING,
                                Repository.settings.stODBPassword,
                                "",
                                new JPasswordField(),
                                Resources.getText(DCTooltips.tpODBPassword),
                                Resources.getText(DCLabels.lblODBPassword),
                                true,
                                true));  */
        Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stHTTPUserAgent,
                                "Mozilla/4.04 (Windows; I; Win95)",
                                new JTextField(),
                                Resources.getText(DCTooltips.tpHTTPUserAgent),
                                Resources.getText(DCLabels.lblHTTPUserAgent),
                                true,
                                true));
        Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stProxyServerName,
                                "",
                                new JTextField(),
                                Resources.getText(DCTooltips.tpProxyServerName),
                                Resources.getText(DCLabels.lblProxyServerName),
                                true,
                                true));
        Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._INTEGER,
                                Repository.settings.stProxyServerPort,
                                new Integer(0),
                                new NumberField(),
                                Resources.getText(DCTooltips.tpProxyServerPort),
                                Resources.getText(DCLabels.lblProxyServerPort),
                                true,
                                true));
        Settings.addSetting(_RequiredMp3Fields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stMp3RequiredFields,
                                new int[] {Mp3._A_FILENAME,
                                           Mp3._I_LOCATION},
                                new RequiredFieldsTable(1),
                                "",
                                "",
                                false,
                                true));
        Settings.addSetting(_RequiredMovieFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stMovieRequiredFields,
                                new int[] {Movie._A_TITLE,
                                           Movie._K_LOCATION},
                                new RequiredFieldsTable(2),
                                "",
                                "",
                                false,
                                true));
        Settings.addSetting(_RequiredSoftFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stSoftwareRequiredFields,
                                new int[] {Software._C_FILENAME,
                                           Software._A_NAME,
                                           Software._F_LOCATION},
                                new RequiredFieldsTable(0),
                                "",
                                "",
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stSoftwareCatRequiredFields,
                                new int[] {Catagory._A_NAME},
                                new JComboBox(),
                                "",
                                "",
                                true,
                                false));                                
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stRecordFormViewSize,
                                new Dimension(300, 400),
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMP3OrganizerViewSize,
                                new Dimension(300, 400),
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMainViewSize,
                                new Dimension(800, 800),
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stSettingsViewSize,
                                new Dimension(800, 450),
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
         Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stImdbSearchFormViewSize,
                                new Dimension(400, 400),
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stIMDBOverwriteExistingValues,
                                new Boolean(false),
                                new JCheckBox(),
                                "",
                                "",
                                false,
                                false));                                
    }
}
