package rjw.dc.util;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import rjw.dc.console.MessageBox;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.software.CatagoryCollection;
import rjw.dc.core.objects.software.Software;
import rjw.dc.db.Database;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.settings.SettingInitializator;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

public class DBUpgrade extends JDialog {

    private final ComponentFactory cf;
    private final WorkFlow wf;
    
    public static void main(String[] args) {
        try {
            DBUpgrade dbup = new DBUpgrade();
            dbup.createUI();
        } catch (Exception exp) {
            System.exit(0);   
        }
    }
    
    
    public DBUpgrade() {

        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
        });

        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCTooltips.class.getName()));
        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCLabels.class.getName()));
        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCMessages.class.getName()));
        new Messages();
        new SettingInitializator();
        cf = new ComponentFactory();
        wf = new WorkFlow();
        
    }
    
    public void startUpgrade() {
        Connection connFrom = getConnection(textOldDbDriver.getText(),
                                            textOldDbString.getText(),
                                            "");
                         
        try {                 
            Statement stmt = connFrom.createStatement();
            
            // hack; just insert the catagory id as is
            Software.setConvertCatagory(false);
            
            Vector v = new Vector(0);
            ResultSet resSoftCat = stmt.executeQuery("select * from Softwaretype");
            v.add(WorkFlow.createDataCrowObjectArray(resSoftCat));
            resSoftCat.close();
            ResultSet resSoft = stmt.executeQuery("select * from Software");
            v.add(WorkFlow.createDataCrowObjectArray(resSoft));
            resSoft.close();
            ResultSet resMp3 = stmt.executeQuery("select * from Mp3");
            v.add(WorkFlow.createDataCrowObjectArray(resMp3));
            resMp3.close();
            ResultSet resMovie = stmt.executeQuery("select * from Movie");
            v.add(WorkFlow.createDataCrowObjectArray(resMovie));
            resMovie.close();

            for (Enumeration enum1 = v.elements(); enum1.hasMoreElements(); ) {
                Vector vObjects = (Vector) enum1.nextElement();
                DataCrowCollection collection = new DataCrowCollection();  
                int i = 0;
                for (Enumeration enum2 = vObjects.elements(); enum2.hasMoreElements(); ) {                
                    DataCrowObject dcObj = (DataCrowObject) enum2.nextElement();
                    collection.addObject(dcObj, new Integer(i++));
                }
                collection.saveNew(new RequestorCollection());
                CatagoryCollection.load();
            }
        } catch (Exception exp) {
            Database.close();
            new MessageBox(exp.getMessage(), true, MessageBox._ERROR);   
        }
        Database.close();
    }
    
    private Connection getConnection(String driver, 
                                     String connectionString,
                                     String username) {
                                        
        Connection con = null;
        String sDriver = driver;
        String sConnectionString = connectionString;

        try {
            Class.forName(sDriver);
        } catch (ClassNotFoundException exp) {
            new MessageBox(exp.getMessage(), true, MessageBox._ERROR);
        }

        try {
            con = DriverManager.getConnection(sConnectionString, username, "");
        } catch (Exception exp) {
            new MessageBox(exp.getMessage(), true, MessageBox._ERROR);
        }      
        
        return con;  
    }
    
    
    JTextField textOldDbDriver;
    JTextField textOldDbString;
    JTextField textNewDbDriver;
    JTextField textNewDbName;
    
    public void createUI() {
        this.setBackground(ComponentFactory.getDisabledColor());
        JPanel panel = new JPanel(false);
        panel.setBackground(ComponentFactory.getDisabledColor());
        JPanel panelActions = new JPanel(false);
        panelActions.setBackground(ComponentFactory.getDisabledColor());
        GridBagLayout gb = new GridBagLayout();

        panel.setLayout(gb);

        // Initialize Components
        JLabel labelOldDbDriver = ComponentFactory.getLabel(Resources.getText(DCLabels.lblOldDatabaseDriver));
        textOldDbDriver = ComponentFactory.getTextField();
        JLabel labelOldDbString = ComponentFactory.getLabel(Resources.getText(DCLabels.lblOldDatabaseName));
        textOldDbString = ComponentFactory.getTextField();
        JLabel labelNewDbDriver = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDatabaseDriver));
        textNewDbName = ComponentFactory.getTextField();
        JLabel labelNewDbString = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDatabaseName));
        textNewDbDriver = ComponentFactory.getTextField();

        textOldDbDriver.setText("sun.jdbc.odbc.JdbcOdbcDriver");
        textOldDbString.setText("jdbc:odbc:libcd");
        textNewDbDriver.setText(Settings.getValueAsString(Repository.settings.stDatabaseDriver));
        textNewDbName.setText(Settings.getValueAsString(Repository.settings.stConnectionString));

        // Build panel
        panel.add(labelOldDbDriver, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(textOldDbDriver,  new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(labelOldDbString, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(textOldDbString,  new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(labelNewDbDriver, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(textNewDbDriver,  new GridBagConstraints(1, 2, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(labelNewDbString, new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(textNewDbName,  new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));

        // ************************************************************************
        // ACTION PANEL
        // Set layout style
        GridBagLayout gridbActions = new GridBagLayout();
        panelActions.setLayout(gridbActions);

        // Initialize components
        JButton buttonStart = ComponentFactory.getButton(Resources.getText(DCLabels.lblStart));
        JButton buttonCancel = ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));

        buttonStart.addActionListener(new StartButtonAction());
        buttonCancel.addActionListener(new CancelButtonAction());

        // Build panel
        panelActions.add(buttonStart,   new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets(20, 10, 10, 0), 0, 0));
        panelActions.add(buttonCancel,  new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets(20, 5, 10, 10), 0, 0));

        this.setResizable(true);
        this.getContentPane().setLayout(new GridBagLayout());

        this.getContentPane().add(panel,        new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                                GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                new Insets(10, 10, 0, 0), 0, 0));
        this.getContentPane().add(panelActions, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                                                GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                new Insets(0, 5, 0, 0), 0, 0));

        this.pack();
        this.setTitle(Resources.getText(DCMessages.msgQuestion));
        this.setVisible(true);
    }


    private class StartButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (!textOldDbDriver.getText().trim().equals("") &&
                !textOldDbString.getText().trim().equals("") &&
                !textNewDbDriver.getText().trim().equals("") &&
                !textNewDbName.getText().trim().equals("")) {
                 
                 Settings.setStringAsValue(Repository.settings.stDatabaseDriver,
                                           textNewDbDriver.getText());
                 Settings.setStringAsValue(Repository.settings.stConnectionString,
                                           textNewDbName.getText());
                                           
                 if (getConnection(textNewDbDriver.getText(),
                                   textNewDbName.getText(),
                                   "sa") != null) {
                     wf.initDatabase();                 
                     startUpgrade();                        
                 }                                     
            } else {
                new MessageBox(Resources.getText(DCMessages.msgDatabasesNotFullyDefined), 
                               true, MessageBox._INFORMATION);    
            }
        }
    } 
    
    private class CancelButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }     
}
