/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   * 
 *                           sub-project: JFlexSett                           *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.flex.settings;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * A settings group contains specific settings. A group can have
 * on child, creating a two level settings hierarchy. Deeper 
 * hierarchies are not supported as yet.
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SettingsGroup {
    
    private SettingsGroup children;
    private String sKey = "";
    private Hashtable htSettings = new Hashtable();
    private Hashtable htChildren = new Hashtable();
    
    /**
     * Creates a new settings group
     * 
     * @param sKey a unique identifier of this group
     */
    public SettingsGroup(String sKey) {
        this.sKey = sKey;
    }

    /**
     * Adds a child to the settings group
     */
    public void addChild(SettingsGroup child) {
        htChildren.put(child.getKey(), child);
    }
    
    public String getKey() {
        return sKey;   
    }
    
    public Hashtable getChildren() {
        return htChildren;
    }
    
    public SettingsGroup getChild(String sGroupKey) {
        return (SettingsGroup) htChildren.get(sGroupKey);
    }
    
    public Hashtable getSettings() {
        return htSettings;
    }
    
    protected Setting getSetting(String sKey) {
        Setting setting = (Setting) htSettings.get(sKey);
        
        if (setting != null) {
            return setting; 
        } 
        
        for (Enumeration enum = htChildren.elements(); enum.hasMoreElements(); ) {
            SettingsGroup settingsGroup = (SettingsGroup) enum.nextElement();
            if (settingsGroup != null) {
                setting = settingsGroup.getSetting(sKey);
                if (setting != null) {
                    break;   
                }
            }
        }
        return setting;
    }
    
    protected void add(Setting setting) {
        htSettings.put(setting.getKey(), setting);
    }
    
    public String toString() {
        return sKey; 
    }
}
